@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -82,11 +85,14 @@ import org.netbeans.api.autoupdate.InstallSupport;
 import org.netbeans.api.autoupdate.InstallSupport.Installer;
 import org.netbeans.api.autoupdate.OperationSupport.Restarter;
 import org.netbeans.api.autoupdate.InstallSupport.Validator;
+import org.netbeans.api.autoupdate.OperationContainer;
 import org.netbeans.api.autoupdate.OperationContainer.OperationInfo;
 import org.netbeans.api.autoupdate.OperationException;
 import org.netbeans.api.autoupdate.UpdateElement;
 import org.netbeans.api.autoupdate.UpdateUnit;
 import org.netbeans.api.progress.ProgressHandle;
+import org.netbeans.modules.autoupdate.updateprovider.NetworkAccess;
+import org.netbeans.modules.autoupdate.updateprovider.NetworkAccess.Task;
 import org.netbeans.updater.ModuleDeactivator;
 import org.netbeans.updater.ModuleUpdater;
 import org.openide.LifecycleManager;
@@ -108,7 +114,7 @@ public class InstallSupportImpl {
     private static final String AUTOUPDATE_SERVICES_MODULE = "org.netbeans.modules.autoupdate.services"; // NOI18N
     
     private Map<UpdateElementImpl, File> element2Clusters = null;
-    private Set<File> downloadedFiles = null;
+    private final Set<File> downloadedFiles = new HashSet<File> ();
     private boolean isGlobal;
     private int wasDownloaded = 0;
     
@@ -148,6 +154,31 @@ public class InstallSupportImpl {
                 }
 
                 infos = support.getContainer ().listAll ();
+                List <OperationInfo<?>> newInfos = new ArrayList <OperationInfo<?>>();
+                for(OperationInfo <?> i : infos) {
+                    if(i.getUpdateUnit().getInstalled()!=null &&
+                            i.getUpdateUnit().getInstalled().equals(i.getUpdateElement())   ) {
+                        //internal update, replace by required elements
+                        for(UpdateElement e : i.getRequiredElements()) {
+                            boolean add = true;
+                            for(OperationInfo <?> in : newInfos) {
+                                if(in.getUpdateElement().equals(e)) {
+                                    add = false;
+                                    break;
+                                }
+                            }
+                            if(add) {
+                                OperationContainer<InstallSupport> upd = OperationContainer.createForUpdate();
+                                OperationInfo<?> ii = upd.add(e);
+                                newInfos.add(ii);
+                            }
+                        }
+                    } else {
+                        newInfos.add(i);
+                    }
+                }
+                infos = newInfos;
+                
                 int size = 0;
                 for (OperationInfo info : infos) {
                     size += info.getUpdateElement().getDownloadSize();
@@ -307,10 +338,9 @@ public class InstallSupportImpl {
                 }
                 
                 boolean needsRestart = false;
-                JarEntry updaterJarEntry = null;
-                JarFile updaterJarFile = null;
-                File updaterTargetCluster = null;
                 File targetCluster = null;
+                List <UpdaterInfo> updaterFiles = new ArrayList <UpdaterInfo> ();
+                
                 for (ModuleUpdateElementImpl moduleImpl : affectedModuleImpls) {
                     synchronized(this) {
                         if (currentStep == STEP.CANCEL) {
@@ -331,41 +361,56 @@ public class InstallSupportImpl {
                     URL source = moduleImpl.getInstallInfo ().getDistribution ();
                     err.log (Level.FINE, "Source URL for " + moduleImpl.getCodeName () + " is " + source);
                     
-                    boolean isNbmFile = source.getFile().toLowerCase(Locale.US).endsWith(Utilities.NBM_EXTENTSION.toLowerCase(Locale.US));
-                    
-                    File dest = getDestination(targetCluster, moduleImpl.getCodeName(), isNbmFile);
+                    File dest = getDestination(targetCluster, moduleImpl.getCodeName(), source);
                     assert dest != null : "Destination file exists for " + moduleImpl + " in " + targetCluster;
                     
-                    // check if 'updater.jar' is being installed
-                    if (AUTOUPDATE_SERVICES_MODULE.equals (moduleImpl.getCodeName ())) {
-                        err.log (Level.FINEST, AUTOUPDATE_SERVICES_MODULE + " is being installed, check if contains " + ModuleUpdater.AUTOUPDATE_UPDATER_JAR_PATH);
-                        JarFile jf = new JarFile (dest);
-                        try {
-                            for (JarEntry entry : Collections.list (jf.entries ())) {
-                                if (ModuleUpdater.AUTOUPDATE_UPDATER_JAR_PATH.equals (entry.toString ())) {
-                                    err.log (Level.FINE, ModuleUpdater.AUTOUPDATE_UPDATER_JAR_PATH + " is being installed from " + moduleImpl.getCodeName ());
-                                    updaterJarEntry = entry;
-                                    updaterJarFile = jf;
-                                    updaterTargetCluster = targetCluster;
-                                    needsRestart = true;
-                                    break;
-                                }
-                            }
-                        } finally {
-                            if (jf != null && ! jf.equals (updaterJarFile)) {
-                                jf.close ();
-                            }
-                        }
+                    // check if 'updater.jar' or 'updater_<branding>_<locale>.jar' is being installed
+                    if (AUTOUPDATE_SERVICES_MODULE.equals(moduleImpl.getCodeName())) {
+                        err.log(Level.FINEST, AUTOUPDATE_SERVICES_MODULE + " is being installed, check if contains " + ModuleUpdater.AUTOUPDATE_UPDATER_JAR_PATH);
                     }
-                    
+
+                    JarFile jf = new JarFile(dest);
+                    boolean added = false;
+                    try {
+                       for (JarEntry entry : Collections.list(jf.entries())) {
+                            if (ModuleUpdater.AUTOUPDATE_UPDATER_JAR_PATH.equals(entry.toString()) ||
+                                    entry.toString().matches(ModuleUpdater.AUTOUPDATE_UPDATER_JAR_LOCALE_PATTERN)) {
+                                err.log(Level.FINE, entry.toString() + " is being installed from " + moduleImpl.getCodeName());
+                                updaterFiles.add(new UpdaterInfo(entry, jf, targetCluster));
+                                needsRestart = true;
+                                added = true;
+                             }
+                         }
+                    } finally {
+                        if (jf != null && !added) {
+                            jf.close();
+                        }
+                     }
+
+
                     needsRestart |= needsRestart(installed != null, moduleImpl, dest);
                 }
                 
                 try {
                     // store source of installed files
                     Utilities.writeAdditionalInformation (getElement2Clusters ());
-                    if (updaterJarFile != null) {
-                        Utilities.writeUpdateOfUpdaterJar (updaterJarEntry, updaterJarFile, updaterTargetCluster);
+                    for(int i=0;i<updaterFiles.size();i++) {
+                        UpdaterInfo info = updaterFiles.get(i);
+                        Utilities.writeUpdateOfUpdaterJar (info.getUpdaterJarEntry(), info.getUpdaterJarFile(), info.getUpdaterTargetCluster());
+                        boolean hasAnotherEntryInSameJarFile = false;
+                        for(int j = i + 1; j < updaterFiles.size(); j++) {
+                            if(updaterFiles.get(j).getUpdaterJarFile() == info.getUpdaterJarFile()) {
+                                hasAnotherEntryInSameJarFile = true;
+                                break;
+                            }
+                        }
+                        if (!hasAnotherEntryInSameJarFile) {
+                            try {
+                                info.getUpdaterJarFile().close();
+                            } catch (IOException e) {
+                                err.log(Level.INFO, "Cannot close jar file " + info.getUpdaterJarFile());
+                            }
+                        }
                     }
 
                     if (! needsRestart) {
@@ -378,11 +423,15 @@ public class InstallSupportImpl {
 
                         if (progress != null) progress.switchToDeterminate (affectedModuleImpls.size ());
 
-                        if (! getDownloadedFiles ().isEmpty ()) {
+                        Set <File> files = null;
+                        synchronized(downloadedFiles) {
+                            files = new HashSet <File> (downloadedFiles);
+                        }
+                        if (! files.isEmpty ()) {
 
                             // XXX: should run in single Thread
                             Thread th = org.netbeans.updater.UpdaterFrame.runFromIDE(
-                                    getDownloadedFiles (),
+                                    files,
                                     new RefreshModulesListener (progress),
                                     NbBundle.getBranding(), false);
 
@@ -399,7 +448,9 @@ public class InstallSupportImpl {
                                         err.log (Level.INFO, "Timeout waiting for loading module " + impl.getCodeName () + '/' + impl.getSpecificationVersion ());
                                         th.interrupt();
                                         afterInstall ();
-                                        downloadedFiles = null;
+                                        synchronized(downloadedFiles) {
+                                            downloadedFiles.clear();
+                                        }
                                         throw new OperationException (OperationException.ERROR_TYPE.INSTALL,
                                                 NbBundle.getMessage(InstallSupportImpl.class, "InstallSupportImpl_TurnOnTimeout", // NOI18N
                                                 impl.getUpdateElement ()));
@@ -411,7 +462,9 @@ public class InstallSupportImpl {
                             }
                         }
                         afterInstall ();
-                        downloadedFiles = null;
+                        synchronized(downloadedFiles) {
+                            downloadedFiles.clear();
+                        }
                     }
                 } finally {
                     // end progress
@@ -494,9 +547,10 @@ public class InstallSupportImpl {
 
         Utilities.writeInstallLater(new HashMap<UpdateElementImpl, File>(getElement2Clusters ()));
         getElement2Clusters ().clear ();
-        downloadedFiles = null;
+        synchronized (downloadedFiles) {
+            downloadedFiles.clear();
+        }
     }
-
     public String getCertificate(Installer validator, UpdateElement uElement) {
         Collection<Certificate> certificates = certs.get (uElement);
         if (certificates != null) {
@@ -596,12 +650,14 @@ public class InstallSupportImpl {
             boolean cancelled = runningTask.cancel (true);
             assert cancelled : runningTask + " was cancelled.";
         }
-        for (File f : getDownloadedFiles ()) {
-            if (f != null && f.exists ()) {
-                f.delete ();
+        synchronized(downloadedFiles) {
+            for (File f : downloadedFiles) {
+                if (f != null && f.exists ()) {
+                    f.delete ();
+                }
             }
+            downloadedFiles.clear ();
         }
-        getDownloadedFiles ().clear ();
         Utilities.cleanUpdateOfUpdaterJar ();
         if (affectedFeatureImpls != null) affectedFeatureImpls = null;
         if (affectedModuleImpls != null) affectedModuleImpls = null;
@@ -668,9 +724,7 @@ public class InstallSupportImpl {
             throw new OperationException (OperationException.ERROR_TYPE.INSTALL, errorString);
         }
 
-        boolean isNbmFile = source.getFile ().toLowerCase (Locale.US).endsWith (Utilities.NBM_EXTENTSION.toLowerCase (Locale.US));
-
-        File dest = getDestination (targetCluster, toUpdateImpl.getCodeName(), isNbmFile);
+        File dest = getDestination (targetCluster, toUpdateImpl.getCodeName(), source);
         
         // skip already downloaded modules
         if (dest.exists ()) {
@@ -683,7 +737,10 @@ public class InstallSupportImpl {
         // download
         try {
             String label = toUpdateImpl.getDisplayName ();
-            getDownloadedFiles ().add (FileUtil.normalizeFile (dest));
+            File normalized = FileUtil.normalizeFile (dest);
+            synchronized(downloadedFiles) {
+                downloadedFiles.add(normalized);
+            }
             c = copy (source, dest, progress, toUpdateImpl.getDownloadSize (), aggregateDownload, totalSize, label);
         } catch (UnknownHostException x) {
             err.log (Level.INFO, x.getMessage (), x);
@@ -737,8 +794,7 @@ public class InstallSupportImpl {
         File targetCluster = getTargetCluster (installed, toUpdateImpl, isGlobal);
 
         URL source = toUpdateImpl.getInstallInfo().getDistribution();
-        boolean isNbm = source.getFile ().toLowerCase (Locale.US).endsWith (Utilities.NBM_EXTENTSION.toLowerCase (Locale.US));
-        File dest = getDestination (targetCluster, toUpdateImpl.getCodeName(), isNbm);
+        File dest = getDestination (targetCluster, toUpdateImpl.getCodeName(), source);
         assert dest.exists () : dest.getAbsolutePath();        
         
         int wasVerified = 0;
@@ -749,41 +805,99 @@ public class InstallSupportImpl {
         return wasVerified;
     }
     
-    static File getDestination (File targetCluster, String codeName, boolean isNbmFile) {
+    static File getDestination (File targetCluster, String codeName, URL source) {
         err.log (Level.FINE, "Target cluster for " + codeName + " is " + targetCluster);
         File destDir = new File (targetCluster, Utilities.DOWNLOAD_DIR);
         if (! destDir.exists ()) {
             destDir.mkdirs ();
         }
         String fileName = codeName.replace ('.', '-');
-        File destFile = new File (destDir, fileName + (isNbmFile ? Utilities.NBM_EXTENTSION : ""));
+        String filePath = source.getFile().toLowerCase(Locale.US);
+        String ext = filePath.endsWith(Utilities.NBM_EXTENTSION.toLowerCase(Locale.US)) ?
+            Utilities.NBM_EXTENTSION : (
+            filePath.endsWith(Utilities.JAR_EXTENSION.toLowerCase(Locale.US)) ?
+                Utilities.JAR_EXTENSION : ""
+            );
+
+        File destFile = new File (destDir, fileName + ext);
         err.log(Level.FINE, "Destination file for " + codeName + " is " + destFile);
         return destFile;
     }
     
-    private static File getDestination (File targetCluster, String codeName) {
-        return getDestination (targetCluster, codeName, true);
-    }
-
     private boolean cancelled() {
         synchronized (this) {
             return STEP.CANCEL == currentStep;
         }
     }
+
+    private class OpenConnectionListener implements NetworkAccess.NetworkListener {
+        private InputStream stream = null;
+        int contentLength = -1;
+        private URL source = null;
+        private Exception ex = null;
+        public OpenConnectionListener (URL source) {
+            this.source = source;
+        }
+        public InputStream getInputStream() {
+            return stream;
+        }
+        public int getContentLength() {
+            return contentLength;
+        }
+        public void streamOpened(InputStream stream, int contentLength) {
+            err.log(Level.FINEST, "Opened connection for " + source);
+            this.stream = stream;
+            this.contentLength = contentLength;
+        }
+
+        public void accessCanceled() {
+            err.log(Level.INFO, "Opening connection for " + source + "was cancelled");
+        }
+
+        public void accessTimeOut() {
+            err.log(Level.INFO, "Opening connection for " + source + "was finised due to timeout");
+        }
+
+        public void notifyException(Exception x) {
+            ex = x;
+        }
+        public Exception getException() {
+            return ex;
+        }
+
+    }
     private int copy (URL source, File dest, 
             ProgressHandle progress, final int estimatedSize, final int aggregateDownload, final int totalSize,
             String label) throws MalformedURLException, IOException {
         
-        int increment = 0;
-        InputStream is = null;
-        URLConnection connection = null;
-        int contentLength = -1;
-        try {
-            connection = source.openConnection();
-            connection.setConnectTimeout(AutoupdateSettings.getOpenConnectionTimeout ());
+        OpenConnectionListener listener = new OpenConnectionListener(source);
+        final Task task = NetworkAccess.createNetworkAcessTask(source,
+                AutoupdateSettings.getOpenConnectionTimeout(),
+                listener);
+        new Thread(new Runnable() {
+            public void run() {
+                while (true) {
+                    if (task.isFinished()) {
+                        break;
+                    } else if(cancelled()) {
+                        task.cancel();
+                        break;
+                    }
+                    try {
+                        Thread.sleep(50);
+                    } catch (InterruptedException e) {
+                        //ignore
+                    }                    
+                }
+            }
+        }).start();
+        
+        task.waitFinished ();
 
-            is = connection.getInputStream();
-            contentLength = connection.getContentLength();
+        try {
+            if(listener.getException()!=null) {
+                throw listener.getException();
+            }
         } catch (FileNotFoundException x) {
             err.log (Level.INFO, x.getMessage(), x);
             throw new IOException(NbBundle.getMessage(InstallSupportImpl.class,
@@ -792,18 +906,30 @@ public class InstallSupportImpl {
             err.log (Level.INFO, x.getMessage(), x);
             throw new IOException(NbBundle.getMessage(InstallSupportImpl.class,
                     "InstallSupportImpl_Download_Unavailable", source));            
+        } catch (Exception x) {
+            err.log (Level.INFO, x.getMessage(), x);
+            throw new IOException(NbBundle.getMessage(InstallSupportImpl.class,
+                    "InstallSupportImpl_Download_Unavailable", source));
         }
+        if (cancelled()) {
+            err.log(Level.FINE, "Download of " + source + " was cancelled");
+            throw new IOException("Download of " + source + " was cancelled");
+        }
+
+        InputStream is = listener.getInputStream();
+        int contentLength = listener.getContentLength();
 
         BufferedInputStream bsrc = new BufferedInputStream (is);
         BufferedOutputStream bdest = null;
         
         err.log (Level.FINEST, "Copy " + source + " to " + dest + "[" + estimatedSize + "]");
-        
+        boolean canceled = false;
+        int increment = 0;
         try {
             byte [] bytes = new byte [1024];
             int size;
             int c = 0;
-            while (!cancelled() && (size = bsrc.read (bytes)) != -1) {
+            while (!(canceled = cancelled()) && (size = bsrc.read (bytes)) != -1) {
                 if(bdest == null) {
                     bdest = new BufferedOutputStream (new FileOutputStream (dest));
                 }
@@ -841,13 +967,24 @@ public class InstallSupportImpl {
             }
         }
         if (contentLength != -1 && increment != contentLength) {
-            err.log(Level.INFO, "Content length was reported as " + contentLength + " byte(s) but read " + increment + " byte(s)");
+            if(canceled) {
+                err.log(Level.FINE, "Download of " + source + " was cancelled");
+            } else {
+                err.log(Level.INFO, "Content length was reported as " + contentLength + " byte(s) but read " + increment + " byte(s)");
+            }
             if(bdest!=null && dest.exists()) {
                 err.log(Level.INFO, "Deleting not fully downloaded file " + dest);
                 dest.delete();
-                getDownloadedFiles ().remove(FileUtil.normalizeFile (dest));
+                File normalized = FileUtil.normalizeFile (dest);
+                synchronized(downloadedFiles) {
+                    downloadedFiles.remove(normalized);
+                }
+            }
+            if(canceled) {
+                throw new IOException("Download of " + source + " was cancelled");
+            } else {
+                throw new IOException("Server closed connection unexpectedly");
             }
-            throw new IOException("Server closed connection unexpectedly");
         }
 
         err.log (Level.FINE, "Destination " + dest + " is successfully wrote. Size " + dest.length());
@@ -1014,11 +1151,39 @@ public class InstallSupportImpl {
         }
         return es;
     }
-    
-    private synchronized Set<File> getDownloadedFiles () {
-        if (downloadedFiles == null) {
-            downloadedFiles = new HashSet<File> ();
+        private static class UpdaterInfo {
+        private JarEntry updaterJarEntry;
+        private JarFile updaterJarFile;
+        private File updaterTargetCluster;
+
+        public UpdaterInfo(JarEntry updaterJarEntry, JarFile updaterJarFile, File updaterTargetCluster) {
+            this.updaterJarEntry = updaterJarEntry;
+            this.updaterJarFile = updaterJarFile;
+            this.updaterTargetCluster = updaterTargetCluster;
+        }
+
+        public JarEntry getUpdaterJarEntry() {
+            return updaterJarEntry;
+        }
+
+        public void setUpdaterJarEntry(JarEntry updaterJarEntry) {
+            this.updaterJarEntry = updaterJarEntry;
+        }
+
+        public JarFile getUpdaterJarFile() {
+            return updaterJarFile;
+        }
+
+        public void setUpdaterJarFile(JarFile updaterJarFile) {
+            this.updaterJarFile = updaterJarFile;
+        }
+
+        public File getUpdaterTargetCluster() {
+            return updaterTargetCluster;
+        }
+
+        public void setUpdaterTargetCluster(File updaterTargetCluster) {
+            this.updaterTargetCluster = updaterTargetCluster;
         }
-        return downloadedFiles;
     }
 }
