@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -272,8 +275,13 @@ public class UnitTab extends javax.swing.JPanel {
     }
     
     private void prepareTopButton (Action action) {
-        topButton.setToolTipText ((String)action.getValue (JComponent.TOOL_TIP_TEXT_KEY));
-        topButton.setAction (action);
+        if(action!=null) {
+            topButton.setToolTipText ((String)action.getValue (JComponent.TOOL_TIP_TEXT_KEY));
+            topButton.setAction (action);
+            topButton.setVisible(true);            
+        } else {
+            topButton.setVisible(false);
+        }
     }
     
     @Override
@@ -400,7 +408,7 @@ public class UnitTab extends javax.swing.JPanel {
             bTabAction.setAction(activateAction);
             bTabAction1.setAction(deactivateAction);
             bTabAction2.setAction (uninstall);
-            prepareTopButton (reloadAction = new ReloadAction ());
+            prepareTopButton (null/*reloadAction = new ReloadAction ()*/);
             table.setEnableRenderer (new EnableRenderer ());
             initReloadTooltip();
             break;
@@ -627,57 +635,57 @@ public class UnitTab extends javax.swing.JPanel {
             }
         });
 
-        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
+        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
         this.setLayout(layout);
         layout.setHorizontalGroup(
-            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
-            .add(layout.createSequentialGroup()
+            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
+            .addGroup(layout.createSequentialGroup()
                 .addContainerGap()
-                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
-                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
-                        .add(topButton)
-                        .add(18, 18, 18)
-                        .add(detailView)
-                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 375, Short.MAX_VALUE)
-                        .add(lSearch)
-                        .add(4, 4, 4)
-                        .add(tfSearch, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 114, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
-                    .add(spTab, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 712, Short.MAX_VALUE)
-                    .add(layout.createSequentialGroup()
-                        .add(bTabAction)
-                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
-                        .add(bTabAction1)
-                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
-                        .add(bTabAction2)
-                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
-                        .add(lSelectionInfo)
-                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
-                        .add(lWarning, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 482, Short.MAX_VALUE)
-                        .add(99, 99, 99)))
+                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
+                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
+                        .addComponent(topButton)
+                        .addGap(18, 18, 18)
+                        .addComponent(detailView)
+                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 443, Short.MAX_VALUE)
+                        .addComponent(lSearch)
+                        .addGap(4, 4, 4)
+                        .addComponent(tfSearch, javax.swing.GroupLayout.PREFERRED_SIZE, 114, javax.swing.GroupLayout.PREFERRED_SIZE))
+                    .addComponent(spTab, javax.swing.GroupLayout.DEFAULT_SIZE, 776, Short.MAX_VALUE)
+                    .addGroup(layout.createSequentialGroup()
+                        .addComponent(bTabAction)
+                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
+                        .addComponent(bTabAction1)
+                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
+                        .addComponent(bTabAction2)
+                        .addGap(8, 8, 8)
+                        .addComponent(lSelectionInfo)
+                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
+                        .addComponent(lWarning, javax.swing.GroupLayout.DEFAULT_SIZE, 542, Short.MAX_VALUE)
+                        .addGap(99, 99, 99)))
                 .addContainerGap())
         );
         layout.setVerticalGroup(
-            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
-            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
+            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
+            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                 .addContainerGap()
-                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
-                    .add(tfSearch, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
-                    .add(lSearch)
-                    .add(topButton)
-                    .add(detailView))
-                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
-                .add(spTab, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 73, Short.MAX_VALUE)
-                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
-                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER, false)
-                    .add(bTabAction)
-                    .add(lSelectionInfo, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
-                    .add(lWarning)
-                    .add(bTabAction1)
-                    .add(bTabAction2))
+                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
+                    .addComponent(tfSearch, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
+                    .addComponent(lSearch)
+                    .addComponent(topButton)
+                    .addComponent(detailView))
+                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
+                .addComponent(spTab, javax.swing.GroupLayout.DEFAULT_SIZE, 73, Short.MAX_VALUE)
+                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
+                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER, false)
+                    .addComponent(bTabAction)
+                    .addComponent(lSelectionInfo, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
+                    .addComponent(lWarning)
+                    .addComponent(bTabAction1)
+                    .addComponent(bTabAction2))
                 .addContainerGap())
         );
 
-        layout.linkSize(new java.awt.Component[] {lSelectionInfo, lWarning}, org.jdesktop.layout.GroupLayout.VERTICAL);
+        layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {lSelectionInfo, lWarning});
 
         lSearch.getAccessibleContext().setAccessibleDescription(org.openide.util.NbBundle.getMessage(UnitTab.class, "ACD_Search")); // NOI18N
         topButton.getAccessibleContext().setAccessibleName(org.openide.util.NbBundle.getMessage(UnitTab.class, "ACN_Reload")); // NOI18N
@@ -1169,6 +1177,8 @@ public class UnitTab extends javax.swing.JPanel {
                 wizardFinished = wizard.invokeWizard ();
             } finally {
                 Containers.forUninstall ().removeAll ();
+                Containers.forDisable().removeAll ();
+                Containers.forEnable().removeAll ();
                 fireUpdataUnitChange ();
                 if (!wizardFinished) {
                     UnitCategoryTableModel.restoreState (model.getUnits (), state, model.isMarkedAsDefault ());
@@ -1188,13 +1198,13 @@ public class UnitTab extends javax.swing.JPanel {
             for (Unit u : units) {
                 if (u instanceof Unit.Installed) {
                     Unit.Installed inst = (Unit.Installed)u;
-                    if (!inst.isUninstallAllowed() && !inst.isDeactivationAllowed()) {
-                        setEnabled(false);
+                    if (inst.isUninstallAllowed()) {
+                        setEnabled(true);
                         return;
                     }
                 }
             }
-            setEnabled (true);
+            setEnabled (false);
         }
 
     }
@@ -1264,7 +1274,10 @@ public class UnitTab extends javax.swing.JPanel {
                 if (wizardFinished) {
                     reloadTask (false).schedule (10);
                 } else {
-                    UnitCategoryTableModel.restoreState (model.getUnits (), state, model.isMarkedAsDefault ());
+                    fireUpdataUnitChange ();
+                    if (!wizardFinished) {
+                        UnitCategoryTableModel.restoreState (model.getUnits (), state, model.isMarkedAsDefault ());
+                    }
                     restoreSelectedRow(row);                
                     refreshState ();
                 }
@@ -1354,12 +1367,17 @@ public class UnitTab extends javax.swing.JPanel {
                 finished = wizard.invokeWizard(true);
             } finally {
                 Containers.forEnable().removeAll();
+                Containers.forDisable().removeAll();
+                Containers.forUninstall ().removeAll();
                 if (finished) {
                     for (Unit u : model.getMarkedUnits()) {
                         u.setMarked(false);
                     }
                 }
                 fireUpdataUnitChange();
+                if(!finished) {
+                    UnitCategoryTableModel.restoreState (model.getUnits (), state, model.isMarkedAsDefault ());
+                }
                 restoreSelectedRow(row);
                 refreshState ();
                 focusTable ();
@@ -1489,6 +1507,7 @@ public class UnitTab extends javax.swing.JPanel {
         @Override
         public void performerImpl() {
             final int row = getSelectedRow ();
+            final Map<String, Boolean> state = UnitCategoryTableModel.captureState (model.getUnits ());
             OperationContainer<OperationSupport> c = Containers.forDisable();
             for (Unit u : model.getUnits()) {
                 if (u.isMarked() && isEnabled(u)) {
@@ -1496,14 +1515,21 @@ public class UnitTab extends javax.swing.JPanel {
                 }
             }
             UninstallUnitWizard wizard = new UninstallUnitWizard ();
-            if (wizard.invokeWizard (false)) {
+            boolean finished = false;
+            try {
+                finished = wizard.invokeWizard(false);
+            } finally {
                 Containers.forUninstall().removeAll();
+                Containers.forDisable().removeAll();
+                Containers.forEnable().removeAll();
+                fireUpdataUnitChange();
+                if (!finished) {
+                    UnitCategoryTableModel.restoreState(model.getUnits(), state, model.isMarkedAsDefault());
+                }
+                restoreSelectedRow(row);
+                refreshState();
+                focusTable();
             }
-            Containers.forDisable().removeAll();
-            fireUpdataUnitChange ();
-            restoreSelectedRow(row);
-            refreshState ();
-            focusTable ();
         }
         /*
             Unit.Installed unit = (Unit.Installed)u;
@@ -1579,7 +1605,13 @@ public class UnitTab extends javax.swing.JPanel {
                     Unit.Installed installed = (Unit.Installed)u;
                     if (installed.getRelevantElement ().isEnabled ()) {
                         OperationInfo info = Containers.forDisable ().add (installed.updateUnit, installed.getRelevantElement ());
-                        assert info != null;
+                        // Issue #169640
+                        // The relevant element can actually be present in forDisable container
+                        // due to current Unit.Installed.setMarked() implementation as
+                        // it operates on the the same ("global") Containers.forDisable() container.
+                        // That is the reason for the commenting the following assertion originally presented
+                        
+                        //assert info != null;
                     }
                 }
             }
