@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -84,6 +87,7 @@ import org.openide.DialogDescriptor;
 import org.openide.DialogDisplayer;
 import org.openide.NotifyDescriptor;
 import org.openide.awt.Mnemonics;
+import org.openide.awt.Notification;
 import org.openide.awt.NotificationDisplayer;
 import org.openide.util.Cancellable;
 import org.openide.util.Exceptions;
@@ -107,6 +111,7 @@ public class InstallStep implements WizardDescriptor.FinishablePanel<WizardDescr
     private boolean indeterminateProgress = false;
     private int processedUnits = 0;
     private int totalUnits = 0;
+    private static Notification restartNotification = null;
     private static  final Logger log = Logger.getLogger (InstallStep.class.getName ());
     private final List<ChangeListener> listeners = new ArrayList<ChangeListener> ();
     
@@ -330,7 +335,7 @@ public class InstallStep implements WizardDescriptor.FinishablePanel<WizardDescr
                         // try again
                         return false;
                     } else if (DialogDescriptor.CLOSED_OPTION.equals (ret)) {
-                        handleCancel ();
+                        model.getCancelButton(wd).doClick();
                     }
                 }
             } else {
@@ -549,12 +554,13 @@ public class InstallStep implements WizardDescriptor.FinishablePanel<WizardDescr
         if (installException == null) {
             component.setHeadAndContent (getBundle (HEAD_INSTALL_DONE), getBundle (CONTENT_INSTALL_DONE));
             panel.setBody (getBundle ("InstallStep_InstallDone_Text"),
-                    InstallUnitWizardModel.getVisibleUpdateElements (model.getAllUpdateElements (), false, model.getOperation ()));
+                    model.getAllVisibleUpdateElements ());
         } else {
             component.setHeadAndContent (getBundle (HEAD_INSTALL_UNSUCCESSFUL), getBundle (CONTENT_INSTALL_UNSUCCESSFUL));
             panel.setBody (getBundle ("InstallStep_InstallUnsuccessful_Text", installException.getLocalizedMessage ()),
-                    InstallUnitWizardModel.getVisibleUpdateElements (model.getAllUpdateElements (), false, model.getOperation ()));
+                    model.getAllVisibleUpdateElements ());
         }
+
         panel.hideRunInBackground ();
     }
     
@@ -563,7 +569,7 @@ public class InstallStep implements WizardDescriptor.FinishablePanel<WizardDescr
         model.modifyOptionsForDoClose (wd, true);
         restarter = r;
         panel.setRestartButtonsVisible (true);
-        panel.setBody (getBundle ("InstallStep_InstallDone_Text"), InstallUnitWizardModel.getVisibleUpdateElements (model.getAllUpdateElements (), false, model.getOperation ()));
+        panel.setBody (getBundle ("InstallStep_InstallDone_Text"), model.getAllVisibleUpdateElements ());
         panel.hideRunInBackground ();
         if (runInBackground ()) {
             InstallSupport support = model.getInstallSupport ();
@@ -596,21 +602,28 @@ public class InstallStep implements WizardDescriptor.FinishablePanel<WizardDescr
     }
     
     static void notifyRestartNeeded (final Runnable onMouseClick, final String tooltip) {
-        final NotifyDescriptor nd = new NotifyDescriptor.Confirmation (
-                                            getBundle ("RestartConfirmation_Message"),
-                                            getBundle ("RestartConfirmation_Title"),
-                                            NotifyDescriptor.YES_NO_OPTION);
+        //final NotifyDescriptor nd = new NotifyDescriptor.Confirmation (
+        //                                    getBundle ("RestartConfirmation_Message"),
+        //                                    getBundle ("RestartConfirmation_Title"),
+        //                                    NotifyDescriptor.YES_NO_OPTION);
         ActionListener onClickAction = new ActionListener() {
             public void actionPerformed(ActionEvent e) {
-                DialogDisplayer.getDefault ().notify (nd);
-                if (NotifyDescriptor.OK_OPTION.equals (nd.getValue ())) {
+                //DialogDisplayer.getDefault ().notify (nd);
+                //if (NotifyDescriptor.OK_OPTION.equals (nd.getValue ())) {
                     onMouseClick.run ();
-                }
+                //}
             }
         };
-        NotificationDisplayer.getDefault().notify(tooltip, 
-                ImageUtilities.loadImageIcon("org/netbeans/modules/autoupdate/ui/resources/restart.png", false),
-                getBundle("RestartNeeded_Details"), onClickAction, NotificationDisplayer.Priority.HIGH);
+        synchronized (InstallStep.class) {
+            if (restartNotification != null) {
+                restartNotification.clear();
+                restartNotification = null;
+            }
+
+            restartNotification = NotificationDisplayer.getDefault().notify(tooltip,
+                    ImageUtilities.loadImageIcon("org/netbeans/modules/autoupdate/ui/resources/restart.png", false),
+                    getBundle("RestartNeeded_Details"), onClickAction, NotificationDisplayer.Priority.HIGH);
+        }
     }
 
     private void notifyNetworkProblem (final OperationException ex) {
