@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -44,6 +47,8 @@ package org.netbeans.modules.autoupdate.ui.wizards;
 import java.awt.Component;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.Comparator;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
@@ -55,11 +60,15 @@ import javax.swing.JPanel;
 import javax.swing.SwingUtilities;
 import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
+import org.netbeans.api.autoupdate.InstallSupport;
+import org.netbeans.api.autoupdate.OperationContainer;
+import org.netbeans.api.autoupdate.OperationContainer.OperationInfo;
 import org.netbeans.api.autoupdate.OperationException;
 import org.netbeans.api.autoupdate.UpdateElement;
 import org.netbeans.api.autoupdate.UpdateManager;
 import org.netbeans.api.autoupdate.UpdateUnit;
 import org.netbeans.modules.autoupdate.ui.Containers;
+import org.netbeans.modules.autoupdate.ui.Utilities;
 import org.netbeans.modules.autoupdate.ui.wizards.OperationWizardModel.OperationType;
 import org.openide.WizardDescriptor;
 import org.openide.modules.Dependency;
@@ -155,12 +164,10 @@ public class OperationDescriptionStep implements WizardDescriptor.Panel<WizardDe
             }
             body = new OperationDescriptionPanel (tableTitle,
                     preparePluginsForShow (
-                        OperationWizardModel.getVisibleUpdateElements (
-                                model.getPrimaryUpdateElements (),
-                                false,
-                                model.getOperation ()),
+                    model.getPrimaryUpdateElements(),
+                    model.getPrimaryVisibleUpdateElements(false),
                         model.getCustomHandledComponents (),
-                        model.getOperation ()),
+                        model.getOperation (), false),
                     "",
                     "",
                     true);
@@ -192,16 +199,19 @@ public class OperationDescriptionStep implements WizardDescriptor.Panel<WizardDe
                 } else {
                     body = new OperationDescriptionPanel (tableTitle,
                             preparePluginsForShow (
-                                OperationWizardModel.getVisibleUpdateElements (
-                                    model.getPrimaryUpdateElements (), false, model.getOperation ()),
+                                model.getPrimaryUpdateElements(),
+                                model.getPrimaryVisibleUpdateElements(true),
                                 model.getCustomHandledComponents (),
-                                model.getOperation ()),
+                                model.getOperation (),
+                                true),
                             dependenciesTitle,
                             preparePluginsForShow (
-                                OperationWizardModel.getVisibleUpdateElements (model.getRequiredUpdateElements (), true, model.getOperation ()),
+                                model.getRequiredUpdateElements(),
+                                model.getRequiredVisibleUpdateElements(),
                                 null,
-                                model.getOperation ()),
-                            ! OperationWizardModel.getVisibleUpdateElements (model.getRequiredUpdateElements (), true, model.getOperation ()).isEmpty ());
+                                model.getOperation (),
+                                true),
+                            ! model.getRequiredVisibleUpdateElements().isEmpty ());
                 }
                 final JPanel finalPanel = body;
                 readyToGo = model != null && ! hasBrokenDependencies;
@@ -357,17 +367,125 @@ public class OperationDescriptionStep implements WizardDescriptor.Panel<WizardDe
         return presentationName == null ? dep : presentationName;
     }
     
-    private String preparePluginsForShow (Set<UpdateElement> plugins, Set<UpdateElement> customHandled, OperationType type) {
+    private String preparePluginsForShow (Set <UpdateElement> allPlugins, Set<UpdateElement> visiblePlugins, Set<UpdateElement> customHandled, OperationType type,boolean checkInternalUpdates) {
         String s = new String ();
         List<String> names = new ArrayList<String> ();
+        List <UpdateUnit> invisibleIncluded = new ArrayList <UpdateUnit>();
+        List <UpdateUnit> units = new ArrayList <UpdateUnit>();
+        List <UpdateElement> plugins = new ArrayList<UpdateElement>();
+        for(UpdateElement el : visiblePlugins) {
+            boolean internal = false;
+            if (OperationWizardModel.OperationType.UPDATE == type && el.getUpdateUnit ().getInstalled () != null) {
+                 String oldVersion = el.getUpdateUnit ().getInstalled ().getSpecificationVersion ();
+                 String newVersion = el.getSpecificationVersion ();
+                 if(oldVersion.equals(newVersion) || !el.getUpdateUnit().getType().equals(UpdateManager.TYPE.KIT_MODULE) ) {
+                     internal = true;
+                 }
+            }
+            if(internal) {
+                plugins.add(el);
+            } else {
+                plugins.add(0, el);
+            }
+        }
+
+        for(UpdateElement p : plugins) {
+            units.add(p.getUpdateUnit());
+        }
+        //HashMap <UpdateUnit, List<UpdateElement>> map = Utilities.getVisibleModulesDependecyMap(UpdateManager.getDefault().getUpdateUnits(Utilities.getUnitTypes()));
+        
+
         if (plugins != null && ! plugins.isEmpty ()) {
+
+            HashMap <UpdateUnit, List<UpdateElement>> map = null;
+            if(checkInternalUpdates) {
+                map = Utilities.getVisibleModulesDependecyMap(units);
+            }
+
             for (UpdateElement el : plugins) {
                 String updatename;
                 updatename = "<b>"  + el.getDisplayName () + "</b> "; // NOI18N
                 if (OperationWizardModel.OperationType.UPDATE == type && el.getUpdateUnit ().getInstalled () != null) {
                     String oldVersion = el.getUpdateUnit ().getInstalled ().getSpecificationVersion ();
                     String newVersion = el.getSpecificationVersion ();
-                    updatename += getBundle ("OperationDescriptionStep_UpdatePluginVersionFormat", oldVersion, newVersion);
+                    OperationContainer<InstallSupport> container = OperationContainer.createForUpdate();
+                    if (oldVersion.equals(newVersion) && checkInternalUpdates) {
+                        //internal update
+                        //updatename += getBundle ("OperationDescriptionStep_PluginVersionFormat", oldVersion);
+                        OperationContainer<InstallSupport> internalUpdate = OperationContainer.createForInternalUpdate();
+                        internalUpdate.add(el);
+                        for (OperationInfo<InstallSupport> info : internalUpdate.listAll()) {
+                            if (!info.getUpdateElement().equals(el)) {
+                                if (!container.contains(info.getUpdateElement()) && allPlugins.contains(info.getUpdateElement())) {
+                                    container.add(info.getUpdateElement());
+                                }
+                            }
+                            for (UpdateElement r : info.getRequiredElements()) {
+                                if (!container.contains(r) && container.canBeAdded(r.getUpdateUnit(), r) && allPlugins.contains(r)) {
+                                    container.add(r);
+                                }
+                            }
+                        }
+                    } else {
+                        updatename += getBundle("OperationDescriptionStep_UpdatePluginVersionFormat", oldVersion, newVersion);
+                        if(checkInternalUpdates) {
+                            container.add(el);
+                        }
+                    }
+
+                    
+                    if(checkInternalUpdates) {
+                        List<UpdateElement> list = new ArrayList<UpdateElement>();
+                        for (OperationInfo<InstallSupport> info : container.listAll()) {
+
+                            if(!info.getUpdateUnit().equals(el.getUpdateUnit()) &&
+                                    info.getUpdateUnit().getInstalled() != null &&
+                                    !info.getUpdateUnit().isPending()) {
+                                list.add(info.getUpdateElement());
+                            }
+                            for (UpdateElement upd : info.getRequiredElements()) {
+                                if (upd.getUpdateUnit().getInstalled() != null &&
+                                        !upd.getUpdateUnit().isPending()) {
+                                    list.add(upd);
+                                }
+                            }
+                        }
+                        
+                        Set<UpdateElement> sorted = new TreeSet<UpdateElement>(new Comparator<UpdateElement>() {
+                            public int compare(UpdateElement o1, UpdateElement o2) {
+                                return o1.getDisplayName().compareTo(o2.getDisplayName());
+                            }
+                        });
+                        sorted.addAll(list);
+
+
+                        for (UpdateElement upd : sorted) {
+                            UpdateUnit unit = upd.getUpdateUnit();
+                            if(unit.getType().equals(UpdateManager.TYPE.KIT_MODULE) || invisibleIncluded.contains(unit)) {
+                                continue;
+                            }
+                            UpdateUnit visibleUnit = Utilities.getVisibleUnitForInvisibleModule(unit, map);
+                            if(visibleUnit!=null && visibleUnit != el.getUpdateUnit()) {
+                                continue;
+                            }
+                            if(!unit.getType().equals(UpdateManager.TYPE.KIT_MODULE)) {
+                                invisibleIncluded.add(unit);
+                            }
+                            if(unit.getInstalled()!=null) {
+                                updatename += "<br>&nbsp;&nbsp;&nbsp;&nbsp;" + upd.getDisplayName() +
+                                    " [" +
+                                    unit.getInstalled().getSpecificationVersion() + " -> " +
+                                    unit.getAvailableUpdates().get(0).getSpecificationVersion() +
+                                    "]";
+                            } else {
+                                updatename += "<br>&nbsp;&nbsp;&nbsp;&nbsp;" + upd.getDisplayName() +
+                                    " [" +
+                                    unit.getAvailableUpdates().get(0).getSpecificationVersion() +
+                                    "]";
+                            }
+                        }
+                        map.remove(el.getUpdateUnit());
+                    }
                 } else {
                     updatename += getBundle ("OperationDescriptionStep_PluginVersionFormat",  // NOI18N
                         el.getSpecificationVersion ());
