@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -153,15 +156,15 @@ public class OutputDocument implements Document, Element, ChangeListener {
     }
     
     public String getText(int offset, int length) throws BadLocationException {
-        if (offset < 0 || offset > getLines().getCharCount() + inBuffer.length() || length < 0) {
-            throw new BadLocationException ("Bad: " + offset + "," +  //NOI18N
-                length, offset);
-        }
         if (length == 0) {
             return ""; //NOI18N
         }
         String result;
         synchronized (getLines().readLock()) {
+            if (offset < 0 || offset + length > getLines().getCharCount() + inBuffer.length() || length < 0) {
+                throw new BadLocationException("Bad: " + offset + "," + //NOI18N
+                        length + " (" + getLines().getCharCount() + ", " + inBuffer.length() + ")", offset);
+            }
             int linesOffset = Math.min(getLines().getCharCount(), offset);
             int linesEnd = Math.min(getLines().getCharCount(), offset + length);
             result = getLines().getText(linesOffset, linesEnd);
@@ -198,6 +201,13 @@ public class OutputDocument implements Document, Element, ChangeListener {
         try {
             synchronized (getLines().readLock()) {
                 int charCount = getLines().getCharCount();
+                // #180404
+                if (charCount < 0) {
+                    txt.array = new char[0];
+                    txt.offset=0;
+                    txt.count = 0;
+                    return;
+                }
                 int linesOffset = Math.min(charCount, offset);
                 int linesEnd = Math.min(charCount, offset + length);
                 char[] chars = getLines().getText(linesOffset, linesEnd, reusableSubrange);
@@ -608,6 +618,9 @@ public class OutputDocument implements Document, Element, ChangeListener {
         
         void calc() {
             synchronized (getLines().readLock()) {
+                if (lineIndex < 0) {
+                    return;
+                }
                 if (startOffset == -1) {
                     startOffset = getLines().getLineStart(lineIndex);
                     if (lineIndex >= getLines().getLineCount()-1) {
