@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -41,7 +44,6 @@
 
 package org.netbeans.core.output2;
 
-import java.awt.Color;
 import java.awt.Component;
 import java.awt.Container;
 import java.awt.Cursor;
@@ -49,7 +51,6 @@ import java.awt.FileDialog;
 import java.awt.Font;
 import java.awt.Frame;
 import java.awt.Point;
-import java.awt.Toolkit;
 import java.awt.event.ActionEvent;
 import java.awt.event.KeyEvent;
 import java.beans.PropertyChangeListener;
@@ -85,6 +86,7 @@ import org.openide.DialogDescriptor;
 import org.openide.DialogDisplayer;
 import org.openide.NotifyDescriptor;
 import org.openide.actions.FindAction;
+import org.openide.awt.StatusDisplayer;
 import org.openide.util.Exceptions;
 import org.openide.util.NbBundle;
 import org.openide.util.Utilities;
@@ -98,11 +100,13 @@ import org.openide.xml.XMLUtil;
  */
 final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks {
     private final NbIO io;
+    private OutWriter outWriter;
 
     OutputTab(NbIO io) {
         this.io = io;
         if (Controller.LOG) Controller.log ("Created an output component for " + io);
-        OutputDocument doc = new OutputDocument (((NbWriter) io.getOut()).out());
+        outWriter = ((NbWriter) io.getOut()).out();
+        OutputDocument doc = new OutputDocument(outWriter);
         setDocument(doc);
 
         installKBActions();
@@ -136,7 +140,6 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
         assert SwingUtilities.isEventDispatchThread();
         Document old = getDocument();
         hasOutputListeners = false;
-        firstNavigableListenerLine = -1;
         super.setDocument(doc);
         if (old != null && old instanceof OutputDocument) {
             ((OutputDocument) old).dispose();
@@ -145,10 +148,11 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
 
     public void reset() {
         if (origPane != null) {
-            setOutputPane(origPane);
+            setFilter(null, false, false);
         }
-        setDocument(new OutputDocument(((NbWriter) io.getOut()).out()));
-        io.setClosed(false);
+        // get new OutWriter
+        outWriter = io.out();
+        setDocument(new OutputDocument(outWriter));
     }
 
     public OutputDocument getDocument() {
@@ -170,7 +174,7 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
         if (in != null) {
             if (Controller.LOG) Controller.log("Sending input to " + in);
             in.pushText(txt + "\n");
-            io.getOut().println(txt);
+            outWriter.println(txt);
         }
     }
 
@@ -197,49 +201,20 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
         io.getIOContainer().requestActive();
     }
 
-    public void lineClicked(int line) {
+    public void lineClicked(int line, int pos) {
         OutWriter out = getOut();
-        if (out == null) {
-            return;
-        }
-        OutputListener l = out.getLines().getListenerForLine(line);
+        int range[] = new int[2];
+        OutputListener l = out.getLines().getListener(pos, range);
         if (l != null) {
+            int size = out.getLines().getCharCount();
+            assert range[1] < size : "Size: " + size + " range: " + range[0] + " " + range[1];
             ControllerOutputEvent oe = new ControllerOutputEvent(io, out, line);
             l.outputLineAction(oe);
             //Select the text on click
-            getOutputPane().sendCaretToLine(line, true);
+            getOutputPane().sendCaretToPos(range[0], range[1], true);
         }
     }
 
-    boolean linePressed(int line, Point p) {
-        OutWriter out = getOut();
-        if (out != null) {
-            return out.getLines().getListenerForLine(line) != null;
-        } else {
-            return false;
-        }
-    }
-
-    private int firstNavigableListenerLine = -1;
-    /**
-     * Do not unlock scrollbar unless there is a bona-fide error to 
-     * show - deprecation warnings should be ignored.
-     */
-    public int getFirstNavigableListenerLine() {
-        if (firstNavigableListenerLine != -1) {
-            return firstNavigableListenerLine;
-        }
-        
-        int result = -1;
-        OutWriter out = getOut();
-        if (out != null) {
-            if (Controller.LOG) Controller.log ("Looking for first appropriate" +
-                " listener line to send the caret to");
-            result = out.getLines().firstImportantListenerLine();
-        }
-        return result;
-    }
-    
     @Override
     public String toString() {
         return "OutputTab@" + System.identityHashCode(this) + " for " + io;
@@ -249,7 +224,7 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
 
     public void documentChanged(OutputPane pane) {
         if (filtOut != null && pane == origPane) {
-            filtOut.readFrom(io.out());
+            filtOut.readFrom(outWriter);
         }
         boolean hadOutputListeners = hasOutputListeners;
         hasOutputListeners = getOut() != null && getOut().getLines().firstListenerLine() >= 0;
@@ -259,6 +234,10 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
 
         IOContainer ioContainer = io.getIOContainer();
         if (io.isFocusTaken()) {
+	    // The following two lines pulled up from select per bug#185209
+	    ioContainer.open();
+	    // ioContainer.requestVisible();
+
             ioContainer.select(this);
             ioContainer.requestVisible();
         }
@@ -275,11 +254,11 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
     private void navigateToFirstErrorLine() {
         OutWriter out = getOut();
         if (out != null) {
-            int line = getFirstNavigableListenerLine();
+            int line = out.getLines().firstImportantListenerLine();
             if (Controller.LOG) {
                 Controller.log("NAV TO FIRST LISTENER LINE: " + line);
             }
-            if (line > 0) {
+            if (line >= 0) {
                 getOutputPane().sendCaretToLine(line, false);
                 if (isSDI()) {
                     requestActive();
@@ -305,6 +284,7 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
 
     public void closed() {
         io.setClosed(true);
+        io.setStreamClosed(true);
         Controller.getDefault().removeTab(io);
         NbWriter w = io.writer();
         if (w != null && w.isClosed()) {
@@ -339,19 +319,6 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
         return false;
     }
 
-    /**
-     * Fetch the output listener for a given line
-     *
-     * @param line The line to find a listener on
-     * @return An output listener or null
-     */
-    private OutputListener listenerForLine(int line) {
-        OutWriter out = getOut();
-        if (out != null) {
-            return out.getLines().getListenerForLine(line);
-        }
-        return null;
-    }
     /**
      * Flag used to block navigating the editor to the first error line when
      * selecting the error line in the output window after a build (or maybe
@@ -360,55 +327,50 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
     boolean ignoreCaretChanges = false;
 
     /**
-     * Called when the text caret has changed lines - will call OutputListener.outputLineSelected if
-     * there is a listener for that line.
+     * Called when the text caret has changed position - will call OutputListener.outputLineSelected if
+     * there is a listener for that position.
      *
-     * @param line The line the caret is in
+     * @param pos The line the caret is in
      */
-    void caretEnteredLine(int line) {
+    int lastCaretListenerRange[];
+    void caretPosChanged(int pos) {
         if (!ignoreCaretChanges) {
-            OutputListener l = listenerForLine(line);
-            if (l != null) {
-                ControllerOutputEvent oe = new ControllerOutputEvent(io, line);
-                l.outputLineSelected(oe);
+            if (lastCaretListenerRange != null && pos >= lastCaretListenerRange[0] && pos < lastCaretListenerRange[1]) {
+                return;
+            }
+            OutWriter out = getOut();
+            if (out != null) {
+                int[] range = new int[2];
+                OutputListener l = out.getLines().getListener(pos, range);
+                if (l != null) {
+                    ControllerOutputEvent oe = new ControllerOutputEvent(io, out.getLines().getLineAt(pos));
+                    l.outputLineSelected(oe);
+                    lastCaretListenerRange = range;
+                } else {
+                    lastCaretListenerRange = null;
+                }
             }
         }
     }
 
-    /**
-     * Sends the caret in a tab to the nearest error line to its current position, selecting
-     * that line.
-     *
-     * @param backward If the search should be done in reverse
-     */
     private void sendCaretToError(boolean backward) {
         OutWriter out = getOut();
         if (out != null) {
-            int line = getOutputPane().getCaretLine();
-            if (!getOutputPane().isLineSelected(line)) {
-                line += backward ? 1 : -1;
-            }
+            AbstractOutputPane op = getOutputPane();
+            int selStart = op.getSelectionStart();
+            int selEnd = op.getSelectionEnd();
+            int pos = op.getCaretPos();
 
-            if (line >= getOutputPane().getLineCount() - 1) {
-                line = 0;
+            // check if link is selected
+            if (selStart != selEnd && pos == selStart && out.getLines().isListener(selStart, selEnd)) {
+                pos = backward ? selStart - 1 : selEnd + 1;
             }
-            //FirstF12: #48485 - caret is already on the first listener line,
-            //so F12 jumps to the second error.  So search from 0 the first time after a reset
-            int newline = out.getLines().nearestListenerLine(line, backward);
-            if (newline == line) {
-                if (!backward && line != getOutputPane().getLineCount()) {
-                    newline = out.getLines().nearestListenerLine(line + 1, backward);
-                } else if (backward && line > 0) {
-                    newline = out.getLines().nearestListenerLine(line - 1, backward);
-                } else {
-                    return;
-                }
-            }
-            if (newline != -1) {
-                getOutputPane().sendCaretToLine(newline, true);
+            int[] lpos = new int[2];
+            OutputListener l = out.getLines().nearestListener(pos, backward, lpos);
+            if (l != null) {
+                op.sendCaretToPos(lpos[0], lpos[1], true);
                 if (!io.getIOContainer().isActivated()) {
-                    OutputListener l = out.getLines().getListenerForLine(newline);
-                    ControllerOutputEvent ce = new ControllerOutputEvent(io, newline);
+                    ControllerOutputEvent ce = new ControllerOutputEvent(io,  out.getLines().getLineAt(lpos[0]));
                     l.outputLineAction(ce);
                 }
             }
@@ -435,13 +397,29 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
             boolean matchCase = FindDialogPanel.matchCase();
             int[] sel = reversed ? out.getLines().rfind(pos, lastPattern, regExp, matchCase)
                     : out.getLines().find(pos, lastPattern, regExp, matchCase);
+            String appendMsg = null;
+            if (sel == null) {
+                sel = reversed ? out.getLines().rfind(out.getLines().getCharCount(), lastPattern, regExp, matchCase)
+                        : out.getLines().find(0, lastPattern, regExp, matchCase);
+                if (sel != null) {
+                    appendMsg = NbBundle.getMessage(OutputTab.class, reversed ? "MSG_SearchFromEnd" : "MSG_SearchFromBeg");
+                }
+            }
+            String msg;
             if (sel != null) {
                 getOutputPane().unlockScroll();
                 getOutputPane().setSelection(sel[0], sel[1]);
-                return true;
+                int line = out.getLines().getLineAt(sel[0]);
+                int col = sel[0] - out.getLines().getLineStart(line);
+                msg = NbBundle.getMessage(OutputTab.class, "MSG_Found", lastPattern, line + 1, col + 1);
+                if (appendMsg != null) {
+                    msg = msg + "; " + appendMsg;
+                }
             } else {
-                Toolkit.getDefaultToolkit().beep();
+                msg = NbBundle.getMessage(OutputTab.class, "MSG_NotFound", lastPattern);
             }
+            StatusDisplayer.getDefault().setStatusText(msg);
+            return sel != null;
         }
         return false;
     }
@@ -543,17 +521,18 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
      * Called when a line is clicked - if an output listener is listening on that
      * line, it will be sent <code>outputLineAction</code>.
      */
-    private void openLineIfError() {
+    private void openHyperlink() {
         OutWriter out = getOut();
         if (out != null) {
-            int line = getOutputPane().getCaretLine();
-            OutputListener lis = out.getLines().getListenerForLine(line);
-            if (lis != null) {
+            int pos = getOutputPane().getCaretPos();
+            int[] range = new int[2];
+            OutputListener l = out.getLines().getListener(pos, range);
+            if (l != null) {
                 ignoreCaretChanges = true;
-                getOutputPane().sendCaretToLine(line, true);
+                getOutputPane().sendCaretToPos(range[0], range[1], true);
                 ignoreCaretChanges = false;
-                ControllerOutputEvent coe = new ControllerOutputEvent(io, line);
-                lis.outputLineAction(coe);
+                ControllerOutputEvent coe = new ControllerOutputEvent(io, out.getLines().getLineAt(pos));
+                l.outputLineAction(coe);
             }
         }
     }
@@ -629,7 +608,7 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
         saveAsAction.setEnabled(enable);
         selectAllAction.setEnabled(enable);
         copyAction.setEnabled(pane.hasSelection());
-        boolean hasErrors = out == null ? false : out.getLines().firstListenerLine() != -1;
+        boolean hasErrors = out == null ? false : out.getLines().hasListeners();
         nextErrorAction.setEnabled(hasErrors);
         prevErrorAction.setEnabled(hasErrors);
     }
@@ -668,7 +647,7 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
             setOutputPane(filtOut.getPane());
             try {
                 waitCursor(true);
-                filtOut.readFrom(io.out());
+                filtOut.readFrom(outWriter);
                 installKBActions();
             } finally {
                 waitCursor(false);
@@ -687,7 +666,7 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
     }
 
     OutWriter getOut() {
-        return origPane != null ? filtOut.getWriter() : io.out();
+        return origPane != null ? filtOut.getWriter() : outWriter;
     }
 
     private void disableHtmlName() {
@@ -721,8 +700,10 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
     private static final int ACTION_SMALLERFONT = 16;
     private static final int ACTION_FONTTYPE = 17;
     private static final int ACTION_FILTER = 18;
+    private static final int ACTION_PASTE = 19;
 
     Action copyAction = new TabAction(ACTION_COPY, "ACTION_COPY"); //NOI18N
+    Action pasteAction = new TabAction(ACTION_PASTE, "ACTION_PASTE"); //NOI18N
     Action wrapAction = new TabAction(ACTION_WRAP, "ACTION_WRAP"); //NOI18N
     Action saveAsAction = new TabAction(ACTION_SAVEAS, "ACTION_SAVEAS"); //NOI18N
     Action closeAction = new TabAction(ACTION_CLOSE, "ACTION_CLOSE"); //NOI18N
@@ -747,7 +728,7 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
     Action prevTabAction = new TabAction(ACTION_PREVTAB, "PreviousViewAction", //NOI18N
             (KeyStroke) null);
     private Object[] popupItems = new Object[]{
-        copyAction, new JSeparator(), findAction, findNextAction, findPreviousAction, filterAction, new JSeparator(),
+        copyAction, pasteAction, new JSeparator(), findAction, findNextAction, findPreviousAction, filterAction, new JSeparator(),
         wrapAction, largerFontAction, smallerFontAction, fontTypeAction, new JSeparator(), saveAsAction,
         clearAction, closeAction,};
 
@@ -825,6 +806,9 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
                 case ACTION_COPY:
                     getOutputPane().copy();
                     break;
+                case ACTION_PASTE:
+                    getOutputPane().paste();
+                    break;
                 case ACTION_WRAP:
                     boolean wrapped = getOutputPane().isWrapped();
                     getOutputPane().setWrapped(!wrapped);
@@ -861,7 +845,7 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
                     find(true);
                     break;
                 case ACTION_NAVTOLINE:
-                    openLineIfError();
+                    openHyperlink();
                     break;
                 case ACTION_POSTMENU:
                     postPopupMenu(new Point(0, 0), OutputTab.this);
@@ -870,8 +854,11 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
                     NbWriter writer = io.writer();
                     if (writer != null) {
                         try {
+                            boolean vis = isInputVisible();
+                            boolean closed = io.isStreamClosed();
                             writer.reset();
-                            disableHtmlName();
+                            setInputVisible(vis);
+                            io.setStreamClosed(closed);
                         } catch (IOException ioe) {
                             Exceptions.printStackTrace(ioe);
                         }
@@ -903,6 +890,12 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
         }
     }
 
+    @Override
+    public void setInputVisible(boolean val) {
+        super.setInputVisible(val);
+        pasteAction.setEnabled(val);
+    }
+
     private boolean validRegExp(String pattern) {
         try {
             Pattern.compile(pattern);
@@ -1054,18 +1047,8 @@ final class OutputTab extends AbstractOutputTab implements IOContainer.CallBacks
                     if (!passFilter(str)) {
                         continue;
                     }
-                    OutputListener l = lines.getListenerForLine(line);
-                    Color c = lines.getLineColor(line);
-                    boolean imp = lines.isImportantHyperlink(line);
-                    boolean err = lines.isErr(line);
-                    if (l != null || c != null) {
-                        out.print(str, l, imp, c, false);
-                    } else {
-                        out.print(str);
-                    }
-                    if (err) {
-                        ((AbstractLines) out.getLines()).markErr();
-                    }
+                    LineInfo info = lines.getExistingLineInfo(line);
+                    out.print(str, info, lines.isImportantLine(line));
                 } catch (IOException ex) {
                     Exceptions.printStackTrace(ex);
                 }
