@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -109,9 +112,8 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
     }
 
     public void setViewFont (Font f) {
-        fontWidth = -1;
-        fontHeight = -1;
         textView.setFont(f);
+        updateFont(getGraphics());
     }
 
     protected abstract JEditorPane createTextView();
@@ -130,7 +132,7 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
             recentlyReset = false;
         }
         if (locked) {
-            setMouseLine(-1);
+            resetCursor();
         }
         if (isWrapped()) {
             //Saves having OutputEditorKit have to do its own listening
@@ -259,6 +261,9 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         }
     }
 
+    /**
+     * Accessed reflectively from org.netbeans.jellytools.OutputTabOperator.
+     */
     public final Document getDocument() {
         return textView.getDocument();
     }
@@ -278,6 +283,10 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         }
     }
 
+    public final void paste() {
+        textView.paste();
+    }
+
     protected void setDocument (Document doc) {
         if (hasSelection()) {
             hasSelectionChanged(false);
@@ -342,6 +351,23 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         }
     }
 
+    public final void sendCaretToPos(int startPos, int endPos, boolean select) {
+        inSendCaretToLine = true;
+        getCaret().setVisible(true);
+        getCaret().setSelectionVisible(true);
+        if (select) {
+            scrollTo(endPos);
+            getCaret().setDot(endPos);
+            getCaret().moveDot(startPos);
+            textView.repaint();
+        } else {
+            getCaret().setDot(startPos);
+        }
+
+        locked = false;
+        inSendCaretToLine = false;
+    }
+
     private boolean inSendCaretToLine = false;
     private int lineToScroll = -1;
     public final boolean sendCaretToLine(int idx, boolean select) {
@@ -357,7 +383,6 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         if (select) {
             getCaret().setDot(el.getEndOffset() - 1);
             getCaret().moveDot(position);
-            getCaret().setSelectionVisible(true);
             textView.repaint();
         } else {
             getCaret().setDot(position);
@@ -370,7 +395,7 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         inSendCaretToLine = false;
         return true;
     }
-    
+
     boolean scrollToLine(int line) {
         int lineIdx = Math.min(getLineCount() - 1, line);
         Rectangle rect = null;
@@ -392,6 +417,25 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         return line == lineIdx && visRect.y + visRect.height == rect.y + rect.height;
     }
 
+    boolean scrollToPos(int pos) {
+        Rectangle rect = null;
+        try {
+            rect = textView.modelToView(pos);
+        } catch (BadLocationException ex) {
+            Exceptions.printStackTrace(ex);
+        }
+
+        if (rect == null) {
+            return false;
+        }
+
+        boolean oldLocked = locked;
+        textView.scrollRectToVisible(rect);
+        locked = oldLocked;
+
+        return true;
+    }
+
     public final void lockScroll() {
         if (!locked) {
             locked = true;
@@ -405,9 +449,9 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         lineToScroll = -1;
     }
 
-    protected abstract void caretEnteredLine (int line);
+    protected abstract void caretPosChanged(int pos);
     
-    protected abstract void lineClicked (int line, Point p);
+    protected abstract void lineClicked (int line, int pos);
     
     protected abstract void postPopupMenu (Point p, Component src);
     
@@ -432,13 +476,22 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
     @Override
     public final void paint (Graphics g) {
         if (fontHeight == -1) {
-            fontHeight = g.getFontMetrics(textView.getFont()).getHeight();
-            fontWidth = g.getFontMetrics(textView.getFont()).charWidth('m'); //NOI18N
-            getVerticalScrollBar().setUnitIncrement(fontHeight);
+            updateFont(g);
         }
         super.paint(g);
     }
 
+    void updateFont(Graphics g) {
+        if (g == null) {
+            fontHeight = fontWidth = -1;
+            return;
+        }
+        fontHeight = g.getFontMetrics(textView.getFont()).getHeight();
+        fontWidth = g.getFontMetrics(textView.getFont()).charWidth('m'); //NOI18N
+        getVerticalScrollBar().setUnitIncrement(fontHeight);
+        getHorizontalScrollBar().setUnitIncrement(fontWidth);
+    }
+
 //***********************Listener implementations*****************************
 
     public void stateChanged(ChangeEvent e) {
@@ -461,13 +514,6 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         }
     }
 
-    private boolean caretLineChanged() {
-        int line = getCaretLine();
-        boolean result = line != lastCaretLine;
-        lastCaretLine = line;
-        return result;
-    }
-
     private void maybeSendCaretEnteredLine() {
         if (EventQueue.getCurrentEvent() instanceof MouseEvent) {
             //User may have clicked a hyperlink, in which case, we'll test
@@ -479,10 +525,10 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         //the selection, or if the caret moved because output was written - 
         //it can cause the controller to send events to OutputListeners which
         //should only happen for user events
-        if ((!locked && caretLineChanged()) && !inSendCaretToLine) {
+        if (!locked && !inSendCaretToLine) {
             boolean sel = textView.getSelectionStart() != textView.getSelectionEnd();
             if (!sel) {
-                caretEnteredLine(getCaretLine());
+                caretPosChanged(getCaretPos());
             }
             if (sel != hadSelection) {
                 hadSelection = sel;
@@ -539,50 +585,47 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
     }
 
     public void mouseExited(MouseEvent e) {
-        setMouseLine (-1);
+        resetCursor();
     }
 
-    private int mouseLine = -1;
-    public void setMouseLine (int line, Point p) {
-        if (mouseLine != line) {
-            mouseLine = line;
-        }
-    }
-    
-    public final void setMouseLine (int line) {
-        setMouseLine (line, null);
-    }
-
-
-    public void mouseMoved(MouseEvent e) {
-        Point p = e.getPoint();
-        int pos = textView.viewToModel(p);
-        if (pos < getLength()) {
+    boolean isOnHyperlink(Point p) {
+        Document doc = getDocument();
+        if (doc instanceof OutputDocument) {
+            int pos = textView.viewToModel(p);
+            if (pos >= getLength()) {
+                return false;
+            }
             int line = getDocument().getDefaultRootElement().getElementIndex(pos);
             int lineStart = getDocument().getDefaultRootElement().getElement(line).getStartOffset();
-            int lineLength = getDocument().getDefaultRootElement().getElement(line).getEndOffset() -
-                    lineStart;
-
+            int lineLength = getDocument().getDefaultRootElement().getElement(line).getEndOffset() - lineStart;
             try {
-                Rectangle r = textView.modelToView(lineStart + lineLength -1);
-                int maxX = r.x + r.width;
-                boolean inLine = p.x <= maxX;
-                if (isWrapped()) {
-                    Rectangle ra = textView.modelToView(lineStart);
-                    if (ra.y <= r.y) {
-                        if (p.y < r.y) {
-                            inLine = true;
-                        }
-                    }
+                Rectangle r = textView.modelToView(lineStart + lineLength - 1);
+                boolean onLine = p.x <= r.x + r.width || (isWrapped() && p.y < r.y);
+                if (onLine) {
+                    return ((OutputDocument) doc).getLines().getListener(pos, null) != null;
                 }
-                
-                if (inLine) {
-                    setMouseLine (line, p);
-                } else {
-                    setMouseLine(-1);
+            } catch (BadLocationException ex) {
+            }
+        }
+        return false;
+    }
+
+    private void resetCursor() {
+        Cursor txtCursor = Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR);
+        if (textView.getCursor() != txtCursor) {
+            textView.setCursor(txtCursor);
+        }
+    }
+
+    public void mouseMoved(MouseEvent e) {
+        if (e.getSource() == textView) {
+            if (isOnHyperlink(e.getPoint())) {
+                Cursor hand = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
+                if (textView.getCursor() != hand) {
+                    textView.setCursor(hand);
                 }
-            } catch (BadLocationException ble) {
-                setMouseLine(-1);
+            } else {
+                resetCursor();
             }
         }
     }
@@ -630,7 +673,7 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
             if (pos != -1 && pos == lastPressedPos) {
                 int line = textView.getDocument().getDefaultRootElement().getElementIndex(pos);
                 if (line >= 0) {
-                    lineClicked(line, e.getPoint());
+                    lineClicked(line,pos);
                     e.consume(); //do NOT allow this window's caret to steal the focus from editor window
                 }
             }
