@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -110,7 +113,6 @@ abstract class ParsingLayerCacheManager extends LayerCacheManager implements Con
     // By path; attrs as folder/file path plus "//" plus attr name.
     private Set<String> oneLayerFiles; // Set<String>
     // Related:
-    private boolean checkingForDuplicates;
     private String currPath;
     private boolean atLeastOneFileOrFolderInLayer;
 
@@ -154,19 +156,16 @@ abstract class ParsingLayerCacheManager extends LayerCacheManager implements Con
             // #23609: reverse these...
             urls = new ArrayList<URL>(urls);
             Collections.reverse(urls);
-            checkingForDuplicates = LayerCacheManager.err.isLoggable(Level.FINE);
             Iterator<URL> it = urls.iterator();
             while (it.hasNext()) {
                 base = it.next(); // store base for resolving in parser
-                if (checkingForDuplicates) {
-                    oneLayerFiles = new HashSet<String>(100);
-                    currPath = null;
-                }
+                oneLayerFiles = new HashSet<String>(100);
+                currPath = null;
                 LayerCacheManager.err.log(Level.FINE, "Parsing: {0}", base);
                 atLeastOneFileOrFolderInLayer = false;
                 try {
                     r.parse(base.toExternalForm());
-                    if (!atLeastOneFileOrFolderInLayer) {
+                    if (!atLeastOneFileOrFolderInLayer && root.attrs == null) {
                         LayerCacheManager.err.log(Level.WARNING, "Inefficient to include an empty layer in a module: {0}", base);
                     }
                 } catch (Exception e) {
@@ -220,7 +219,11 @@ abstract class ParsingLayerCacheManager extends LayerCacheManager implements Con
         } else if (qname.equals("folder")) {
             fileOrFolder(qname, attrs);
         } else if (qname.equals("file")) {
-            MemFile file = (MemFile)fileOrFolder(qname, attrs);
+            MemFileOrFolder mfof = fileOrFolder(qname, attrs);
+            if (!(mfof instanceof MemFile)) { // a collision between modules
+                throw new ClassCastException("Stack: " + curr); // NOI18N
+            }
+            MemFile file = (MemFile)mfof;
             file.contents = null;
             String u = attrs.getValue("url");
             if (u != null) {
@@ -267,15 +270,15 @@ abstract class ParsingLayerCacheManager extends LayerCacheManager implements Con
             if (attr.type == null) throw new SAXParseException("unknown <attr> value type for " + attr.name, locator);
             MemFileOrFolder parent = (MemFileOrFolder)curr.peek();
             if (parent.attrs == null) parent.attrs = new LinkedList<MemAttr>();
-            Iterator it = parent.attrs.iterator();
+            Iterator<MemAttr> it = parent.attrs.iterator();
             while (it.hasNext()) {
-                if (((MemAttr)it.next()).name.equals(attr.name)) {
+                if (it.next().name.equals(attr.name)) {
                     attrCount--;
                     it.remove();
                 }
             }
             parent.attrs.add(attr);
-            if (checkingForDuplicates && !oneLayerFiles.add(currPath + "//" + attr.name)) { // NOI18N
+            if (!oneLayerFiles.add(currPath + "//" + attr.name)) { // NOI18N
                 LayerCacheManager.err.warning("layer " + base + " contains duplicate attributes " + attr.name + " for " + currPath);
             }
         } else {
@@ -294,11 +297,10 @@ abstract class ParsingLayerCacheManager extends LayerCacheManager implements Con
             parent.children = new LinkedList<MemFileOrFolder>();
         }
         else {
-            Iterator it = parent.children.iterator();
-            while (it.hasNext()) {
-                MemFileOrFolder f2 = (MemFileOrFolder)it.next();
+            for (MemFileOrFolder f2 : parent.children) {
                 if (f2.name.equals(name)) {
                     f = f2;
+                    f.registerURL(base);
                     break;
                 }
             }
@@ -315,15 +317,13 @@ abstract class ParsingLayerCacheManager extends LayerCacheManager implements Con
             parent.children.add(f);
         }
         curr.push(f);
-        if (checkingForDuplicates) {
-            if (currPath == null) {
-                currPath = name;
-            } else {
-                currPath += "/" + name;
-            }
-            if (!oneLayerFiles.add(currPath)) {
-                LayerCacheManager.err.warning("layer " + base + " contains duplicate " + qname + "s named " + currPath);
-            }
+        if (currPath == null) {
+            currPath = name;
+        } else {
+            currPath += "/" + name;
+        }
+        if (!oneLayerFiles.add(currPath)) {
+            LayerCacheManager.err.warning("layer " + base + " contains duplicate " + qname + "s named " + currPath);
         }
         return f;
     }
@@ -370,13 +370,11 @@ abstract class ParsingLayerCacheManager extends LayerCacheManager implements Con
         }
         if (qname.equals("file") || qname.equals("folder")) { // NOI18N
             curr.pop();
-            if (checkingForDuplicates) {
-                int i = currPath.lastIndexOf('/'); // NOI18N
-                if (i == -1) {
-                    currPath = null;
-                } else {
-                    currPath = currPath.substring(0, i);
-                }
+            int i = currPath.lastIndexOf('/'); // NOI18N
+            if (i == -1) {
+                currPath = null;
+            } else {
+                currPath = currPath.substring(0, i);
             }
         }
     }
@@ -438,11 +436,45 @@ abstract class ParsingLayerCacheManager extends LayerCacheManager implements Con
     protected static abstract class MemFileOrFolder {
         public String name;
         public List<MemAttr> attrs = null; // {null | List<MemAttr>}
-        public final URL base;
+        private Object base;
         
         public MemFileOrFolder (URL base) {
             this.base = base;
         }
+
+        @SuppressWarnings("unchecked")
+        final URL getBase() {
+            Object o = base;
+            if (o instanceof URL) {
+                return (URL)o;
+            } else {
+                return (URL)((Iterable)o).iterator().next();
+            }
+        }
+
+        @SuppressWarnings("unchecked")
+        final List<URL> getURLs() {
+            Object o = base;
+            if (o instanceof URL) {
+                return Collections.singletonList((URL)o);
+            } else {
+                return o != null ? (List<URL>)o : Collections.<URL>emptyList();
+            }
+        }
+
+        @SuppressWarnings("unchecked")
+        final void registerURL(URL url) {
+            List<URL> urls;
+            if (base instanceof URL) {
+                URL u = (URL)base;
+                base = urls = new ArrayList<URL>();
+                urls.add(u);
+            } else {
+                urls = (List<URL>)base;
+            }
+            urls.add(url);
+        }
+
     }
     
     /** Struct for <folder>.
