@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -169,7 +172,7 @@ class ModeParser {
     private void readTCRefs (ModeConfig mc) throws IOException {
         if (DEBUG) Debug.log(ModeParser.class, "readTCRefs ENTER" + " mo:" + getName());
         
-        for (Iterator it = tcRefParserMap.keySet().iterator(); it.hasNext(); ) {
+        for (Iterator<String> it = tcRefParserMap.keySet().iterator(); it.hasNext(); ) {
             TCRefParser tcRefParser = tcRefParserMap.get(it.next());
             tcRefParser.setInModuleFolder(false);
             tcRefParser.setInLocalFolder(false);
@@ -444,7 +447,7 @@ class ModeParser {
         }
         //if (DEBUG) Debug.log(ModeParser.class, "writeTCRefs" + " localFolder:" + localFolder);
         
-        for (Iterator it = tcRefParserMap.keySet().iterator(); it.hasNext(); ) {
+        for (Iterator<String> it = tcRefParserMap.keySet().iterator(); it.hasNext(); ) {
             tcRefParser = tcRefParserMap.get(it.next());
             tcRefParser.setLocalParentFolder(localFolder);
             tcRefParser.setInLocalFolder(true);
@@ -682,7 +685,7 @@ class ModeParser {
         
         // Update order
         List<TCRefParser> localList = new ArrayList<TCRefParser>(10);
-        Map localMap = (Map) ((HashMap) tcRefParserMap).clone();
+        Map<String,TCRefParser> localMap = (Map) ((HashMap) tcRefParserMap).clone();
         
         TCRefParser [] tcRefParserArray = new TCRefParser[tcRefOrder.size()];
         for (Iterator it = tcRefOrder.entrySet().iterator(); it.hasNext(); ) {
@@ -690,6 +693,7 @@ class ModeParser {
             String name = (String) en.getKey();
             int index = ((Integer) en.getValue()).intValue();
             tcRefParser = (TCRefParser) localMap.remove(name);
+            assert tcRefParser != null : "No parser for " + name + " in " + tcRefParser;
             //Put instances to array according to defined order
             //Order should be defined from 0 to N-1
             //log("-- -- ADD [" + index + "]: " + tcRefParser.getName());
@@ -699,8 +703,10 @@ class ModeParser {
             localList.add(tcRefParserArray[i]);
         }
         //Append remaining instances if any
-        for (Iterator it = localMap.keySet().iterator(); it.hasNext(); ) {
-            tcRefParser = (TCRefParser) localMap.get(it.next());
+        for (Iterator<String> it = localMap.keySet().iterator(); it.hasNext(); ) {
+            String key = it.next();
+            tcRefParser = localMap.get(key);
+            assert tcRefParser != null : "No parser for " + key;
             localList.add(tcRefParser);
         }
         
