@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -97,7 +100,7 @@ public final class ToolbarConfiguration implements ToolbarPool.Configuration {
 
     private final List<ToolbarRow> rows;
 
-    private final DnDSupport dndSupport;
+    private DnDSupport dndSupport;
 
     private Saver saver;
 
@@ -118,8 +121,13 @@ public final class ToolbarConfiguration implements ToolbarPool.Configuration {
 //        toolbarPanel.setOpaque(false);
 
         this.rows = new ArrayList<ToolbarRow>(rows);
+    }
 
-        dndSupport = new DnDSupport( this );
+    private synchronized DnDSupport dndSupport() {
+        if (dndSupport == null) {
+            dndSupport = new DnDSupport(this);
+        }
+        return dndSupport;
     }
 
     /** Finds toolbar configuration which has given name.
@@ -275,7 +283,7 @@ public final class ToolbarConfiguration implements ToolbarPool.Configuration {
                     continue;
                 Toolbar tb = bars.get(tc.getName());
                 if( null != tb ) {
-                    ToolbarContainer container = new ToolbarContainer( tb, dndSupport, tc.isDraggable() );
+                    ToolbarContainer container = new ToolbarContainer( tb, dndSupport(), tc.isDraggable() );
                     row.add( tc.getName(), container );
                 }
             }
@@ -426,7 +434,7 @@ public final class ToolbarConfiguration implements ToolbarPool.Configuration {
      * @param buttonDndAllowed
      */
     void setToolbarButtonDragAndDropAllowed(boolean buttonDndAllowed) {
-        dndSupport.setButtonDragAndDropAllowed(buttonDndAllowed);
+        dndSupport().setButtonDragAndDropAllowed(buttonDndAllowed);
     }
 
     /**
