@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -46,14 +49,13 @@ import java.awt.EventQueue;
 import java.awt.event.ActionEvent;
 import java.awt.event.KeyEvent;
 import javax.swing.KeyStroke;
-import javax.swing.text.Keymap;
 import javax.swing.tree.TreePath;
 //import org.netbeans.core.NbKeymap;
 
 import org.netbeans.jellytools.JellyVersion;
 import org.netbeans.jellytools.MainWindowOperator;
 import org.netbeans.jellytools.nodes.Node;
-import org.netbeans.jemmy.ClassReference;
+import org.netbeans.jellytools.nodes.OutlineNode;
 import org.netbeans.jemmy.EventTool;
 import org.netbeans.jemmy.JemmyException;
 import org.netbeans.jemmy.JemmyProperties;
@@ -67,7 +69,6 @@ import org.netbeans.jemmy.operators.Operator.ComponentVisualizer;
 import org.netbeans.jemmy.operators.Operator.DefaultStringComparator;
 import org.netbeans.jemmy.operators.Operator.StringComparator;
 import org.netbeans.jemmy.util.EmptyVisualizer;
-import org.openide.util.Lookup;
 import org.openide.util.actions.SystemAction;
 
 
@@ -119,10 +120,7 @@ public class Action {
     protected String popupPath;
     /** SystemAction class of current action or null when API_MODE is not supported */    
     protected Class systemActionClass;
-    /** array of shortcuts of current action or null when SHORTCUT_MODE is not supported.
-     * @deprecated Use {@link #keystrokes} instead
-    */
-    protected Shortcut[] shortcuts;
+
     /** Array of key strokes or null when SHORTCUT_MODE is not supported. */
     protected KeyStroke[] keystrokes;
 
@@ -249,8 +247,10 @@ public class Action {
             } catch (ClassNotFoundException e2) {
                 this.systemActionClass = null;
             }
-        }            
-        this.keystrokes = keystrokes;
+        }
+
+        if (keystrokes != null)
+            this.keystrokes = keystrokes.clone();
     }
 
     
@@ -259,7 +259,7 @@ public class Action {
         JellyVersion.checkJemmyVersion();
 
         if (JemmyProperties.getCurrentProperty("Action.DefaultMode")==null)
-            JemmyProperties.setCurrentProperty("Action.DefaultMode", new Integer(POPUP_MODE));
+            JemmyProperties.setCurrentProperty("Action.DefaultMode", Integer.valueOf(POPUP_MODE));
         Timeouts.initDefault("Action.WaitAfterShortcutTimeout", WAIT_AFTER_SHORTCUT_TIMEOUT);
         // Set case sensitive comparator as default because of 
         // very often clash between Cut and Execute menu items.
@@ -267,7 +267,9 @@ public class Action {
         boolean compareExactly = !Operator.getDefaultStringComparator().equals("abc", "a"); // NOI18N
         defaultComparator = new DefaultStringComparator(compareExactly, true);
     }
-    
+
+    //This is to suppress the FindBugs warning that all targets throw UnsupportedOperationException
+    @org.netbeans.api.annotations.common.SuppressWarnings("DMI_UNSUPPORTED_METHOD")
     private void perform(int mode) {
         switch (mode) {
             case POPUP_MODE: performPopup(); break;
@@ -297,6 +299,16 @@ public class Action {
             default: throw new IllegalArgumentException("Wrong Action.MODE");
         }
     }
+
+    private void perform(OutlineNode node, int mode) {
+        switch (mode) {
+            case POPUP_MODE: performPopup(node); break;
+            case MENU_MODE: performMenu(node); break;
+            case API_MODE: performAPI(node); break;
+            case SHORTCUT_MODE: performShortcut(node); break;
+            default: throw new IllegalArgumentException("Wrong Action.MODE");
+        }
+    }
             
     private void perform(ComponentOperator component, int mode) {
         switch (mode) {
@@ -344,6 +356,20 @@ public class Action {
         } catch (UnsupportedOperationException e) {}
     }
 
+    /**
+     * performs action depending on default mode,<br>
+     * calls performPopup(), performMenu(), performShortcut or performAPI(),<br>
+     * when default mode is not supported, others are tried
+     * @param node node to be action performed on
+     */
+    public void perform(OutlineNode node) {
+        int modes[] = sequence[getDefaultMode()];
+        for (int i=0; i<modes.length; i++) try {
+            perform(node, modes[i]);
+            return;
+        } catch (UnsupportedOperationException e) {}
+    }
+
     /** performs action depending on default mode,<br>
      * calls performPopup(), performMenu() or performAPI(),<br>
      * when default mode is not supported, others are tried
@@ -397,6 +423,18 @@ public class Action {
         }
         performMenu();
     }
+
+    public void performMenu(OutlineNode node) {
+        if (menuPath==null)
+            throw new UnsupportedOperationException(getClass().toString()+" does not define menu path");
+        node.select();
+        try {
+            Thread.sleep(SELECTION_WAIT_TIME);
+        } catch (Exception e) {
+            throw new JemmyException("Sleeping interrupted", e);
+        }
+        performMenu();
+    }
     
     /** performs action through main menu
      * @param component component to be action performed on
@@ -435,6 +473,24 @@ public class Action {
             throw new JemmyException("Sleeping interrupted", e);
         }
     }
+
+    /**
+     * Performs action on an OutlineNode through popup menu.
+     *
+     * @param node node to be performed on
+     */
+    public void performPopup(OutlineNode node)
+    {
+        if (popupPath==null) {
+            throw new UnsupportedOperationException(getClass().toString()+" does not define popup path");
+        }
+        node.callPopup().pushMenu(popupPath, "|", getComparator());
+        try {
+            Thread.sleep(AFTER_ACTION_WAIT_TIME);
+        } catch (Exception e) {
+            throw new JemmyException("Sleeping interrupted", e);
+        }
+    }
     
     /** Calls popup on given nodes and returns JPopupMenuOperator instance.
      * @param nodes nodes to be action performed on  
@@ -554,6 +610,21 @@ public class Action {
         }
         performAPI();
     }
+
+     /** performs action through API
+     * @param node node to be action performed on
+     * @throws UnsupportedOperationException when action does not support API mode */
+    public void performAPI(OutlineNode node) {
+        if (systemActionClass==null)
+            throw new UnsupportedOperationException(getClass().toString()+" does not define SystemAction");
+        node.select();
+        try {
+            Thread.sleep(SELECTION_WAIT_TIME);
+        } catch (Exception e) {
+            throw new JemmyException("Sleeping interrupted", e);
+        }
+        performAPI();
+    }
     
     /** performs action through API
      * @param component component to be action performed on
@@ -612,6 +683,23 @@ public class Action {
         }
         performShortcut();
     }
+
+    /** performs action through shortcut
+     * @param node node to be action performed on
+     * @throws UnsupportedOperationException when action does not support shortcut mode */
+    public void performShortcut(OutlineNode node) {
+        final KeyStroke[] strokes = getKeyStrokes();
+        if (strokes == null) {
+            throw new UnsupportedOperationException(getClass().toString()+" does not define shortcut");
+        }
+        node.select();
+        try {
+            Thread.sleep(SELECTION_WAIT_TIME);
+        } catch (Exception e) {
+            throw new JemmyException("Sleeping interrupted", e);
+        }
+        performShortcut();
+    }
     
     /** performs action through shortcut
      * @param component component to be action performed on
@@ -631,12 +719,9 @@ public class Action {
      * @throws IllegalArgumentException when given nodes does not pass */    
     protected void testNodes(Node[] nodes) {
         if ((nodes==null)||(nodes.length==0))
-            throw new IllegalArgumentException("argument nodes is null or empty");
-        Class myClass = getClass();
+            throw new IllegalArgumentException("argument nodes is null or empty");       
         Component nodesTree=nodes[0].tree().getSource();
         for (int i=0; i<nodes.length; i++) {
-//            if (!nodes[i].hasAction(myClass))
-//                throw new IllegalArgumentException(this.toString()+" could not be performed on "+nodes[i].toString());
             if (nodes[i]==null)
                 throw new IllegalArgumentException("argument nodes contains null value");
             if (!nodesTree.equals(nodes[i].tree().getSource()))
@@ -672,7 +757,7 @@ public class Action {
         if (mode<0 || mode>3) {
             mode = POPUP_MODE;
         }
-        JemmyProperties.setCurrentProperty("Action.DefaultMode", new Integer(mode));
+        JemmyProperties.setCurrentProperty("Action.DefaultMode", Integer.valueOf(mode));
         return oldMode;
     }
     
@@ -717,47 +802,16 @@ public class Action {
     public Class getSystemActionClass() {
         return systemActionClass;
     }
-    
-    /** getter for array of shortcuts
-     * @return Shortcut[] (or null if not suported)
-     * @deprecated Use {@link #getKeyStrokes} instead
-     */    
-    public Shortcut[] getShortcuts() {
-        return shortcuts;
-    }
-
-    /** Returns an array of KeyStroke objects.
-     * If systemActionClass is defined and no keystrokes defined in constructor,
-     * it tries to find key strokes in NbKeymap.
-     * Otherwise it returns keystrokes defined in constructor, which can be null.
-     * It helps to overcome differencies on platforms (e.g. on mac).
-     * @return an array of KeyStroke objects or null if not found.
+   
+    /** Returns an array of KeyStroke objects.     
+     * @return an array of KeyStroke objects or null if not defined.
      */
     public KeyStroke[] getKeyStrokes() {
-        if (this.keystrokes == null && systemActionClass != null) {
-            try {
-            //ClassReference keymapRef = new ClassReference(getClass().getClassLoader(), "org.netbeans.core.NbKeymap");
-            Lookup lu = Lookup.getDefault();
-            //ClassReference keymapRef = new ClassReference()
-            ClassReference keymapRef = new ClassReference(lu.lookup(Keymap.class));
-            //NbKeymap keymap = (NbKeymap) Lookup.getDefault().lookup(NbKeymap.class);
-            javax.swing.Action myAction = null;
-            javax.swing.Action[] actions = (javax.swing.Action[]) keymapRef.invokeMethod("getBoundActions", new Object[0], new Class[0]);
-            //javax.swing.Action[] actions = keymap.getBoundActions();
-            for (int i = 0; i < actions.length; i++) {
-                if(actions[i].getClass().equals(systemActionClass)) {
-                    // Returns just first keystroke. For actions like copy can 
-                    // can exists more than one (Ctrl+C, Ctrl+Insert, Copy key).
-                    KeyStroke[] strokes = (KeyStroke[]) keymapRef.invokeMethod("getKeyStrokesForAction", new Object[] {actions[i]}, new Class[] {javax.swing.Action.class});
-                    return new KeyStroke[] {strokes[0]};
-                    //return new KeyStroke[] {keymap.getKeyStrokesForAction(actions[i])[0]};
-                }
-            }
-            } catch (Exception e) {
-                throw new JemmyException("Exception while gettting keystrokes", e);
-            }
+        if (keystrokes != null){
+            return keystrokes.clone();
         }
-        return this.keystrokes;
+
+        return null;
     }
     
     /** Checks whether this action is enabled. If IDE system action class
