@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -40,7 +43,11 @@
  */
 package org.netbeans.jellytools;
 
+import org.netbeans.jellytools.actions.Action;
+import org.netbeans.jemmy.operators.ComponentOperator;
+import org.netbeans.jemmy.operators.JCheckBoxOperator;
 import org.netbeans.jemmy.operators.JFileChooserOperator;
+import org.netbeans.jemmy.operators.JTabbedPaneOperator;
 import org.netbeans.junit.NbTest;
 
 /** Test PluginsOperator.
@@ -98,10 +105,23 @@ public class PluginsOperatorTest extends JellyTestCase {
     }
     private static PluginsOperator pluginsOper;
     
-    private static final String TEST_PLUGIN = "Clearcase"; //NOI18N
+    private static final String TEST_PLUGIN = "Archiver"; //NOI18N
 
     /** Test of invoke method. */
-    public void testInvoke() {
+    public void testInvoke() throws InterruptedException {
+
+        //Make sure the menu has time to load
+        new Action(Bundle.getStringTrimmed(
+            "org.netbeans.core.ui.resources.Bundle", "Menu/Tools"), null).performMenu();
+
+        Thread.sleep(1000);
+        
+
+        new Action(Bundle.getStringTrimmed("org.netbeans.modules.project.ui.Bundle",
+                "Menu/BuildProject"), null).performMenu();
+
+        Thread.sleep(1000);
+
         pluginsOper = PluginsOperator.invoke();
     }
 
@@ -110,8 +130,9 @@ public class PluginsOperatorTest extends JellyTestCase {
      * - type "netbeans.org Source Browser" into Search text field
      * - finish installation
      */
-    public void testInstall() {
+    public void testInstall() {        
         pluginsOper.selectAvailablePlugins();
+        pluginsOper.reloadCatalog();
         pluginsOper.search(TEST_PLUGIN);
         pluginsOper.install(TEST_PLUGIN);
     }
@@ -123,8 +144,9 @@ public class PluginsOperatorTest extends JellyTestCase {
      * - wait for "NetBeans IDE Installer" dialog
      * - click Cancel
      */
-    public void testUninstall() {
-        pluginsOper.selectInstalled();
+    public void testUninstall() throws Exception{
+        pluginsOper.cbShowDetails().setSelected(true);
+
         pluginsOper.selectPlugins(new String[]{
             "Java",
             TEST_PLUGIN
@@ -141,7 +163,7 @@ public class PluginsOperatorTest extends JellyTestCase {
      * - click Cancel
      */
     public void testDeactivate() {
-        pluginsOper.selectInstalled();
+        pluginsOper.cbShowDetails().setSelected(true);
         pluginsOper.selectPlugin(TEST_PLUGIN);
         pluginsOper.deactivate();
         pluginsOper.installer().cancel();
