@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -50,7 +53,6 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -69,15 +71,17 @@ import org.openide.filesystems.FileUtil;
 import org.openide.util.Exceptions;
 import org.openide.util.Mutex;
 import org.openide.util.Utilities;
+import org.openide.util.WeakSet;
 
 /**
  * @author Radek Matous
  */
 public final class FileObjectFactory {
-    public static Map AllFactories = new HashMap();
+    public static final Map<File, FileObjectFactory> AllFactories = new HashMap<File, FileObjectFactory>();
     public static boolean WARNINGS = true;
-    final Map allIBaseFileObjects = Collections.synchronizedMap(new WeakHashMap());
+    final Map<Integer, Object> allIBaseFileObjects = Collections.synchronizedMap(new WeakHashMap<Integer, Object>());
     private BaseFileObj root;
+    private static final Logger LOG_REFRESH = Logger.getLogger("org.netbeans.modules.masterfs.REFRESH"); // NOI18N
     public static enum Caller {
         ToFileObject, GetFileObject, GetChildern, GetParent, Others
     }
@@ -87,10 +91,6 @@ public final class FileObjectFactory {
     }
 
     private FileObjectFactory(final FileInfo fInfo) {
-        final File rootFile = fInfo.getFile();
-        // can be not null for UNC 
-        // assert rootFile.getParentFile() == null;
-
         final BaseFileObj realRoot = create(fInfo);
         root = realRoot;
     }
@@ -105,12 +105,12 @@ public final class FileObjectFactory {
         final File rootFile = rootInfo.getFile();
 
         synchronized (FileObjectFactory.AllFactories) {
-            retVal = (FileObjectFactory) FileObjectFactory.AllFactories.get(rootFile);
+            retVal = FileObjectFactory.AllFactories.get(rootFile);
         }
         if (retVal == null && addMising) {
             if (rootInfo.isConvertibleToFileObject()) {
                 synchronized (FileObjectFactory.AllFactories) {
-                    retVal = (FileObjectFactory) FileObjectFactory.AllFactories.get(rootFile);
+                    retVal = FileObjectFactory.AllFactories.get(rootFile);
                     if (retVal == null) {
                         retVal = new FileObjectFactory(rootFile);
                         FileObjectFactory.AllFactories.put(rootFile, retVal);
@@ -121,9 +121,9 @@ public final class FileObjectFactory {
         return retVal;
     }
 
-    public static Collection getInstances() {
+    public static Collection<FileObjectFactory> getInstances() {
         synchronized (FileObjectFactory.AllFactories) {
-            return new ArrayList(AllFactories.values());
+            return new ArrayList<FileObjectFactory>(AllFactories.values());
         }
     }
     
@@ -141,25 +141,25 @@ public final class FileObjectFactory {
     public int getSize() {
         int retval = 0;
 
-        List list = new ArrayList();
+        List<Object> list = new ArrayList<Object>();
         synchronized (allIBaseFileObjects) {
             list.addAll(allIBaseFileObjects.values());
         }
-        List list2 = new ArrayList();
+        List<Object> list2 = new ArrayList<Object>();
 
 
-        for (Iterator it = list.iterator(); it.hasNext();) {
-            Object obj = it.next();
-            if (obj instanceof Reference) {
+        for (Object obj : list) {
+            if (obj instanceof Reference<?>) {
                 list2.add(obj);
             } else {
-                list2.addAll((List) obj);
+                list2.addAll((List<?>) obj);
             }
         }
 
-        for (Iterator it = list2.iterator(); it.hasNext();) {
-            Reference ref = (Reference) it.next();
-            FileObject fo = (ref != null) ? (FileObject) ref.get() : null;
+        for (Iterator<Object> it = list2.iterator(); it.hasNext();) {
+            @SuppressWarnings("unchecked")
+            Reference<FileObject> ref = (Reference<FileObject>) it.next();
+            FileObject fo = (ref != null) ? ref.get() : null;
             if (fo != null) {
                 retval++;
             }
@@ -304,7 +304,11 @@ public final class FileObjectFactory {
             }
         } else {
             if (foForFile == null) {
-                exist = touchExists(file, realExists);
+                if (caller == Caller.GetParent) {
+                    exist = true;
+                } else {
+                    exist = touchExists(file, realExists);
+                }
             } else if (foForFile.isValid()) {
                 if (parent == null) {
                     exist = (realExists == -1) ? true : touchExists(file, realExists);
@@ -344,7 +348,7 @@ public final class FileObjectFactory {
                         exist = touchExists(file, realExists);
                         if (!exist) {
                             //parent is exception must be issued even if not valid
-                            ((BaseFileObj) retval).setValid(false);
+                            retval.setValid(false);
                         }
                     }
                     assert checkCacheState(exist, file, caller);
@@ -371,11 +375,11 @@ public final class FileObjectFactory {
         return (state == 1) ? true : false;
     }
 
-    private final BaseFileObj getOrCreate(final FileInfo fInfo) {
+    private BaseFileObj getOrCreate(final FileInfo fInfo) {
         BaseFileObj retVal = null;
         File f = fInfo.getFile();
 
-        boolean issue45485 = fInfo.isWindows() && f.getName().endsWith(".");//NOI18N        
+        boolean issue45485 = fInfo.isWindows() && f.getName().endsWith(".") && !f.getName().matches("[.]{1,2}");//NOI18N
         if (issue45485) {
             File f2 = FileUtil.normalizeFile(f);
             issue45485 = !f2.getName().endsWith(".");
@@ -425,54 +429,95 @@ public final class FileObjectFactory {
         return null;
     }
 
-    public final void refreshAll(final boolean expected) {
-        Set all2Refresh = collectForRefresh();
-        refresh(all2Refresh, expected);
+    final void refreshAll(RefreshSlow slow, final boolean expected) {
+        Set<BaseFileObj> all2Refresh = collectForRefresh();
+        refresh(all2Refresh, slow, expected);
     }
 
-    private Set collectForRefresh() {
-        final Set all2Refresh = new HashSet();
+    private Set<BaseFileObj> collectForRefresh() {
+        final Set<BaseFileObj> all2Refresh;
         synchronized (allIBaseFileObjects) {
-            final Iterator it = allIBaseFileObjects.values().iterator();
+            all2Refresh = new WeakSet<BaseFileObj>(allIBaseFileObjects.size() * 3);
+            final Iterator<Object> it = allIBaseFileObjects.values().iterator();
             while (it.hasNext()) {
                 final Object obj = it.next();
-                if (obj instanceof List) {
-                    for (Iterator iterator = ((List) obj).iterator(); iterator.hasNext();) {
-                        WeakReference ref = (WeakReference) iterator.next();
-                        final BaseFileObj fo = (BaseFileObj) ((ref != null) ? ref.get() : null);
+                if (obj instanceof List<?>) {
+                    for (Iterator<?> iterator = ((List<?>) obj).iterator(); iterator.hasNext();) {
+                        @SuppressWarnings("unchecked")
+                        WeakReference<BaseFileObj> ref = (WeakReference<BaseFileObj>) iterator.next();
+                        final BaseFileObj fo = (ref != null) ? ref.get() : null;
                         if (fo != null) {
                             all2Refresh.add(fo);
                         }
                     }
                 } else {
-                    final WeakReference ref = (WeakReference) obj;
-                    final BaseFileObj fo = (BaseFileObj) ((ref != null) ? ref.get() : null);
+                    @SuppressWarnings("unchecked")
+                    final WeakReference<BaseFileObj> ref = (WeakReference<BaseFileObj>) obj;
+                    final BaseFileObj fo = (ref != null) ? ref.get() : null;
                     if (fo != null) {
                         all2Refresh.add(fo);
                     }
                 }
             }
         }
+        all2Refresh.remove(root); // #182793
         return all2Refresh;
     }
 
-    private void refresh(final Set all2Refresh, File... files) {
-        for (Iterator iterator = all2Refresh.iterator(); iterator.hasNext();) {
-            final BaseFileObj fo = (BaseFileObj) iterator.next();
-            for (File file : files) {
-                if (isParentOf(file, fo.getFileName().getFile())) {
-                    fo.refresh(true);
-                    break;
-                }                
+    private boolean refresh(final Set<BaseFileObj> all2Refresh, RefreshSlow slow, File... files) {
+        return refresh(all2Refresh, slow, true, files);
+    }
+
+    private static boolean isInFiles(BaseFileObj fo, File[] files) {
+        if (fo == null) {
+            return false;
+        }
+        if (files == null) {
+            return true;
+        }
+        for (File file : files) {
+            if (isParentOf(file, fo.getFileName().getFile())) {
+                return true;
             }
         }
-    }    
-    
-    private void refresh(final Set all2Refresh, final boolean expected) {
-        for (Iterator iterator = all2Refresh.iterator(); iterator.hasNext();) {
-            final BaseFileObj fo = (BaseFileObj) iterator.next();
-            fo.refresh(expected);
+        return false;
+    }
+
+    private boolean refresh(final Set<BaseFileObj> all2Refresh, RefreshSlow slow, final boolean expected) {
+        return refresh(all2Refresh, slow, expected, null);
+    }
+
+    private boolean refresh(final Set<BaseFileObj> all2Refresh, RefreshSlow slow, final boolean expected, File[] files) {
+        int add = 0;
+        Iterator<BaseFileObj> it = all2Refresh.iterator();
+        while (it.hasNext()) {
+            BaseFileObj fo = null;
+            if (slow != null) {
+                BaseFileObj pref = slow.preferrable();
+                if (pref != null && all2Refresh.remove(pref)) {
+                    LOG_REFRESH.log(Level.FINER, "Preferring {0}", pref);
+                    fo = pref;
+                    it = all2Refresh.iterator();
+                }
+            }
+            if (fo == null) {
+                fo = it.next();
+                it.remove();
+            }
+            add++;
+            if (!isInFiles(fo, files)) {
+                continue;
+            }
+            if (slow != null) {
+                if (!slow.refreshFileObject(fo, expected, add)) {
+                    return false;
+                }
+                add = 0;
+            } else {
+                fo.refresh(expected);
+            }
         }
+        return true;
     }
 
     public static boolean isParentOf(final File dir, final File file) {
@@ -484,17 +529,18 @@ public final class FileObjectFactory {
     }
 
     public final void rename() {
-        final Map toRename = new HashMap();
+        final Map<Integer, Object> toRename = new HashMap<Integer, Object>();
         synchronized (allIBaseFileObjects) {
-            final Iterator it = allIBaseFileObjects.entrySet().iterator();
+            final Iterator<Map.Entry<Integer, Object>> it = allIBaseFileObjects.entrySet().iterator();
             while (it.hasNext()) {
-                Map.Entry entry = (Map.Entry) it.next();
+                Map.Entry<Integer, Object> entry = it.next();
                 final Object obj = entry.getValue();
-                final Integer key = (Integer) entry.getKey();
-                if (!(obj instanceof List)) {
-                    final WeakReference ref = (WeakReference) obj;
+                final Integer key = entry.getKey();
+                if (!(obj instanceof List<?>)) {
+                    @SuppressWarnings("unchecked")
+                    final WeakReference<BaseFileObj> ref = (WeakReference<BaseFileObj>) obj;
 
-                    final BaseFileObj fo = (BaseFileObj) ((ref != null) ? ref.get() : null);
+                    final BaseFileObj fo = (ref != null) ? ref.get() : null;
 
                     if (fo != null) {
                         Integer computedId = fo.getFileName().getId();
@@ -503,9 +549,10 @@ public final class FileObjectFactory {
                         }
                     }
                 } else {
-                    for (Iterator iterator = ((List) obj).iterator(); iterator.hasNext();) {
-                        WeakReference ref = (WeakReference) iterator.next();
-                        final BaseFileObj fo = (BaseFileObj) ((ref != null) ? ref.get() : null);
+                    for (Iterator<?> iterator = ((List<?>) obj).iterator(); iterator.hasNext();) {
+                        @SuppressWarnings("unchecked")
+                        WeakReference<BaseFileObj> ref = (WeakReference<BaseFileObj>) iterator.next();
+                        final BaseFileObj fo = (ref != null) ? ref.get() : null;
                         if (fo != null) {
                             Integer computedId = fo.getFileName().getId();
                             if (!key.equals(computedId)) {
@@ -517,12 +564,11 @@ public final class FileObjectFactory {
                 }
             }
 
-            for (Iterator iterator = toRename.entrySet().iterator(); iterator.hasNext();) {
-                final Map.Entry entry = (Map.Entry) iterator.next();
-                Object key = entry.getKey();
+            for (Map.Entry<Integer, Object> entry : toRename.entrySet()) {
+                Integer key = entry.getKey();
                 Object previous = allIBaseFileObjects.remove(key);
-                if (previous instanceof List) {
-                    List list = (List) previous;
+                if (previous instanceof List<?>) {
+                    List<?> list = (List<?>) previous;
                     list.remove(entry.getValue());
                     allIBaseFileObjects.put(key, previous);
                 } else {
@@ -537,9 +583,9 @@ public final class FileObjectFactory {
         final Object o;
         synchronized (allIBaseFileObjects) {
             final Object value = allIBaseFileObjects.get(NamingFactory.createID(file));
-            Reference ref = null;
-            ref = (Reference) (value instanceof Reference ? value : null);
-            ref = (ref == null && value instanceof List ? FileObjectFactory.getReference((List) value, file) : ref);
+            @SuppressWarnings("unchecked")
+            Reference<BaseFileObj> ref = (Reference<BaseFileObj>) (value instanceof Reference<?> ? value : null);
+            ref = (ref == null && value instanceof List<?> ? FileObjectFactory.getReference((List<?>) value, file) : ref);
 
             o = (ref != null) ? ref.get() : null;
             assert (o == null || o instanceof BaseFileObj);
@@ -555,11 +601,12 @@ public final class FileObjectFactory {
         return retval;
     }
 
-    private static Reference getReference(final List list, final File file) {
-        Reference retVal = null;
+    private static Reference<BaseFileObj> getReference(final List<?> list, final File file) {
+        Reference<BaseFileObj> retVal = null;
         for (int i = 0; retVal == null && i < list.size(); i++) {
-            final Reference ref = (Reference) list.get(i);
-            final BaseFileObj cachedElement = (ref != null) ? (BaseFileObj) ref.get() : null;
+            @SuppressWarnings("unchecked")
+            final Reference<BaseFileObj> ref = (Reference<BaseFileObj>) list.get(i);
+            final BaseFileObj cachedElement = (ref != null) ? ref.get() : null;
             if (cachedElement != null && cachedElement.getFileName().getFile().compareTo(file) == 0) {
                 retVal = ref;
             }
@@ -569,20 +616,23 @@ public final class FileObjectFactory {
 
     private BaseFileObj putInCache(final BaseFileObj newValue, final Integer id) {
         synchronized (allIBaseFileObjects) {
-            final WeakReference newRef = new WeakReference(newValue);
+            final WeakReference<BaseFileObj> newRef = new WeakReference<BaseFileObj>(newValue);
             final Object listOrReference = allIBaseFileObjects.put(id, newRef);
 
             if (listOrReference != null) {
-                if (listOrReference instanceof List) {
-                    ((List) listOrReference).add(newRef);
+                if (listOrReference instanceof List<?>) {
+                    @SuppressWarnings("unchecked")
+                    List<Reference<BaseFileObj>> list = (List<Reference<BaseFileObj>>) listOrReference;
+                    list.add(newRef);
                     allIBaseFileObjects.put(id, listOrReference);
                 } else {
-                    assert (listOrReference instanceof WeakReference);
-                    final Reference oldRef = (Reference) listOrReference;
-                    BaseFileObj oldValue = (oldRef != null) ? (BaseFileObj) oldRef.get() : null;
+                    assert (listOrReference instanceof WeakReference<?>);
+                    @SuppressWarnings("unchecked")
+                    final Reference<BaseFileObj> oldRef = (Reference<BaseFileObj>) listOrReference;
+                    BaseFileObj oldValue = (oldRef != null) ? oldRef.get() : null;
 
                     if (oldValue != null && !newValue.getFileName().equals(oldValue.getFileName())) {
-                        final List l = new ArrayList();
+                        final List<Reference<BaseFileObj>> l = new ArrayList<Reference<BaseFileObj>>();
                         l.add(oldRef);
                         l.add(newRef);
                         allIBaseFileObjects.put(id, l);
@@ -596,20 +646,19 @@ public final class FileObjectFactory {
 
     @Override
     public String toString() {
-        List list = new ArrayList();
+        List<Object> list = new ArrayList<Object>();
         synchronized (allIBaseFileObjects) {
             list.addAll(allIBaseFileObjects.values());
         }
-        List l2 = new ArrayList();
-        for (Iterator it = list.iterator(); it.hasNext();) {
-            Reference ref = (Reference) it.next();
-            FileObject fo = (ref != null) ? (FileObject) ref.get() : null;
+        List<String> l2 = new ArrayList<String>();
+        for (Iterator<Object> it = list.iterator(); it.hasNext();) {
+            @SuppressWarnings("unchecked")
+            Reference<FileObject> ref = (Reference<FileObject>) it.next();
+            FileObject fo = (ref != null) ? ref.get() : null;
             if (fo != null) {
                 l2.add(fo.getPath());
             }
         }
-
-
         return l2.toString();
     }
 
@@ -623,7 +672,7 @@ public final class FileObjectFactory {
 
     //only for tests purposes
     public static void reinitForTests() {
-        FileObjectFactory.AllFactories = new HashMap();
+        FileObjectFactory.AllFactories.clear();
     }
 
 
@@ -633,33 +682,49 @@ public final class FileObjectFactory {
         return (retVal != null && retVal.isValid()) ? retVal : null;
     }
 
-    public final void refresh(final boolean expected) {
+    public void refresh(boolean expected) {
+        refresh(null, expected);
+    }
+    final void refresh(final RefreshSlow slow, final boolean expected) {
         Statistics.StopWatch stopWatch = Statistics.getStopWatch(Statistics.REFRESH_FS);
         final Runnable r = new Runnable() {
+            @Override
             public void run() {
-                refreshAll(expected);                
+                refreshAll(slow, expected);
             }            
         };        
         
         stopWatch.start();
         try {
-            FileBasedFileSystem.getInstance().runAtomicAction(new FileSystem.AtomicAction() {
-                public void run() throws IOException {
-                    FileBasedFileSystem.runAsInconsistent(r);
-                }
-            });
+            if (slow != null) {
+                FileBasedFileSystem.runAsInconsistent(r);
+            } else {
+                FileBasedFileSystem.getInstance().runAtomicAction(new FileSystem.AtomicAction() {
+                    @Override
+                    public void run() throws IOException {
+                        FileBasedFileSystem.runAsInconsistent(r);
+                    }
+                });
+            }
         } catch (IOException iex) {/*method refreshAll doesn't throw IOException*/
 
         }
         stopWatch.stop();
 
         // print refresh stats unconditionally in trunk
-        Logger.getLogger("org.netbeans.modules.masterfs.REFRESH").fine(
-                "FS.refresh statistics (" + Statistics.fileObjects() + "FileObjects):\n  " +
-                Statistics.REFRESH_FS.toString() + "\n  " +
-                Statistics.LISTENERS_CALLS.toString() + "\n  " +
-                Statistics.REFRESH_FOLDER.toString() + "\n  " +
-                Statistics.REFRESH_FILE.toString() + "\n");
+        if (LOG_REFRESH.isLoggable(Level.FINE)) {
+            LOG_REFRESH.log(
+                Level.FINE,
+                "FS.refresh statistics ({0}FileObjects):\n  {1}\n  {2}\n  {3}\n  {4}\n",
+                new Object[]{
+                    Statistics.fileObjects(),
+                    Statistics.REFRESH_FS.toString(),
+                    Statistics.LISTENERS_CALLS.toString(),
+                    Statistics.REFRESH_FOLDER.toString(),
+                    Statistics.REFRESH_FILE.toString()
+                }
+            );
+        }
 
         Statistics.REFRESH_FS.reset();
         Statistics.LISTENERS_CALLS.reset();
@@ -667,33 +732,52 @@ public final class FileObjectFactory {
         Statistics.REFRESH_FILE.reset();
     }
 
-    public final void refreshFor(final File... files) {
+    final void refreshFor(final RefreshSlow slow, final File... files) {
         Statistics.StopWatch stopWatch = Statistics.getStopWatch(Statistics.REFRESH_FS);
         final Runnable r = new Runnable() {
+            @Override
             public void run() {
-                Set all2Refresh = collectForRefresh();
-                refresh(all2Refresh, files);
-            }            
+                Set<BaseFileObj> all2Refresh = collectForRefresh();
+                refresh(all2Refresh, slow, files);
+                if (LOG_REFRESH.isLoggable(Level.FINER)) {
+                    LOG_REFRESH.log(Level.FINER, "Refresh for {0} objects", all2Refresh.size());
+                    for (BaseFileObj baseFileObj : all2Refresh) {
+                        LOG_REFRESH.log(Level.FINER, "  {0}", baseFileObj);
+                    }
+                }
+            }
         };        
         stopWatch.start();
         try {
-            FileBasedFileSystem.getInstance().runAtomicAction(new FileSystem.AtomicAction() {
-                public void run() throws IOException {
-                    FileBasedFileSystem.runAsInconsistent(r);
-                }
-            });
+            if (slow != null) {
+                FileBasedFileSystem.runAsInconsistent(r);
+            } else {
+                FileBasedFileSystem.getInstance().runAtomicAction(new FileSystem.AtomicAction() {
+                    @Override
+                    public void run() throws IOException {
+                        FileBasedFileSystem.runAsInconsistent(r);
+                    }
+                });
+            }
         } catch (IOException iex) {/*method refreshAll doesn't throw IOException*/
 
         }
         stopWatch.stop();
 
         // print refresh stats unconditionally in trunk
-        Logger.getLogger("org.netbeans.modules.masterfs.REFRESH").fine(
-                "FS.refresh statistics (" + Statistics.fileObjects() + "FileObjects):\n  " +
-                Statistics.REFRESH_FS.toString() + "\n  " +
-                Statistics.LISTENERS_CALLS.toString() + "\n  " +
-                Statistics.REFRESH_FOLDER.toString() + "\n  " +
-                Statistics.REFRESH_FILE.toString() + "\n");
+        if (LOG_REFRESH.isLoggable(Level.FINE)) {
+            LOG_REFRESH.log(
+                Level.FINE,
+                "FS.refresh statistics ({0}FileObjects):\n  {1}\n  {2}\n  {3}\n  {4}\n",
+                new Object[]{
+                    Statistics.fileObjects(),
+                    Statistics.REFRESH_FS.toString(),
+                    Statistics.LISTENERS_CALLS.toString(),
+                    Statistics.REFRESH_FOLDER.toString(),
+                    Statistics.REFRESH_FILE.toString()
+                }
+            );
+        }
 
         Statistics.REFRESH_FS.reset();
         Statistics.LISTENERS_CALLS.reset();
@@ -706,6 +790,7 @@ public final class FileObjectFactory {
         AsyncRefreshAtomicAction(FileObject fo) {
             this.fo = fo;
         }
+        @Override
         public void run() throws IOException {
             this.fo.refresh();
         }
