@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -210,5 +213,34 @@ public class FileNameTest extends NbTestCase {
         assertFalse(f2.equals(f));
         assertTrue (f2.getName().equals("renamed3"));        
     }
-    
+
+    public void testTwoNamingsAreOnlyEqualIfTheyRepresentTheSamePath() {
+        File hc1 = new HashCodeFile("/space/root/myfile", 444);
+        File hc2 = new HashCodeFile("/space/myfile", 444);
+
+        FileNaming nf1 = NamingFactory.fromFile(hc1);
+        FileNaming nf2 = NamingFactory.fromFile(hc2);
+        
+        assertFalse("namings are different", nf1.equals(nf2));
+    }
+
+
+    private static final class HashCodeFile extends File {
+        private final int hash;
+        public HashCodeFile(String path, int hash) {
+            super(path);
+            this.hash = hash;
+        }
+
+        @Override
+        public int hashCode() {
+            return hash;
+        }
+
+        @Override
+        public boolean equals(Object obj) {
+            return super.equals(obj) && (obj instanceof HashCodeFile) && ((HashCodeFile)obj).hash == hash;
+        }
+
+    }
 }
