@@ -1,8 +1,11 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
- * 
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
- * 
+ *
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
+ *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
  * Development and Distribution License("CDDL") (collectively, the
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -51,6 +54,7 @@ import java.net.URISyntaxException;
 import java.net.URL;
 import java.net.URLClassLoader;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Enumeration;
@@ -94,9 +98,11 @@ import junit.framework.TestResult;
  * @author Jaroslav Tulach <jaroslav.tulach@netbeans.org>
  */
 public class NbModuleSuite {
+    private static final Logger LOG;
 
     static {
         System.setProperty("org.netbeans.MainImpl.154417", "true");
+        LOG = Logger.getLogger(NbModuleSuite.class.getName());
     }
 
     private NbModuleSuite() {}
@@ -157,7 +163,7 @@ public class NbModuleSuite {
         
         /** Regular expression to match clusters that shall be enabled.
          * To enable all cluster, one can use <code>".*"</code>. To enable
-         * ide and java clusters, it is handy to pass in <code>"ide.*|java.*</code>.
+         * ide and java clusters, it is handy to pass in <code>"ide[0-9]*|java[0-9]*</code>.
          * There is no need to requrest presence of <code>platform.*</code> cluster,
          * as that is available all the time by default.
          * <p>
@@ -432,7 +438,7 @@ public class NbModuleSuite {
      * determined from the actual classpath of a module, which is common
      * when in all NetBeans tests. All other modules are kept disabled.
      * In addition,it allows one limit the clusters that shall be made available.
-     * For example <code>ide.*|java.*</code> will start the container just
+     * For example <code>ide[0-9]*|java[0-9]*</code> will start the container just
      * with platform, ide and java clusters.
      * 
      * 
@@ -456,7 +462,7 @@ public class NbModuleSuite {
      * determined from the actual classpath of a module, which is common
      * when in all NetBeans tests. All other modules are kept disabled.
      * In addition,it allows one limit the clusters that shall be made available.
-     * For example <code>ide.*|java.*</code> will start the container just
+     * For example <code>ide[0-9]*|java[0-9]*</code> will start the container just
      * with platform, ide and java clusters.
      * 
      * 
@@ -639,17 +645,17 @@ public class NbModuleSuite {
                     path.add(f.getCanonicalFile());
                 }
             }
+            if (path == null) {
+                path = new HashSet<File>(Arrays.asList(plat.getParentFile().listFiles()));
+            }
             for (String c : regExps) {
-                for (File f : plat.getParentFile().listFiles()) {
+                for (File f : path) {
                     if (f.equals(plat)) {
                         continue;
                     }
                     if (!f.getName().matches(c)) {
                         continue;
                     }
-                    if (path != null && !path.contains(f)) {
-                        continue;
-                    }
                     File m = new File(new File(f, "config"), "Modules");
                     if (m.exists()) {
                         clusters.add(f);
@@ -682,16 +688,32 @@ public class NbModuleSuite {
                     }
                 }
             }
+
+            //in case we're running code coverage, load the coverage libraries
+            if (System.getProperty("code.coverage.classpath") != null)
+            {
+                File coveragePath = new File(System.getProperty("code.coverage.classpath"));
+                if (coveragePath.isDirectory()) {
+                    for (File jar : coveragePath.listFiles()) {
+                        if (jar.getName().endsWith(".jar")) {
+                            bootCP.add(jar.toURI().toURL());
+                        }
+                    }
+                }
+            }            
             
             // loader that does not see our current classloader
             JUnitLoader junit = new JUnitLoader(config.parentClassLoader, NbModuleSuite.class.getClassLoader());
             URLClassLoader loader = new URLClassLoader(bootCP.toArray(new URL[0]), junit);
             Class<?> main = loader.loadClass("org.netbeans.Main"); // NOI18N
             Assert.assertEquals("Loaded by our classloader", loader, main.getClassLoader());
-            Method m = main.getDeclaredMethod("main", String[].class); // NOI18N
+            Method m = main.getDeclaredMethod("main", String[].class); // NOI18N            
 
             System.setProperty("java.util.logging.config", "-");
             System.setProperty("netbeans.logger.console", "true");
+            if (System.getProperty("netbeans.logger.noSystem") == null) {
+                System.setProperty("netbeans.logger.noSystem", "true");
+            }
             System.setProperty("netbeans.home", platform.getPath());
             System.setProperty("netbeans.full.hack", "true");
 
@@ -761,9 +783,9 @@ public class NbModuleSuite {
                         Class<? extends TestCase> sndClazz =
                             testLoader.loadClass(item.clazz.getName()).asSubclass(TestCase.class);
                         if (item.fileNames == null) {
-                            toRun.addTest(new NbTestSuite(sndClazz));
+                            toRun.addTest(new NbTestSuiteLogCheck(sndClazz));
                         } else {
-                            NbTestSuite t = new NbTestSuite();
+                            NbTestSuite t = new NbTestSuiteLogCheck();
                             t.addTests(sndClazz, item.fileNames);
                             toRun.addTest(t);
                         }
@@ -968,7 +990,13 @@ public class NbModuleSuite {
                     is.close();
                 }
             }
-
+            for (;;) {
+                int index = builder.indexOf("\r\n");
+                if (index == -1) {
+                    break;
+                }
+                builder.deleteCharAt(index);
+            }
             return builder.toString();
         }
 
@@ -1126,16 +1154,72 @@ public class NbModuleSuite {
         }
 
         private static void writeModule(File file, String xml) throws IOException {
+            String previous = null;
             if (file.exists()) {
-                String previous = asString(new FileInputStream(file), true);
+                previous = asString(new FileInputStream(file), true);
                 if (previous.equals(xml)) {
                     return;
                 }
+                LOG.fine("rewrite module file: " + file);
+                charDump(previous);
+                LOG.fine("new----");
+                charDump(xml);
+                LOG.fine("end----");
             }
-
             FileOutputStream os = new FileOutputStream(file);
             os.write(xml.getBytes("UTF-8"));
             os.close();
         }
+
+        private static void charDump(String text) {
+            StringBuilder sb = new StringBuilder(5 * text.length());
+            for (int i = 0; i < text.length(); i++) {
+                if (i % 8 == 0) {
+                    if (i > 0) {
+                        sb.append('\n');
+                    }
+                } else {
+                    sb.append(' ');
+                }
+
+                int ch = text.charAt(i);
+                if (' ' <= ch && ch <= 'z') {
+                    sb.append('\'').append((char)ch).append('\'');
+                } else {
+                    sb.append('x').append(two(Integer.toHexString(ch).toUpperCase()));
+                }
+            }
+            sb.append('\n');
+            LOG.fine(sb.toString());
+        }
+
+        private static String two(String s) {
+            int len = s.length();
+            switch (len) {
+                case 0: return "00";
+                case 1: return "0" + s;
+                case 2: return s;
+                default: return s.substring(len - 2);
+            }
+        }
+
     } // end of S
+
+    private static class NbTestSuiteLogCheck extends NbTestSuite {
+        public NbTestSuiteLogCheck() {
+        }
+        public NbTestSuiteLogCheck(Class<? extends TestCase> clazz) {
+            super(clazz);
+        }
+
+        @Override
+        public void runTest(Test test, TestResult result) {
+            int e = result.errorCount();
+            int f = result.failureCount();
+            super.runTest(test, result);
+            if (e == result.errorCount() && f == result.failureCount()) {
+                NbModuleLogHandler.checkFailures((TestCase) test, result);
+            }
+        }
+    }
 }
