@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -72,6 +75,8 @@ class PlatformLauncher {
     static const char *OPT_HEAP_DUMP_PATH;
     static const char *OPT_KEEP_WORKING_SET_ON_MINIMIZE;
     static const char *OPT_CLASS_PATH;
+    static const char *OPT_SPLASH;
+    static const char *OPT_SPLASH_PATH;
 
     static const char *UPDATER_MAIN_CLASS;
     static const char *IDE_MAIN_CLASS;
@@ -108,10 +113,11 @@ private:
     bool restartRequested();
 
 private:
-    bool exiting;
     bool separateProcess;
     bool suppressConsole;
     bool heapDumpPathOptFound;
+    bool nosplash;
+    bool exiting;
     std::string platformDir;
     std::string userDir;
     std::string clusters;
