@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -87,21 +90,6 @@ final class MainImpl extends Object {
         m.get().invoke(null, new Object[] {args});
     }
 
-    /** Returns string describing usage of the system. Does that by talking to
-     * all registered handlers and asking them to show their usage.
-     *
-     * @return the usage string for the system
-     */
-    public static String usage () throws Exception {
-        java.io.ByteArrayOutputStream os = new java.io.ByteArrayOutputStream ();
-        java.io.ByteArrayOutputStream err = new java.io.ByteArrayOutputStream ();
-
-        String[] newArgs = { "--help" };
-
-        execute(newArgs, System.in, os, err, null);
-        return new String (os.toByteArray ());
-    }
-
     /** Constructs the correct ClassLoader, finds main method to execute
      * and invokes all registered CLIHandlers.
      *
@@ -183,7 +171,7 @@ final class MainImpl extends Object {
             MainImpl.class.getClassLoader()
         });
 
-        // Needed for Lookup.getDefault to find NbTopManager.Lkp.
+        // Needed for Lookup.getDefault to find MainLookup.
         // Note that ModuleManager.updateContextClassLoaders will later change
         // the loader on this and other threads to be MM.SystemClassLoader anyway.
         Thread.currentThread().setContextClassLoader (loader);
@@ -282,7 +270,7 @@ final class MainImpl extends Object {
                     InputStream is = u.openStream();
                     mf = new java.util.jar.Manifest(is);
                     is.close();
-                    value = mf.getMainAttributes().getValue("OpenIDE-Module-Build-Version"); // NOI18N
+                    value = mf.getMainAttributes().getValue("OpenIDE-Module-Implementation-Version"); // NOI18N
                     if (value != null) {
                         break;
                     }
