@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -70,7 +73,7 @@ import org.openide.util.Lookup;
  * 
  * @author  Petr Nejedly, Jesse Glick
  */
-public class ProxyClassLoader extends ClassLoader implements Util.PackageAccessibleClassLoader {
+public class ProxyClassLoader extends ClassLoader {
 
     private static final Logger LOGGER = Logger.getLogger(ProxyClassLoader.class.getName());
     private static final boolean LOG_LOADING;
@@ -239,10 +242,12 @@ public class ProxyClassLoader extends ClassLoader implements Util.PackageAccessi
                                     cls.getClassLoader() + " and " + pcl + " starting from " + this +
                                     "; see http://wiki.netbeans.org/DevFaqModuleCCE";
                             ClassNotFoundException cnfe = new ClassNotFoundException(message);
-                            if (LOGGER.isLoggable(Level.FINE)) {
-                                LOGGER.log(Level.FINE, null, cnfe);
-                            } else {
-                                LOGGER.warning(message);
+                            if (arbitraryLoadWarnings.add(message)) {
+                                if (LOGGER.isLoggable(Level.FINE)) {
+                                    LOGGER.log(Level.FINE, null, cnfe);
+                                } else {
+                                    LOGGER.warning(message);
+                                }
                             }
                             throw cnfe;
                         }
@@ -266,6 +271,8 @@ public class ProxyClassLoader extends ClassLoader implements Util.PackageAccessi
         return cls; 
     }
     private String diagnosticCNFEMessage(String base, Set<ProxyClassLoader> del) {
+        /* the message collapsing disabled - it heavily reduces usefulness
+           of the diagnostic message
         String parentSetS;
         int size = parentSet.size();
         if (size <= 10) {
@@ -280,11 +287,13 @@ public class ProxyClassLoader extends ClassLoader implements Util.PackageAccessi
             }
             b.append(", ..." + (size - 10) + " more]");
             parentSetS = b.toString();
-        }
+        }*/
+
         return base + " starting from " + this +
                 " with possible defining loaders " + del +
-                " and declared parents " + parentSetS;
+                " and declared parents " + parentSet;
     }
+    private static final Set<String> arbitraryLoadWarnings = Collections.synchronizedSet(new HashSet<String>());
 
     /** May return null */ 
     private synchronized Class selfLoadClass(String pkg, String name) { 
@@ -339,10 +348,6 @@ public class ProxyClassLoader extends ClassLoader implements Util.PackageAccessi
         name = stripInitialSlash(name);
 
         int last = name.lastIndexOf('/');
-        if (last == -1) {
-            printDefaultPackageWarning(name);
-        }
-
         String pkg = (last >= 0) ? name.substring(0, last).replace('/', '.') : "";
         String path = name.substring(0, last+1);
         
@@ -397,52 +402,28 @@ public class ProxyClassLoader extends ClassLoader implements Util.PackageAccessi
         return super.findResource(name);
     }
     
-    /**
-     * Finds all the resource with the given name. The implementation of
-     * this method uses the {@link #simpleFindResources(String)} method to find
-     * all the resources available from this classloader and adds all the 
-     * resources from all the parents.
-     *
-     * @param  name the resource name
-     * @return an Enumeration of URLs for the resources
-     * @throws IOException if I/O errors occur
-     */    
     @Override
     public final synchronized Enumeration<URL> getResources(String name) throws IOException {
         name = stripInitialSlash(name);
         final int slashIdx = name.lastIndexOf('/');
-        if (slashIdx == -1) {
-            printDefaultPackageWarning(name);
-        }
-
         final String path = name.substring(0, slashIdx + 1);
+        String pkg = name.startsWith("META-INF/") ? name.substring(8) : (slashIdx >= 0) ? name.substring(0, slashIdx).replace('/', '.') : "";
         List<Enumeration<URL>> sub = new ArrayList<Enumeration<URL>>();
 
         // always consult SCL first
         if (shouldDelegateResource(path, null)) sub.add(systemCL.getResources(name));
         
-        if(name.startsWith("META-INF/")) { // common but expensive resources
+        Set<ProxyClassLoader> del = packageCoverage.get(pkg);
 
-            String relName = name.substring(8);
-            Set<ProxyClassLoader> del = packageCoverage.get(relName);
-
-            if (del != null) { // unclaimed resource, go directly to SCL
-                for (ProxyClassLoader pcl : parents) { // all our accessible parents
-                    if (del.contains(pcl) && shouldDelegateResource(path, pcl)) { // that cover given package
-                        sub.add(pcl.findResources(name));
-                    }
-                }
-                if (del.contains(this)) {
-                    sub.add(findResources(name));
-                }
-            }
-        } else { // Don't bother optimizing this call by domains.
-            for (ProxyClassLoader pcl : parents) { 
-                if (shouldDelegateResource(path, pcl)) {
+        if (del != null) {
+            for (ProxyClassLoader pcl : parents) { // all our accessible parents
+                if (del.contains(pcl) && shouldDelegateResource(path, pcl)) { // that cover given package
                     sub.add(pcl.findResources(name));
                 }
             }
-            sub.add(findResources(name));
+            if (del.contains(this)) {
+                sub.add(findResources(name));
+            }
         }
         // Should not be duplicates, assuming the parent loaders are properly distinct
         // from one another and do not overlap in JAR usage, which they ought not.
@@ -557,28 +538,6 @@ public class ProxyClassLoader extends ClassLoader implements Util.PackageAccessi
         return packages.values().toArray(new Package[packages.size()]);
     }
     
-    public Package getPackageAccessibly(String name) {
-        return getPackage(name);
-    }
-    
-    public Package[] getPackagesAccessibly() {
-        return getPackages();
-    }
-
-    /**
-     * #38368: Warn that the default package should not be used.
-     */
-    private static void printDefaultPackageWarning(String name) {
-        if (!"commons-logging.properties".equals(name) &&
-            !"jndi.properties".equals(name) &&
-            !"log4j.properties".equals(name) &&
-            !"simplelog.properties".equals(name) &&
-            !"emma_default.properties".equals(name) &&
-            !"emma.properties".equals(name)) { // NOI18N
-            LOGGER.log(Level.INFO, null, new IllegalStateException("You are trying to access file: " + name + " from the default package. Please see http://www.netbeans.org/download/dev/javadoc/org-openide-modules/org/openide/modules/doc-files/classpath.html#default_package"));
-        }
-    }
-    
     /** Coalesce parent classloaders into an optimized set.
      * This means that all parents of the specified classloaders
      * are also added recursively, removing duplicates along the way.
@@ -685,4 +644,11 @@ public class ProxyClassLoader extends ClassLoader implements Util.PackageAccessi
         }
     }
 
+    final ClassLoader firstParent() {
+        if (parents == null || parents.length == 0) {
+            return null;
+        }
+        return parents[0];
+    }
+
 }
