@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -38,13 +41,9 @@
  */
 package org.netbeans;
 
-import java.io.File;
-import java.io.IOException;
 import java.lang.reflect.Field;
 import java.lang.reflect.Method;
-import java.net.Proxy;
 import java.net.URL;
-import java.net.URLConnection;
 import java.net.URLStreamHandler;
 import java.net.URLStreamHandlerFactory;
 import java.util.Collection;
@@ -53,7 +52,6 @@ import java.util.logging.Logger;
 import org.openide.util.Lookup;
 import org.openide.util.LookupEvent;
 import org.openide.util.LookupListener;
-import org.openide.util.Utilities;
 
 /**
  * A stream handler factory that delegates to others in lookup.
@@ -107,21 +105,15 @@ public class ProxyURLStreamHandlerFactory implements URLStreamHandlerFactory, Lo
     private final URLStreamHandler originalJarHandler;
     private Lookup.Result<URLStreamHandlerFactory> r;
     private URLStreamHandlerFactory[] handlers;
-    private boolean isWindows;
 
     private ProxyURLStreamHandlerFactory(URLStreamHandlerFactory delegate, URLStreamHandler originalJarHandler) {
         this.delegate = delegate;
         this.originalJarHandler = originalJarHandler;
-        // #154032: call to Utilities.isWindows() in createURLStreamHandler caused infinite recursion in WebStart
-        isWindows = Utilities.isWindows();
     }
 
     public URLStreamHandler createURLStreamHandler(String protocol) {
         if (protocol.equals("jar")) {
-            return new JarClassLoader.ResURLStreamHandler(originalJarHandler);
-        } else if (protocol.equals("file") && isWindows && System.getProperty("java.version").startsWith("1.5")) {  // NOI18N
-            LOG.fine("Registering UNCFileStreamHandler.");  // NOI18N
-            return UNCFileURLStreamHandler.getInstance();
+            return new JarClassLoader.JarURLStreamHandler(originalJarHandler);
         } else if (protocol.equals("file") || protocol.equals("http") || protocol.equals("https") || protocol.equals("resource")) { // NOI18N
             // Well-known handlers in JRE. Do not try to initialize lookup, etc.
             return null;
@@ -157,75 +149,4 @@ public class ProxyURLStreamHandlerFactory implements URLStreamHandlerFactory, Lo
             handlers = c.toArray(new URLStreamHandlerFactory[0]);
         }
     }
-    
-    private static class UNCFileURLStreamHandler extends URLStreamHandler {
-
-        private static final URLStreamHandler DELEGATE = getDelegate();
-        private static final Method OPEN_CONNECTION_METHOD = getOpenConnectionMethod();
-        private static UNCFileURLStreamHandler instance;
-        
-        /** Returns null if not possible to get instance of delegate handler. */
-        public static UNCFileURLStreamHandler getInstance() {
-            if(instance == null) {
-                if(DELEGATE != null && OPEN_CONNECTION_METHOD != null) {
-                    instance = new UNCFileURLStreamHandler();
-                }
-            }
-            return instance;
-        }
-        
-        private static URLStreamHandler getDelegate() {
-            try {
-                Class sunHandlerClass = Class.forName("sun.net.www.protocol.file.Handler"); // NOI18N
-                return (URLStreamHandler) sunHandlerClass.newInstance();
-            } catch (Exception ex) {
-                // ignore
-                LOG.log(Level.FINE, "Exception while instantiating sun.net.www.protocol.file.Handler.", ex);  // NOI18N
-            }
-            return null;
-        }
-        
-        private static Method getOpenConnectionMethod() {
-            try {
-                return DELEGATE.getClass().getMethod("openConnection", URL.class, Proxy.class); // NOI18N
-            } catch (Exception ex) {
-                LOG.log(Level.FINE, "Exception while getting method sun.net.www.protocol.file.Handler.openConnection.", ex);  // NOI18N
-            }
-            return null;
-        }
-        
-        @Override
-        protected void parseURL(URL u, String spec, int start, int limit) {
-            super.parseURL(u, spec.replace(File.separatorChar, '/'), start, limit);
-            // Fix UNC URLs - set authority to null and add // before path.
-            if ((u.getAuthority() != null) && u.getPath().startsWith("//")) {  //NOI18N
-                try {
-                    Field authorityField = URL.class.getDeclaredField("authority");  //NOI18N
-                    authorityField.setAccessible(true);
-                    authorityField.set(u, null);
-                    Field pathField = URL.class.getDeclaredField("path");  //NOI18N
-                    pathField.setAccessible(true);
-                    pathField.set(u, "//" + u.getPath());  //NOI18N
-                } catch (Exception e) {
-                    e.printStackTrace();
-                }
-            }
-        }
-
-        @Override
-        protected URLConnection openConnection(URL url) throws IOException {
-            return openConnection(url, null);
-        }
-        
-        @Override
-        protected URLConnection openConnection(URL url, Proxy proxy) throws IOException {
-            try {
-                return (URLConnection) OPEN_CONNECTION_METHOD.invoke(DELEGATE, url, proxy);
-            } catch (Exception ex) {
-                IOException ioe = new IOException("openConnection "+url+" failed.");  // NOI18N
-                ioe.initCause(ex);
-                throw ioe;
-            }
-        }
-    }
 }
