@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -104,7 +107,6 @@ import javax.swing.event.DocumentEvent;
 import javax.swing.event.DocumentListener;
 import javax.swing.event.TableModelEvent;
 import javax.swing.table.*;
-import javax.swing.table.TableCellRenderer;
 
 /**
  * <UL>Extended JTable (ETable) adds these features to JTable:
@@ -389,29 +391,34 @@ public class ETable extends JTable {
         if(editing == FULLY_NONEDITABLE) {
             return false;
         }
-        int modelRow = convertRowIndexToModel(row);
-        return super.isCellEditable(modelRow, column);
+        return super.isCellEditable(row, column);
     }
 
-    /**
+    /*
      * Overriden to call convertRowIndexToModel(...).
      * @see javax.swing.JTable#getCellRenderer(int, int)
-     */
+     *
+     * NOT NECESSARY - JTable does not use the "row" argument.
+     *
     @Override
     public TableCellRenderer getCellRenderer(int row, int column) {
         int modelRow = convertRowIndexToModel(row);
         return super.getCellRenderer(modelRow, column);
     }
+     */
 
-    /**
+    /*
      * Overriden to call convertRowIndexToModel(...).
      * @see javax.swing.JTable#getCellEditor(int, int)
-     */
+     *
+     * NOT NECESSARY - JTable does not use the "row" argument.
+     *
     @Override
     public TableCellEditor getCellEditor(int row, int column) {
         int modelRow = convertRowIndexToModel(row);
         return super.getCellEditor(modelRow, column);
     }
+     */
 
     /**
      * Sets all the cells in the <code>ETable</code> to be editable if
@@ -634,11 +641,6 @@ public class ETable extends JTable {
     @Override
     public Object getValueAt(int row, int column) {
         int modelRow = row;
-        //#144502: in 1.6 JTable adds method convertRowIndexToModel which is called in
-        //its getValueAt, otherwise we have translate the index
-        if( System.getProperty("java.version").startsWith("1.5") ) { //NOI18N //NOI18N
-            modelRow = convertRowIndexToModel(row);
-        }
         return super.getValueAt(modelRow, column);
     }
 
@@ -648,8 +650,7 @@ public class ETable extends JTable {
      */
     @Override
     public void setValueAt(Object aValue, int row, int column) {
-        int modelRow = convertRowIndexToModel(row);
-        super.setValueAt(aValue, modelRow, column);
+        super.setValueAt(aValue, row, column);
     }
 
     /**
@@ -688,6 +689,26 @@ public class ETable extends JTable {
         filteredRowCount = -1; // force to recompute the rowCount
         super.tableChanged(new TableModelEvent(getModel()));
     }
+
+    /**
+     * Get the "quick-filter" object that is currently active.
+     *
+     * @return the filter object or <code>null</code>
+     * @see #setQuickFilter(int, java.lang.Object)
+     */
+    public Object getQuickFilterObject() {
+        return quickFilterObject;
+    }
+
+    /**
+     * Get the column which is currently filtered by "quick-filter" object.
+     *
+     * @return the filtered column or <code>-1</code>
+     * @see #setQuickFilter(int, java.lang.Object)
+     */
+    public int getQuickFilterColumn() {
+        return quickFilterColumn;
+    }
     
     /**
      * Makes the table show all the rows, resetting the filter state
@@ -940,12 +961,11 @@ public class ETable extends JTable {
      * Convert indices of selected rows to model.
      */
     private int[] getSelectedRowsInModel() {
-        int inView[] = getSelectedRows();
-        int result[] = new int[inView.length];
-        for (int i = 0; i < inView.length; i++) {
-            result[i] = convertRowIndexToModel(inView[i]);
+        int rows[] = getSelectedRows();
+        for (int i = 0; i < rows.length; i++) {
+            rows[i] = convertRowIndexToModel(rows[i]);
         }
-        return result;
+        return rows;
     }
     
     /**
@@ -1220,7 +1240,14 @@ public class ETable extends JTable {
         if ((newFormats == null) || (newFormats.length != quickFilterFormatStrings.length)) {
             return;
         }
-        quickFilterFormatStrings = newFormats;
+        quickFilterFormatStrings = newFormats.clone();
+    }
+
+    /**
+     * Returns the quickFilterFormatStrings array.
+     */
+    public String[] getQuickFilterFormatStrings() {
+        return quickFilterFormatStrings.clone();
     }
 
     /**
@@ -1993,6 +2020,7 @@ public class ETable extends JTable {
             if ((resColumn == null) && (me.getClickCount() == 1)) {
                 // ok, do the sorting
                 int column = columnAtPoint(me.getPoint());
+                if (column < 0) return ;
                 TableColumnModel tcm = getColumnModel();
                 if (tcm instanceof ETableColumnModel) {
                     ETableColumnModel etcm = (ETableColumnModel)tcm;
