@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -24,7 +27,7 @@
  * Contributor(s):
  *
  * The Original Software is NetBeans. The Initial Developer of the Original
- * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
+ * Software is Sun Microsystems, Inc. Portions Copyright 1997-2009 Sun
  * Microsystems, Inc. All Rights Reserved.
  *
  * If you wish your version of this file to be governed by only the CDDL
@@ -53,6 +56,7 @@ import java.awt.event.ComponentListener;
 import java.awt.event.MouseEvent;
 import java.util.Comparator;
 import java.util.EventObject;
+import java.util.logging.Logger;
 import javax.swing.AbstractAction;
 import javax.swing.Action;
 import javax.swing.ActionMap;
@@ -66,6 +70,8 @@ import javax.swing.Timer;
 import javax.swing.UIManager;
 import javax.swing.border.Border;
 import javax.swing.event.TableModelEvent;
+import javax.swing.event.TreeModelEvent;
+import javax.swing.table.TableCellEditor;
 import javax.swing.table.TableCellRenderer;
 import javax.swing.table.TableColumn;
 import javax.swing.table.TableModel;
@@ -198,6 +204,7 @@ public class Outline extends ETable {
     private Boolean cachedRootVisible = null;
     private RenderDataProvider renderDataProvider = null;
     private ComponentListener componentListener = null;
+    private boolean selectionDisabled = false;
     /** Creates a new instance of Outline */
     public Outline() {
         init();
@@ -306,9 +313,13 @@ public class Outline extends ETable {
             cachedRootVisible = val ? Boolean.TRUE : Boolean.FALSE;
         }
         if (val != isRootVisible()) {
-            //TODO - need to force a property change on the model,
-            //the layout cache doesn't have direct listener support
             getLayoutCache().setRootVisible(val);
+            if( getLayoutCache().getRowCount() > 0 ) {
+                TreePath rootPath = getLayoutCache().getPathForRow(0);
+                if( null != rootPath )
+                    getLayoutCache().treeStructureChanged(new TreeModelEvent(this, rootPath));
+            }
+            sortAndFilter();
             firePropertyChange("rootVisible", !val, val); //NOI18N
         }
     }
@@ -323,8 +334,12 @@ public class Outline extends ETable {
         }
     }
 
-    /**
-     */
+    @Override
+    public void setRowHeight (int row, int rowHeight) {
+        Logger.getLogger (Outline.class.getName ()).warning ("Not supported yet."); // NOI18N
+    }
+
+
     @Override
     protected TableColumn createColumn(int modelIndex) {
         return new OutlineColumn(modelIndex);
@@ -344,7 +359,7 @@ public class Outline extends ETable {
         }
         @Override
         public boolean isSortingAllowed() {
-            return getModelIndex() != 0;
+            return true;
         }
         /**
          * Comparator used for sorting the rows according to value in
@@ -357,6 +372,7 @@ public class Outline extends ETable {
                 this.ascending = ascending;
             }
             @Override
+            @SuppressWarnings("unchecked")
             public int compare(RowMapping rm1, RowMapping rm2) {
                 int index1 = rm1.getModelRowIndex();
                 int index2 = rm2.getModelRowIndex();
@@ -365,6 +381,15 @@ public class Outline extends ETable {
                 }
                 TreePath tp1 = getLayoutCache().getPathForRow(index1);
                 TreePath tp2 = getLayoutCache().getPathForRow(index2);
+                if (tp1 == null) {
+                    if (tp2 == null) {
+                        return 0;
+                    } else {
+                        return -1;
+                    }
+                } else if (tp2 == null) {
+                    return 1;
+                }
                 if (tp1.isDescendant(tp2)) {
                     return -1;
                 }
@@ -408,7 +433,11 @@ public class Outline extends ETable {
                 if (obj2 == null) {
                     return 1;
                 }
-                if ((obj1 instanceof Comparable) && (obj1.getClass().isAssignableFrom(obj2.getClass()))){
+                // check nested comparator
+                if (getNestedComparator () != null) {
+                    int res = getNestedComparator ().compare (obj1, obj2);
+                    return ascending ? res : -res;
+                } else if ((obj1 instanceof Comparable) && (obj1.getClass().isAssignableFrom(obj2.getClass()))){
                     Comparable c1 = (Comparable) obj1;
                     return ascending ? c1.compareTo(obj2) : - c1.compareTo(obj2);
                 }
@@ -469,8 +498,6 @@ public class Outline extends ETable {
     }
     
     public boolean isExpanded (TreePath path) {
-        if( !isRootVisible() && null == path.getParentPath() )
-            return true;
         return getTreePathSupport().isExpanded(path);
     }
     
@@ -491,18 +518,26 @@ public class Outline extends ETable {
     
     public TreePath getClosestPathForLocation(int x, int y) {
         Insets i = getInsets();
+        TreePath tp;
         if (i != null) {
-            return getLayoutCache().getPathClosestTo(x - i.left, y - i.top);
+            tp = getLayoutCache().getPathClosestTo(x - i.left, y - i.top);
         } else {
-            return getLayoutCache().getPathClosestTo(x,y);
+            tp = getLayoutCache().getPathClosestTo(x,y);
         }
+        int row = getLayoutCache().getRowForPath(tp);
+        // The UI row needs to be converted to the model row.
+        row = convertRowIndexToModel(row);
+        // Now we can get the correct path from the model row:
+        tp = getLayoutCache().getPathForRow(row);
+        return tp;
     }
     
     @Override
     public boolean editCellAt (int row, int column, EventObject e) {
         //If it was on column 0, it may be a request to expand a tree
         //node - check for that first.
-        if (isTreeColumnIndex (column) && e instanceof MouseEvent) {
+        boolean isTreeColumn = isTreeColumnIndex(column);
+        if (isTreeColumn && e instanceof MouseEvent) {
             MouseEvent me = (MouseEvent) e;
             TreePath path = getLayoutCache().getPathForRow(convertRowIndexToModel(row));
             if (!getOutlineModel().isLeaf(path.getLastPathComponent())) {
@@ -514,12 +549,19 @@ public class Outline extends ETable {
                 }
                 int handleStart = ins.left + (nd * DefaultOutlineCellRenderer.getNestingWidth());
                 int handleEnd = ins.left + handleStart + handleWidth;
-                //TODO: Translate x/y to position of column if non-0
-                
+                // Translate 'x' to position of column if non-0:
+                int columnStart = getCellRect(row, column, false).x;
+                handleStart += columnStart;
+                handleEnd += columnStart;
+
+                TableColumn tableColumn = getColumnModel().getColumn(column);
+                TableCellEditor columnCellEditor = tableColumn.getCellEditor();
                 if ((me.getX() > ins.left && me.getX() >= handleStart && me.getX() <= handleEnd) ||
-                     me.getClickCount() > 1) {
+                    (me.getClickCount() > 1 && columnCellEditor == null)) {
 
                     boolean expanded = getLayoutCache().isExpanded(path);
+                    //me.consume();  - has no effect!
+                    //System.err.println("  event consumed.");
                     if (!expanded) {
                         getTreePathSupport().expandPath(path);
                         
@@ -529,25 +571,86 @@ public class Outline extends ETable {
                             Object lastChild = getOutlineModel().getChild(ourObject, cCount - 1);
                             TreePath lastChildPath = path.pathByAddingChild(lastChild);
                             int lastRow = getLayoutCache().getRowForPath(lastChildPath);
-                            Rectangle rect = getCellRect(lastRow, 0, true);
-                            scrollRectToVisible(rect);
+                            Rectangle rectLast = getCellRect(lastRow, 0, true);
+                            Rectangle rectFirst = getCellRect(getLayoutCache().getRowForPath(path), 0, true);
+                            Rectangle rectFull = new Rectangle(
+                                    rectFirst.x,
+                                    rectFirst.y,
+                                    rectLast.x + rectLast.width - rectFirst.x,
+                                    rectLast.y + rectLast.height - rectFirst.y);
+                            scrollRectToVisible(rectFull);
                         }
                         
                     } else {
                         getTreePathSupport().collapsePath(path);
                     }
+                    selectionDisabled = true;
                     return false;
                 }
             }
+            // It may be a request to check/uncheck a check-box
+            if (checkAt(row, column, me)) {
+                return false;
+            }
         }
             
-        boolean res = super.editCellAt(row, column, e);
-        if( res && isTreeColumnIndex(column) && null != getEditorComponent() ) {
+        boolean res = false;
+        if (!isTreeColumn || e instanceof MouseEvent && ((MouseEvent)e).getClickCount() > 1) {
+            res = super.editCellAt(row, column, e);
+        }
+        if( res && isTreeColumn && null != getEditorComponent() ) {
             configureTreeCellEditor(getEditorComponent(), row, column);
         }
+        if (e == null && !res && isTreeColumn) {
+            // Handle SPACE
+            checkAt(row, column, null);
+        }
         return res;
     }
 
+    private boolean checkAt(int row, int column, MouseEvent me) {
+        RenderDataProvider render = getRenderDataProvider();
+        TableCellRenderer tcr = getDefaultRenderer(Object.class);
+        if (render instanceof CheckRenderDataProvider && tcr instanceof DefaultOutlineCellRenderer) {
+            CheckRenderDataProvider crender = (CheckRenderDataProvider) render;
+            DefaultOutlineCellRenderer ocr = (DefaultOutlineCellRenderer) tcr;
+            Object value = getValueAt(row, column);
+            if (crender.isCheckable(value) && crender.isCheckEnabled(value)) {
+                boolean chBoxPosition;
+                if (me == null) {
+                    chBoxPosition = true;
+                } else {
+                    int handleWidth = DefaultOutlineCellRenderer.getExpansionHandleWidth();
+                    int chWidth = ocr.getTheCheckBoxWidth();
+                    Insets ins = getInsets();
+                    TreePath path = getLayoutCache().getPathForRow(convertRowIndexToModel(row));
+                    int nd = path.getPathCount() - (isRootVisible() ? 1 : 2);
+                    if (nd < 0) {
+                        nd = 0;
+                    }
+                    int chStart = ins.left + (nd * DefaultOutlineCellRenderer.getNestingWidth()) + handleWidth;
+                    int chEnd = chStart + chWidth;
+                    //TODO: Translate x/y to position of column if non-0
+
+                    chBoxPosition = (me.getX() > ins.left && me.getX() >= chStart && me.getX() <= chEnd) &&
+                                    me.getClickCount() == 1;
+                }
+                if (chBoxPosition) {
+                    Boolean selected = crender.isSelected(value);
+                    if (selected == null || Boolean.TRUE.equals(selected)) {
+                        crender.setSelected(value, Boolean.FALSE);
+                    } else {
+                        crender.setSelected(value, Boolean.TRUE);
+                    }
+                    Rectangle r = getCellRect(row, column, true);
+                    repaint (r.x, r.y, r.width, r.height);
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+
     protected void configureTreeCellEditor( Component editor, int row, int column ) {
         if( !(editor instanceof JComponent) ) {
             return;
@@ -577,15 +680,16 @@ public class Outline extends ETable {
         //Users of themes can set an explicit row height, so check for it
         Integer i = (Integer) UIManager.get("netbeans.outline.rowHeight"); //NOI18N
         
-        int rHeight;
+        int rHeight = 20;
         if (i != null) {
             rHeight = i.intValue();
         } else {
             //Derive a row height to accomodate the font and expando icon
             Font f = getFont();
             FontMetrics fm = getFontMetrics(f);
-            rHeight = Math.max(fm.getHeight()+3,
-                DefaultOutlineCellRenderer.getExpansionHandleHeight());
+            int h = Math.max (fm.getHeight () + fm.getMaxDescent (),
+                DefaultOutlineCellRenderer.getExpansionHandleHeight ());
+            rHeight = Math.max (rHeight, h) + 2; // XXX: two pixels for cell's insets
         }
         //Set row height.  If displayable, this will generate a new call
         //to paint()
@@ -599,6 +703,50 @@ public class Outline extends ETable {
 //        System.err.println("row count is " + getRowCount());
     }
     
+    @Override
+    public void changeSelection(int rowIndex, int columnIndex, boolean toggle, boolean extend) {
+        if (selectionDisabled) {
+            //selectionDisabled = false;
+            //System.err.println("\nSelection DISABLED.");
+            return ;
+        }
+        super.changeSelection(rowIndex, columnIndex, toggle, extend);
+    }
+
+    @Override
+    protected void processMouseEvent(MouseEvent e) {
+        switch(e.getID()) {
+          case MouseEvent.MOUSE_PRESSED:
+              selectionDisabled = false;
+              break;
+          case MouseEvent.MOUSE_RELEASED:
+              selectionDisabled = false;
+              break;
+          case MouseEvent.MOUSE_CLICKED:
+              selectionDisabled = false;
+              break;
+          case MouseEvent.MOUSE_ENTERED:
+              selectionDisabled = false;
+              break;
+          case MouseEvent.MOUSE_EXITED:
+              selectionDisabled = false;
+              break;
+          case MouseEvent.MOUSE_MOVED:
+              break;
+          case MouseEvent.MOUSE_DRAGGED:
+              if (selectionDisabled) {
+                  //System.err.println("\nDrag DISABLED.");
+                  return ;
+              }
+              break;
+          case MouseEvent.MOUSE_WHEEL:
+              break;
+        }
+        super.processMouseEvent(e);
+    }
+
+
+
     /** Create a component listener to handle size changes if the table model
      * is large-model */
     private ComponentListener getComponentListener() {
@@ -759,8 +907,7 @@ public class Outline extends ETable {
 
         public void actionPerformed(ActionEvent e) {
             if( getSelectedRowCount() == 1 && isTreeColumnIndex (getSelectedColumn()) ) {
-                int selRow = getSelectedRow();
-                TreePath selPath = getLayoutCache().getPathForRow(selRow);
+                TreePath selPath = getLayoutCache().getPathForRow(convertRowIndexToModel (getSelectedRow ()));
                 if( null != selPath 
                         && !getOutlineModel().isLeaf(selPath.getLastPathComponent()) ) {
                     boolean expanded = getLayoutCache().isExpanded(selPath);
