@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -41,26 +44,33 @@
 
 package org.openide.awt;
 
-import java.awt.AWTEvent;
+import java.awt.Color;
 import java.awt.Component;
 import java.awt.Container;
 import java.awt.Dimension;
+import java.awt.FlowLayout;
 import java.awt.FocusTraversalPolicy;
-import java.awt.Graphics;
 import java.awt.Image;
-import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.Toolkit;
+import java.beans.PropertyChangeEvent;
+import javax.swing.Icon;
 import javax.swing.plaf.UIResource;
 import org.openide.util.ImageUtilities;
 import org.openide.util.Utilities;
-import javax.swing.event.ChangeEvent;
-import javax.swing.event.ChangeListener;
-import java.awt.event.AWTEventListener;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
+import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
+import java.beans.PropertyChangeListener;
 import java.util.logging.Level;
 import java.util.logging.Logger;
+import java.util.regex.Pattern;
+import javax.swing.BorderFactory;
+import javax.swing.JButton;
 import javax.swing.JComponent;
+import javax.swing.JLabel;
+import javax.swing.JPanel;
 import javax.swing.JTabbedPane;
 import javax.swing.SwingUtilities;
 import javax.swing.UIManager;
@@ -74,7 +84,7 @@ import org.openide.util.Exceptions;
  * @since 6.10.0
  *
  */
-final class CloseButtonTabbedPane extends JTabbedPane {
+final class CloseButtonTabbedPane extends JTabbedPane implements PropertyChangeListener {
 
     private Image closeTabImage;
     private Image closeTabPressedImage;
@@ -83,16 +93,36 @@ final class CloseButtonTabbedPane extends JTabbedPane {
     static final String PROP_CLOSE = "close";
 
     CloseButtonTabbedPane() {
-        addChangeListener( new ChangeListener() {
-            public void stateChanged(ChangeEvent e) {
-                reset();
-            }
-        });
-        CloseButtonListener.install();
+            // close tab via middle button
+            addMouseListener(new MouseAdapter() {
+                int lastIdx = -1;
+
+                @Override
+                public void mousePressed(MouseEvent e) {
+                    if (SwingUtilities.isMiddleMouseButton(e)) {
+                        lastIdx = getUI().tabForCoordinate(CloseButtonTabbedPane.this, e.getX(), e.getY());
+                    }
+                }
+
+                @Override
+                public void mouseReleased(MouseEvent e) {
+                    if (SwingUtilities.isMiddleMouseButton(e)) {
+                        int idx = getUI().tabForCoordinate(CloseButtonTabbedPane.this, e.getX(), e.getY());
+                        if (idx >= 0) {
+                            Component comp = getComponentAt(idx);
+                            if (idx == lastIdx && comp != null && !hideCloseButton(comp)) {
+                                fireCloseRequest(comp);
+                            }
+                        }
+                        lastIdx = -1;
+                    }
+                }
+            });
         //Bugfix #28263: Disable focus.
         setFocusable(false);
         setFocusCycleRoot(true);
         setFocusTraversalPolicy(new CBTPPolicy());
+        setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
     }
 
     private Component sel() {
@@ -124,40 +154,84 @@ final class CloseButtonTabbedPane extends JTabbedPane {
 
     private int pressedCloseButtonIndex = -1;
     private int mouseOverCloseButtonIndex = -1;
-    private boolean draggedOut = false;
 
     @Override
-    public Component add (Component c) {
-        Component result = super.add(c);
-        // #75317 - don't try to set the title if LF (such as Substance LF)
-        // is adding some custom UI components into tabbed pane
-        if (!(c instanceof UIResource)) {
-            String s = c.getName();
-            if (s != null) {
-                s += "  ";
+    public void insertTab(String title, Icon icon, Component component, String tip, int index) {
+        super.insertTab(title, icon, component, tip, index);
+            component.addPropertyChangeListener(TabbedPaneFactory.NO_CLOSE_BUTTON, this);
+            if (!hideCloseButton(component)) {
+                setTabComponentAt(index, new ButtonTab());
             }
-            setTitleAt (getComponentCount() - 1, s);
+        if (title != null) {
+            setTitleAt(index, title);
         }
-        return result;
+        validate();
     }
 
+    @Override
+    public void removeTabAt(int index) {
+        Component c = getComponentAt(index);
+        c.removePropertyChangeListener(TabbedPaneFactory.NO_CLOSE_BUTTON, this);
+        super.removeTabAt(index);
+    }
+
+    private static final boolean HTML_TABS_BROKEN = htmlTabsBroken();
+    private static boolean htmlTabsBroken() {
+        String version = System.getProperty("java.version");
+        for (int i = 14; i < 18; i++) {
+            if (version.startsWith("1.6.0_" + i)) {
+                return true;
+            }
+        }
+        if( version.startsWith("1.6.0_20") && isAquaLaF() )
+            return true;
+        return false;
+    }
+    private final Pattern removeHtmlTags = HTML_TABS_BROKEN ? Pattern.compile("\\<.*?\\>") : null;
+
     @Override
     public void setTitleAt(int idx, String title) {
-        String nue = title.indexOf("</html>") != -1 ? //NOI18N
-            Utilities.replaceString(title, "</html>", "&nbsp;&nbsp;</html>") //NOI18N
-            : title + "  ";
-        if (!title.equals(getTitleAt(idx))) {
-            super.setTitleAt(idx, nue);
+        if (title == null) {
+            super.setTitleAt(idx, null);
+            return;
         }
+        // workaround for JDK bug (http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6670274)
+        // NB issue #113388
+        if (removeHtmlTags != null && title.startsWith("<html>")) {
+            title = removeHtmlTags.matcher(title).replaceAll("");
+            title = title.replace("&nbsp;", "");
+        }
+
+        super.setTitleAt(idx, title);
     }
 
-    private void reset() {
-        setMouseOverCloseButtonIndex(-1);
-        setPressedCloseButtonIndex(-1);
-        draggedOut = false;
+    private Component findTabAt(int index) {
+        int componentIndex = -1;
+        for( Component c : getComponents() ) {
+            if( c instanceof UIResource )
+                continue;
+            if( ++componentIndex == index )
+                return c;
+        }
+        return null;
+    }
+
+    private boolean hideCloseButton(Component c) {
+        if (c!=null && c instanceof JComponent) {
+            Object prop = ((JComponent) c).getClientProperty(TabbedPaneFactory.NO_CLOSE_BUTTON);
+            if (prop!=null && prop instanceof Boolean && (Boolean) prop) {
+                return true;
+            }
+        }
+        return false;
     }
 
     private Rectangle getCloseButtonBoundsAt(int i) {
+        Component c = findTabAt(i);
+        //if NO_CLOSE_BUTTON -> return null
+        if (hideCloseButton(c)) {
+            return null;
+        }
         Rectangle b = getBoundsAt(i);
         if (b == null)
             return null;
@@ -178,6 +252,9 @@ final class CloseButtonTabbedPane extends JTabbedPane {
                 b.y -= 2;
             } else if( isWindowsXPLaF() || isWindowsLaF() || isAquaLaF() ) {
                 b.x -= 2;
+            } else if( isGTKLaF() && i == getSelectedIndex() ) {
+                b.x -= 1;
+                b.y -= 2;
             }
             if( i == getTabCount()-1 ) {
                 if( isMetalLaF() )
@@ -210,7 +287,7 @@ final class CloseButtonTabbedPane extends JTabbedPane {
         return lfID.endsWith("Windows"); //NOI18N
     }
     
-    private boolean isAquaLaF() {
+    private static boolean isAquaLaF() {
         return "Aqua".equals( UIManager.getLookAndFeel().getID() );
     }
     
@@ -218,31 +295,11 @@ final class CloseButtonTabbedPane extends JTabbedPane {
         String lfID = UIManager.getLookAndFeel().getID();
         return "Metal".equals( lfID ); //NOI18N
     }
-    
-    @Override
-    public void paint(Graphics g) {
-        super.paint(g);
 
-        // Have a look at
-        // http://ui.netbeans.org/docs/ui/closeButton/closeButtonUISpec.html
-        // to see how the buttons are specified to be drawn.
-
-        for (int i = 0, n = getTabCount(); i < n; i++) {
-            Rectangle r = getCloseButtonBoundsAt(i);
-            if (r == null)
-                continue;
-
-            if (i == mouseOverCloseButtonIndex
-            || (i == pressedCloseButtonIndex && draggedOut)) {
-                g.drawImage(getCloseTabMouseOverImage(), r.x, r.y , this);
-            } else if (i == pressedCloseButtonIndex) {
-                g.drawImage(getCloseTabPressedImage(), r.x, r.y , this);
-            } else {
-                g.drawImage(getCloseTabImage(), r.x, r.y , this);
-            }
-        }
+    private boolean isGTKLaF () {
+        return "GTK".equals( UIManager.getLookAndFeel().getID() ); //NOI18N
     }
-
+    
     private Image getCloseTabImage() {
         if( null == closeTabImage ) {
             if( isWindowsVistaLaF() ) {
@@ -277,7 +334,7 @@ final class CloseButtonTabbedPane extends JTabbedPane {
         return closeTabPressedImage;
     }
     
-    private Image getCloseTabMouseOverImage() {
+    private Image getCloseTabRolloverImage() {
         if( null == closeTabMouseOverImage ) {
             if( isWindowsVistaLaF() ) {
                 closeTabMouseOverImage = ImageUtilities.loadImage("org/openide/awt/resources/vista_close_rollover.png"); // NOI18N
@@ -358,6 +415,13 @@ final class CloseButtonTabbedPane extends JTabbedPane {
     
     private void fireCloseRequest(Component c) {
         firePropertyChange(PROP_CLOSE, null, c);
+        if (getTabLayoutPolicy() == JTabbedPane.SCROLL_TAB_LAYOUT) {
+            int idx = getSelectedIndex();
+            if (idx > 0) {
+                setSelectedIndex(0);
+                setSelectedIndex(idx);
+            }
+        }
     }
 
     static void fixGetBoundsAt(Rectangle b) {
@@ -410,119 +474,92 @@ final class CloseButtonTabbedPane extends JTabbedPane {
         }
     }
 
+    @Override
+    public Color getBackgroundAt(int index) {
+        if( isWindowsLaF() && !isWindowsXPLaF() ) {
+            // In Windows L&F selected and unselected tab may have same color
+            // which make hard to distinguish which tab is selected (especially
+            // in SCROLL_TAB_LAYOUT). In such case manage tab colors manually.
+            Color selected = UIManager.getColor("controlHighlight");
+            Color unselected = UIManager.getColor("control");
+            if (selected.equals(unselected)) {
+                //make unselected tabs darker
+                unselected = new Color(Math.max(selected.getRed() - 12, 0),
+                        Math.max(selected.getGreen() - 12, 0),
+                        Math.max(selected.getBlue() - 12, 0));
+            }
+            return index == getSelectedIndex() ? selected : unselected;
+        }
+        return super.getBackgroundAt(index);
+    }
 
-    private static class CloseButtonListener implements AWTEventListener
-    {
-        private static boolean installed = false;
+    public void propertyChange(PropertyChangeEvent evt) {
+        if (evt.getSource() instanceof Component) {
+            assert evt.getPropertyName().equals(TabbedPaneFactory.NO_CLOSE_BUTTON);
+            Component c = (Component) evt.getSource();
+            int idx = indexOfComponent(c);
+            boolean noCloseButton = (Boolean) evt.getNewValue();
+            setTabComponentAt(idx, noCloseButton ? null : new ButtonTab());
+        }
+    }
+    
+    /**
+     * Custom tab component for JTabbedPane
+     */
+    class ButtonTab extends JPanel {
+        JLabel label;
 
-        private CloseButtonListener() {}
+        public ButtonTab() {
+            super(new FlowLayout(FlowLayout.LEFT, 0, 0));
+            setOpaque(false);
+            label = new JLabel("") {
+                @Override
+                public String getText() {
+                    int i = indexOfTabComponent(ButtonTab.this);
+                    if (i >= 0) {
+                        String tabTitle = getTitleAt(i);
+                        if (!super.getText().equals(tabTitle)) {
+                            setText(tabTitle);
+                        }
+                        return tabTitle;
+                    }
+                    return "";
+                }
 
-        private static synchronized void install() {
-            if (installed)
-                return;
-
-            installed = true;
-            Toolkit.getDefaultToolkit().addAWTEventListener(
-                new CloseButtonListener(),
-                AWTEvent.MOUSE_EVENT_MASK | AWTEvent.MOUSE_MOTION_EVENT_MASK);
+                @Override
+                public Icon getIcon() {
+                    int i = indexOfTabComponent(ButtonTab.this);
+                    Icon icon = i >= 0 ? getIconAt(i) : null;
+                    if (super.getIcon() != icon) {
+                        setIcon(icon);
+                    }
+                    return icon;
+                }
+            };
+            add(label);
+            add(new TabButton());
         }
 
-        public void eventDispatched (AWTEvent ev) {
-            MouseEvent e = (MouseEvent) ev;
-            //#118828
-            if (! (ev.getSource() instanceof Component)) {
-                return;
+        private class TabButton extends JButton implements ActionListener {
+            public TabButton() {
+                int size = 16;
+                setPreferredSize(new Dimension(size, size));
+                setContentAreaFilled(false);
+                setFocusable(false);
+                setBorder(BorderFactory.createEmptyBorder());
+                setBorderPainted(false);
+                setRolloverEnabled(true);
+                addActionListener(this);
+                setIcon(ImageUtilities.image2Icon(getCloseTabImage()));
+                setRolloverIcon(ImageUtilities.image2Icon(getCloseTabRolloverImage()));
+                setPressedIcon(ImageUtilities.image2Icon(getCloseTabPressedImage()));
             }
-        
-            
-            Component c = (Component) e.getSource();
-            while (c != null && !(c instanceof CloseButtonTabbedPane))
-                c = c.getParent();
-            if (c == null)
-                return;
-            final CloseButtonTabbedPane tab = (CloseButtonTabbedPane) c;
 
-            Point p = SwingUtilities.convertPoint((Component) e.getSource(),
-                                                  e.getPoint(),
-                                                  tab);
-
-            if (e.getID() == MouseEvent.MOUSE_CLICKED) {
-                //Not interested in clicked, and it can cause an NPE
-                return;
-            }
-            
-            int index = findTabForCoordinate(tab, p.x, p.y);
-
-            Rectangle r = null;
-            if (index >= 0)
-                r = tab.getCloseButtonBoundsAt(index);
-            if (r == null)
-                r = new Rectangle(0,0,0,0);
-
-            switch(e.getID()) {
-                case MouseEvent.MOUSE_PRESSED:
-                    if (r.contains(p)) {
-                        tab.setPressedCloseButtonIndex(index);
-                        tab.draggedOut = false;
-                        e.consume();
-                        return;
-                    }
-                    break;
-
-                case MouseEvent.MOUSE_RELEASED:
-                    if (r.contains(p) && tab.pressedCloseButtonIndex >= 0) {
-                        Component tc =
-                            tab.getComponentAt(tab.pressedCloseButtonIndex);
-                        tab.reset();
-
-                        tab.fireCloseRequest(tc);
-                        e.consume();
-                        return;
-                    }
-                    else {
-                        tab.reset();
-                    }
-                    break;
-
-                case MouseEvent.MOUSE_ENTERED:
-                    break;
-
-                case MouseEvent.MOUSE_EXITED:
-                    //tab.reset();
-
-                    // XXX(-ttran) when the user clicks on the close button on
-                    // an unfocused (internal) frame the focus is transferred
-                    // to the frame and an unexpected MOUSE_EXITED event is
-                    // fired.  If we call reset() at every MOUSE_EXITED event
-                    // then when the mouse button is released the tab is not
-                    // closed.  See bug #24450
-                    
-                    break;
-
-                case MouseEvent.MOUSE_MOVED:
-                    if (r.contains(p)) {
-                        tab.setMouseOverCloseButtonIndex(index);
-                        tab.draggedOut = false;
-                        e.consume();
-                        return;
-                    }
-                    else if (tab.mouseOverCloseButtonIndex >= 0) {
-                        tab.setMouseOverCloseButtonIndex(-1);
-                        tab.draggedOut = false;
-                        e.consume();
-                    }
-                    break;
-
-                case MouseEvent.MOUSE_DRAGGED:
-                    if (tab.pressedCloseButtonIndex >= 0) {
-                        if (tab.draggedOut != !r.contains(p)) {
-                            tab.draggedOut = !r.contains(p);
-                            tab.repaint(r.x, r.y, r.width + 2+6, r.height + 2+6);
-                        }
-                        e.consume();
-                        return;
-                    }
-                    break;
+            public void actionPerformed(ActionEvent e) {
+                int i = indexOfTabComponent(ButtonTab.this);
+                if (i != -1) {
+                    fireCloseRequest(CloseButtonTabbedPane.this.getComponentAt(i));
+                }
             }
         }
     }
