@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -152,8 +155,7 @@ final class ExplorerActionsImpl {
 
         // Sets action state updater and registers listening on manager and
         // exclipboard.
-        actionStateUpdater = new ActionStateUpdater();
-        manager.addPropertyChangeListener(WeakListeners.propertyChange(actionStateUpdater, manager));
+        actionStateUpdater = new ActionStateUpdater(manager);
 
         Clipboard c = getClipboard();
 
@@ -171,11 +173,12 @@ final class ExplorerActionsImpl {
 
     /** Detach from manager currently being listened on. */
     public synchronized void detach() {
-        if (manager == null) {
+        if (manager == null || actionStateUpdater == null) {
             return;
         }
 
         // Unregisters (weak) listening on manager and exclipboard (see attach).
+        actionStateUpdater.unlisten(manager);
         actionStateUpdater = null;
 
         stopActions();
@@ -669,27 +672,37 @@ final class ExplorerActionsImpl {
      * them if they are frequent and performs the update of actions state. */
     private class ActionStateUpdater implements PropertyChangeListener, ClipboardListener, ActionListener, Runnable {
         private final Timer timer;
+        private final PropertyChangeListener weakL;
 
-        ActionStateUpdater() {
+        ActionStateUpdater(ExplorerManager m) {
             timer = new Timer(200, this);
             timer.setCoalesce(true);
             timer.setRepeats(false);
+            weakL = WeakListeners.propertyChange(this, m);
+            m.addPropertyChangeListener(weakL);
+        }
+
+        void unlisten(ExplorerManager m) {
+            m.removePropertyChangeListener(weakL);
         }
 
         boolean updateScheduled() {
             return timer.isRunning();
         }
 
+        @Override
         public synchronized void propertyChange(PropertyChangeEvent e) {
             timer.restart();
         }
 
+        @Override
         public void clipboardChanged(ClipboardEvent ev) {
             if (!ev.isConsumed()) {
                 Mutex.EVENT.readAccess(this);
             }
         }
 
+        @Override
         public void run() {
             ExplorerManager em = manager;
 
@@ -698,6 +711,7 @@ final class ExplorerActionsImpl {
             }
         }
 
+        @Override
         public void actionPerformed(ActionEvent evt) {
             updateActions(true);
         }
