@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -95,6 +98,8 @@ import org.openide.util.Utilities;
 final class ViewTooltips extends MouseAdapter implements MouseMotionListener {
     /** The default instance, reference counted */
     private static ViewTooltips INSTANCE = null;
+    private static Exception previousUnregister;
+    
     /** A reference count for number of comps listened to */
     private int refcount = 0;
     /** The last known component we were invoked against, nulled on hide() */
@@ -127,9 +132,18 @@ final class ViewTooltips extends MouseAdapter implements MouseMotionListener {
      * from the component's removeNotify() method.
      */
     static void unregister (JComponent comp) {
-        assert INSTANCE != null : "Unregister asymmetrically called";
-        if (INSTANCE.detachFrom (comp) == 0) {
-            INSTANCE.hide();
+        ViewTooltips inst = INSTANCE;
+        boolean doChecks = false;
+        assert doChecks = true;
+        if (doChecks) {
+            if (inst == null) {
+                throw new IllegalStateException("Unregister asymmetrically called for " + comp, previousUnregister);
+            }
+            previousUnregister = new Exception("Unregister for " + comp);
+        }
+
+        if (inst != null && inst.detachFrom(comp) == 0) {
+            inst.hide();
             INSTANCE = null;
         }
     }
@@ -147,7 +161,7 @@ final class ViewTooltips extends MouseAdapter implements MouseMotionListener {
         assert comp instanceof JTree || comp instanceof JList;
         comp.removeMouseMotionListener (this);
         comp.removeMouseListener (this);
-        return refcount--;
+        return --refcount;
     }
     
     public void mouseMoved(MouseEvent e) {
@@ -165,10 +179,12 @@ final class ViewTooltips extends MouseAdapter implements MouseMotionListener {
         hide();
     }
 
+    @Override
     public void mouseEntered(MouseEvent e) {
         hide();
     }
 
+    @Override
     public void mouseExited(MouseEvent e) {
         hide();
     }
@@ -408,8 +424,7 @@ final class ViewTooltips extends MouseAdapter implements MouseMotionListener {
             // run with the above line switch and 
             // -J-Dnb.explorer.hw.cocoahack=true
             
-            PopupFactory result = (PopupFactory) Lookup.getDefault().lookup (
-                    PopupFactory.class);
+            PopupFactory result = Lookup.getDefault().lookup (PopupFactory.class);
             return result == null ? PopupFactory.getSharedInstance() : result;
         } else {
             return PopupFactory.getSharedInstance();
@@ -541,6 +556,7 @@ final class ViewTooltips extends MouseAdapter implements MouseMotionListener {
             setImage (nue);
         }
         
+        @Override
         public Rectangle getBounds() {
             Dimension dd = getPreferredSize();
             return new Rectangle (0, 0, dd.width, dd.height);
@@ -551,6 +567,7 @@ final class ViewTooltips extends MouseAdapter implements MouseMotionListener {
             d = null;
         }
         
+        @Override
         public Dimension getPreferredSize() {
             if (d == null) {
                 d = new Dimension (img.getWidth(), img.getHeight());
@@ -558,10 +575,12 @@ final class ViewTooltips extends MouseAdapter implements MouseMotionListener {
             return d;
         }
         
+        @Override
         public Dimension getSize() {
             return getPreferredSize();
         }
         
+        @Override
         public void paint (Graphics g) {
             Graphics2D g2d = (Graphics2D) g;
             g2d.drawRenderedImage (img, at);
@@ -575,9 +594,13 @@ final class ViewTooltips extends MouseAdapter implements MouseMotionListener {
             }
         }
         
+        @Override
         public void firePropertyChange (String s, Object a, Object b) {}
+        @Override
         public void invalidate() {}
+        @Override
         public void validate() {}
+        @Override
         public void revalidate() {}
     }
     
