@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -377,11 +380,6 @@ final class VisualizerNode extends EventListenerList implements NodeListener, Tr
     * @param ev event describing the change
     */
     public void childrenReordered(NodeReorderEvent ev) {
-        doChildrenReordered(ev);
-    }
-
-    // helper method (called from TreeTableView.sort)
-    void doChildrenReordered(NodeReorderEvent ev) {
         VisualizerChildren ch = children.get();
 
         int[] perm = ev.getPermutation();
@@ -395,38 +393,20 @@ final class VisualizerNode extends EventListenerList implements NodeListener, Tr
         LOG.log(Level.FINER, "childrenReordered - end"); // NOI18N
     }
 
-    void reorderChildren(Comparator<VisualizerNode> c) {
-        assert SwingUtilities.isEventDispatchThread();
-
-        VisualizerChildren ch = children.get();
-
-        if (ch == null) {
-            return;
-        }
-
-        new VisualizerEvent.Reordered(ch, c, null).run();
-    }
-
-    void naturalOrder() {
-        //force new creation of the children list in the natural order
-        children.clear();
-        getChildren();
-
-        //sort the children list with a dummy comparator to throw events needed
-        reorderChildren(
-            new Comparator<VisualizerNode>() {
-                public int compare(VisualizerNode o1, VisualizerNode o2) {
-                    return 0;
-                }
-            }
-        );
-    }
-
     /** Fired when the node is deleted.
     * @param ev event describing the node
     */
+    @Override
     public void nodeDestroyed(NodeEvent ev) {
-        // ignore for now
+        node = Node.EMPTY;
+        parent = null;
+        Enumeration<VisualizerNode> ch = getChildren(false).children(false);
+        while (ch.hasMoreElements()) {
+            final VisualizerNode v = ch.nextElement();
+            if (v != null) {
+                v.nodeDestroyed(ev);
+            }
+        }
     }
 
     /** Change in the node properties (icon, etc.)
