@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -60,7 +63,6 @@ import javax.swing.JScrollPane;
 import javax.swing.SwingUtilities;
 import org.netbeans.junit.Log;
 import org.netbeans.junit.NbTestCase;
-import org.netbeans.junit.RandomlyFails;
 import org.openide.explorer.ExplorerManager;
 import org.openide.nodes.AbstractNode;
 import org.openide.nodes.Children;
@@ -78,8 +80,11 @@ import org.openide.util.actions.SystemAction;
  * @author  Marian Petras, Andrei Badea
  */
 public final class TreeViewTest extends NbTestCase {
+
+    protected @Override int timeOut() {
+        return 500000;
+    }
     
-    private TestTreeView treeView;
     private ExplorerWindow testWindow;
     private volatile boolean isScrolledDown;
     private final Object semaphore = new Object();
@@ -99,10 +104,8 @@ public final class TreeViewTest extends NbTestCase {
         }
     }
 
-
-
 //    public static TreeViewTest suite() {
-//        return new TreeViewTest("testSetSelectedNodeIsSynchronizedLazy");
+//        return new TreeViewTest("test");
 //    }
 
     /**
@@ -113,16 +116,6 @@ public final class TreeViewTest extends NbTestCase {
     public void testAutoscrollOnOff() throws InterruptedException {
         assert !EventQueue.isDispatchThread();
         
-        testWindow = new ExplorerWindow();
-        testWindow.getContentPane().add(treeView = new TestTreeView());
-        
-        class WindowDisplayer implements Runnable {
-            public void run() {
-                testWindow.pack();
-                testWindow.setVisible(true);
-            }
-        }
-        
         class Detector implements Runnable {
             public void run() {
                 if (!EventQueue.isDispatchThread()) {
@@ -130,7 +123,7 @@ public final class TreeViewTest extends NbTestCase {
                     return;
                 }
                 
-                isScrolledDown = !treeView.isUp();
+                isScrolledDown = !testWindow.treeView.isUp();
                 
                 synchronized (semaphore) {
                     semaphore.notify();
@@ -148,26 +141,18 @@ public final class TreeViewTest extends NbTestCase {
             public void run() {
                 assert (part == 1) || (part == 2);
                 if (part == 1) {
-                    treeView.collapse();
-                    treeView.scrollUp();
-                    assert treeView.isUp();
+                    testWindow.treeView.collapse();
+                    testWindow.treeView.scrollUp();
+                    assert testWindow.treeView.isUp();
                 } else {
-                    treeView.setAutoscroll(autoscroll);
-                    treeView.expand(); //<-- posts a request to the RequestProcessor
+                    testWindow.treeView.setAutoscroll(autoscroll);
+                    testWindow.treeView.expand(); //<-- posts a request to the RequestProcessor
                     RequestProcessor.getDefault().post(new Detector(), 1000 /*ms*/);
                 }
             }
         }
-
-        try {
-            EventQueue.invokeAndWait(new WindowDisplayer());
-        } catch (InvocationTargetException ex) {
-            ex.printStackTrace();
-        }
-        
-        //Wait for the AWT thread to actually display the dialog:
-        Thread.sleep(5000);
-        
+        testWindow = new ExplorerWindow();
+        testWindow.showWindow();
         EventQueue.invokeLater(new Tester(true, 1));
         Thread.sleep(2000);      //wait for update of the screen
         EventQueue.invokeLater(new Tester(true, 2));
@@ -187,6 +172,52 @@ public final class TreeViewTest extends NbTestCase {
         EventQueue.invokeLater(new Tester(true, 1));    //just collapse the tree
         Thread.sleep(2000);
     }
+
+    public void testExpandNodePreparedOutsideOfAWT() throws Exception {
+        assertFalse(EventQueue.isDispatchThread());
+
+        class OutOfAWT extends Keys {
+            Exception noAWTAddNotify;
+            Exception noAWTCreateNodes;
+
+            public OutOfAWT(boolean lazy, String... args) {
+                super(lazy, args);
+            }
+
+            @Override
+            protected void addNotify() {
+                if (EventQueue.isDispatchThread()) {
+                    noAWTAddNotify = new Exception();
+                }
+                super.addNotify();
+            }
+
+            @Override
+            protected Node[] createNodes(Object key) {
+                if (EventQueue.isDispatchThread()) {
+                    noAWTCreateNodes = new Exception();
+                }
+                return super.createNodes(key);
+            }
+        }
+        AbstractNode root = new AbstractNode(new Children.Array());
+        final OutOfAWT ch = new OutOfAWT(false, "A", "B", "C");
+        AbstractNode an = new AbstractNode(ch);
+        root.getChildren().add(new Node[] { an });
+        testWindow = new ExplorerWindow();
+        testWindow.showWindow();
+        testWindow.getExplorerManager().setRootContext(root);
+
+        testWindow.treeView.expandNode(an);
+        Thread.sleep(2000);
+
+        if (ch.noAWTAddNotify != null) {
+            throw ch.noAWTAddNotify;
+        }
+        if (ch.noAWTCreateNodes != null) {
+            throw ch.noAWTCreateNodes;
+        }
+    }
     
     
     private static final class TestTreeView extends BeanTreeView {
@@ -282,21 +313,45 @@ public final class TreeViewTest extends NbTestCase {
     }
     
     
-    private static final class ExplorerWindow extends JFrame
-                               implements ExplorerManager.Provider {
+    private final class ExplorerWindow extends JFrame
+            implements ExplorerManager.Provider, Runnable {
         
         private final ExplorerManager explManager = new ExplorerManager();
+        TestTreeView treeView;
         
         ExplorerWindow() {
             super("TreeView test");                                     //NOI18N
+            getContentPane().add(treeView = new TestTreeView());
         }
         
         public ExplorerManager getExplorerManager() {
             return explManager;
         }
-        
+        public void run() {
+            pack();
+            setVisible(true);
+        }
+
+        void waitShown() throws InterruptedException {
+            while (!isShowing()) {
+                Thread.sleep(100);
+            }
+            Thread.sleep(500);
+        }
+
+        void showWindow() {
+            try {
+                EventQueue.invokeAndWait(this);
+                waitShown();
+            } catch (InterruptedException ex) {
+                Exceptions.printStackTrace(ex);
+            } catch (InvocationTargetException ex) {
+                ex.printStackTrace();
+            }
+            waitAWT();
+        }
     }
-    
+
     /**
      * Used as the preferred actions by the nodes below
      */
@@ -400,11 +455,17 @@ public final class TreeViewTest extends NbTestCase {
         assertEquals("The context lookup created by TreeView.takeAction() should contain the node only once.", 1, count);
     }
 
-    private void waitAWT() throws InterruptedException, InvocationTargetException {
-        SwingUtilities.invokeAndWait(new Runnable() {
-            public void run() {
-            }
-        });
+    private static void waitAWT() {
+        try {
+            SwingUtilities.invokeAndWait(new Runnable() {
+                public void run() {
+                }
+            });
+        } catch (InterruptedException ex) {
+            Exceptions.printStackTrace(ex);
+        } catch (InvocationTargetException ex) {
+            Exceptions.printStackTrace(ex);
+        }
     }
     private void setSelectedNodes(final Node[] arr) throws Exception {
         class R implements Runnable {
@@ -433,56 +494,13 @@ public final class TreeViewTest extends NbTestCase {
     private void doPreventGCOfVisibleNodes(boolean lazy) throws Exception {
         assert !EventQueue.isDispatchThread();
 
-        testWindow = new ExplorerWindow();
-        testWindow.getContentPane().add(treeView = new TestTreeView());
-
-        class WindowDisplayer implements Runnable {
-            public void run() {
-                testWindow.pack();
-                testWindow.setVisible(true);
-            }
-
-            void waitShown() throws InterruptedException {
-                while (!testWindow.isShowing()) {
-                    Thread.sleep(100);
-                }
-                Thread.sleep(500);
-            }
-        }
-
-        class Detector implements Runnable {
-            public void run() {
-                if (!EventQueue.isDispatchThread()) {
-                    EventQueue.invokeLater(this);
-                    return;
-                }
-
-                isScrolledDown = !treeView.isUp();
-
-                synchronized (semaphore) {
-                    semaphore.notify();
-                }
-            }
-        }
-
         Keys keys = new Keys(lazy, "A", "B", "C");
         AbstractNode node = new AbstractNode(keys);
         node.setName(getName());
         AbstractNode root = node;
 
-        try {
-            WindowDisplayer wd = new WindowDisplayer();
-
-            testWindow.getExplorerManager().setRootContext(root);
-
-
-            EventQueue.invokeAndWait(wd);
-            wd.waitShown();
-        } catch (InvocationTargetException ex) {
-            ex.printStackTrace();
-        }
-
-        waitAWT();
+        testWindow = new ExplorerWindow();
+        testWindow.showWindow();
         testWindow.getExplorerManager().setRootContext(root);
         assertSame("Root is OK", root, testWindow.getExplorerManager().getRootContext());
 
@@ -509,6 +527,45 @@ public final class TreeViewTest extends NbTestCase {
         fail("Node shall not be GCed: " + ref.get());
     }
 
+    public void testNodesGCedAfterSetChildrenLazy() {
+        doTestNodesGCedAfterSetChildren(true);
+    }
+
+    public void testNodesGCedAfterSetChildrenEager() {
+        doTestNodesGCedAfterSetChildren(false);
+    }
+
+    void doTestNodesGCedAfterSetChildren(boolean lazy) {
+        Keys keys = new Keys(lazy, "A", "B", "C");
+        class MyNode extends AbstractNode {
+
+            public MyNode(Children children) {
+                super(children);
+            }
+            void callSetChildren(Children newCh) {
+                setChildren(newCh);
+            }
+        }
+        MyNode root = new MyNode(keys);
+        root.setName(getName());
+        testWindow = new ExplorerWindow();
+        testWindow.showWindow();
+        testWindow.getExplorerManager().setRootContext(root);
+        testWindow.getExplorerManager().setExploredContext(root);
+        Node[] nodes = root.getChildren().getNodes();
+        try {
+            testWindow.getExplorerManager().setSelectedNodes(nodes);
+        } catch (PropertyVetoException ex) {
+            Exceptions.printStackTrace(ex);
+        }
+        waitAWT();
+        Reference<Object> ref = new WeakReference<Object>(nodes[2]);
+        nodes = null;
+        root.callSetChildren(Children.LEAF);
+        waitAWT();
+        assertGC("should gc children", ref);
+    }
+
     public void testSetSelectedNodeIsSynchronizedEager() throws Exception {
         doSetSelectedNodeIsSynchronized(false);
     }
@@ -520,36 +577,14 @@ public final class TreeViewTest extends NbTestCase {
     private void doSetSelectedNodeIsSynchronized(boolean lazy) throws Exception {
         assert !EventQueue.isDispatchThread();
 
-        testWindow = new ExplorerWindow();
-        testWindow.getContentPane().add(treeView = new TestTreeView());
-
-        class WindowDisplayer implements Runnable {
-            public void run() {
-                testWindow.pack();
-                testWindow.setVisible(true);
-            }
-
-            void waitShown() throws InterruptedException {
-                while (!testWindow.isShowing()) {
-                    Thread.sleep(100);
-                }
-                Thread.sleep(500);
-            }
-        }
-
         final Keys keys = new Keys(lazy, "A", "B", "C");
         AbstractNode node = new AbstractNode(keys);
         node.setName(getName());
         final AbstractNode root = node;
 
-        try {
-            WindowDisplayer wd = new WindowDisplayer();
-            testWindow.getExplorerManager().setRootContext(root);
-            EventQueue.invokeAndWait(wd);
-            wd.waitShown();
-        } catch (InvocationTargetException ex) {
-            ex.printStackTrace();
-        }
+        testWindow = new ExplorerWindow();
+        testWindow.getExplorerManager().setRootContext(root);
+        testWindow.showWindow();
 
         waitAWT();
         testWindow.getExplorerManager().setRootContext(root);
@@ -597,38 +632,14 @@ public final class TreeViewTest extends NbTestCase {
     private void doTestPartialNodeSelection(boolean lazy) throws Exception {
         assert !EventQueue.isDispatchThread();
 
-        testWindow = new ExplorerWindow();
-        testWindow.getContentPane().add(treeView = new TestTreeView());
-
-        class WindowDisplayer implements Runnable {
-            public void run() {
-                testWindow.pack();
-                testWindow.setVisible(true);
-            }
-
-            void waitShown() throws InterruptedException {
-                while (!testWindow.isShowing()) {
-                    Thread.sleep(100);
-                }
-                Thread.sleep(500);
-            }
-        }
-
         final Keys keys = new Keys(lazy, "A", "B", "-B", "C");
         AbstractNode node = new AbstractNode(keys);
         node.setName(getName());
         final AbstractNode root = node;
 
-        try {
-            WindowDisplayer wd = new WindowDisplayer();
-            testWindow.getExplorerManager().setRootContext(root);
-            EventQueue.invokeAndWait(wd);
-            wd.waitShown();
-        } catch (InvocationTargetException ex) {
-            ex.printStackTrace();
-        }
-
-        waitAWT();
+        testWindow = new ExplorerWindow();
+        testWindow.getExplorerManager().setRootContext(root);
+        testWindow.showWindow();
         testWindow.getExplorerManager().setRootContext(root);
         assertSame("Root is OK", root, testWindow.getExplorerManager().getRootContext());
 
