@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -46,6 +49,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -53,6 +57,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.TreeMap;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import org.openide.filesystems.FileChangeAdapter;
@@ -80,8 +85,7 @@ public final class MIMEResolverImpl {
     // enable some tracing
     private static final Logger ERR = Logger.getLogger(MIMEResolverImpl.class.getName());
         
-    private static final boolean CASE_INSENSITIVE =
-        Utilities.isWindows() || Utilities.getOperatingSystem() == Utilities.OS_VMS;
+    private static final boolean CASE_INSENSITIVE = Utilities.getOperatingSystem() == Utilities.OS_VMS;
 
     // notification limit in bytes for reading file content. It should not exceed 4192 (4kB) because it is read in one disk touch.
     private static final int READ_LIMIT = 4000;
@@ -104,6 +108,7 @@ public final class MIMEResolverImpl {
 
     /** Returns resolvable MIME Types for given declarative resolver. */
     public static String[] getMIMETypes(MIMEResolver resolver) {
+        ((Impl) resolver).init();  // #171312 - resolver must be parsed
         return ((Impl)resolver).implResolvableMIMETypes;
     }
 
@@ -243,13 +248,12 @@ public final class MIMEResolverImpl {
         });
     }
 
-    /** Returns map of all registered MIMEResolver instances in revers order,
+    /** Lists registered MIMEResolver instances in reverse order,
      * i.e. first are ones with lower priority (position attribute higher)
      * and last are ones with highest prority (position attribute lower).
-     * @return map of all registered MIMEResolver instances in revers order
-     * (highest priority last)
+     * @return list of all registered MIMEResolver instances in reverse order
      */
-    public static Map<Integer, FileObject> getOrderedResolvers() {
+    public static Collection<? extends FileObject> getOrderedResolvers() {
         // scan resolvers and order them to be able to assign extension to mime type from resolver with the lowest position
         FileObject[] resolvers = FileUtil.getConfigFile(MIME_RESOLVERS_PATH).getChildren();
         TreeMap<Integer, FileObject> orderedResolvers = new TreeMap<Integer, FileObject>(Collections.reverseOrder());
@@ -263,7 +267,7 @@ public final class MIMEResolverImpl {
             }
             orderedResolvers.put(position, mimeResolverFO);
         }
-        return orderedResolvers;
+        return orderedResolvers.values();
     }
 
     // MIMEResolver ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -309,14 +313,14 @@ public final class MIMEResolverImpl {
             FileElement[] smell2 = smell;  //#163378, #157838 - copy to prevent concurrent modification and not synchronize to prevent deadlock
             // smell is filled in reverse order
             for (int i = smell2.length - 1; i >= 0; i--) {
-                ERR.fine("findMIMEType - smell.resolve.");
+                if (ERR.isLoggable(Level.FINE)) ERR.fine("findMIMEType - smell.resolve.");
                 String s = smell2[i].resolve(fo);
                 if (s != null) {
                     if (s.equals(FileElement.EXIT_MIME_TYPE)) {
                         // if file matches conditions and exit element is present, do not continue in loop and return null
                         return null;
                     }
-                    ERR.fine("MIMEResolverImpl.findMIMEType(" + fo + ")=" + s);  // NOI18N
+                    if (ERR.isLoggable(Level.FINE)) ERR.fine("MIMEResolverImpl.findMIMEType(" + fo + ")=" + s);  // NOI18N
                     return s;
                 }
             }
@@ -759,7 +763,6 @@ public final class MIMEResolverImpl {
             private final int range;
             private final boolean ignoreCase;
             private FilePattern inner;
-            private int pointer = 0;
             private final byte[] bytes;
             private final int valueLength;
 
@@ -779,35 +782,32 @@ public final class MIMEResolverImpl {
                 this.inner = inner;
             }
 
-            private boolean match(byte b) {
-                if (pointer < valueLength) {
-                    if (b == bytes[pointer]) {
-                        pointer++;
-                    } else {
-                        pointer = 0;
-                        return false;
-                    }
+            private boolean match(byte b, AtomicInteger pointer) {
+                if (b == bytes[pointer.get()]) {
+                    return pointer.incrementAndGet() >= valueLength;
+                } else {
+                    pointer.set(0);
+                    return false;
                 }
-                return pointer >= valueLength;
             }
 
             /** Read from given file and compare byte-by-byte if pattern
              * appers in given range.
              */
             public boolean match(FileObject fo) throws IOException {
-                pointer = 0;
                 InputStream is = null;
                 boolean matched = false;
                 try {
                     is = fo.getInputStream();  // it is CachedInputStream, so you can call getInputStream and read more times without performance penalty
                     byte[] byteRange = new byte[range];
                     int read = is.read(byteRange);
+                    AtomicInteger pointer = new AtomicInteger(0);
                     for (int i = 0; i < read; i++) {
                         byte b = byteRange[i];
                         if (ignoreCase) {
                             b = (byte) Character.toLowerCase(b);
                         }
-                        if (match(b)) {
+                        if (match(b, pointer)) {
                             matched = true;
                             break;
                         }
