@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -58,6 +61,7 @@ import java.util.Enumeration;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.StringTokenizer;
+import java.util.logging.Level;
 import org.openide.util.Enumerations;
 import org.openide.util.NbBundle;
 import org.openide.util.UserQuestionException;
@@ -116,7 +120,12 @@ public abstract class FileObject extends Object implements Serializable {
     public FileObject copy(FileObject target, String name, String ext)
     throws IOException {
         if (isFolder()) {
-            throw new IOException(NbBundle.getBundle(FileObject.class).getString("EXC_FolderCopy"));
+            FileObject peer = target.createFolder(name);
+            FileUtil.copyAttributes(this, peer);
+            for (FileObject fo : getChildren()) {
+                fo.copy(peer, fo.getName(), fo.getExt());
+            }
+            return peer;
         }
 
         FileObject dest = FileUtil.copyFileImpl(this, target, name, ext);
@@ -413,6 +422,62 @@ public abstract class FileObject extends Object implements Serializable {
     */
     public abstract void removeFileChangeListener(FileChangeListener fcl);
 
+
+    /** Adds a listener to this {@link FileObject} and all its children and
+     * children or its children.
+     * It is guaranteed that whenever a change
+     * is made via the FileSystem API itself under this {@link FileObject}
+     * that it is notified to the <code>fcl</code> listener. Whether external
+     * changes (if they make sense) are detected and
+     * notified depends on actual implementation. As some implementations may
+     * need to perform non-trivial amount of work during initialization of
+     * listeners, this methods can take long time. Usage of this method may
+     * consume a lot of system resources and as such it shall be used with care.
+     * Traditional {@link #addFileChangeListener(org.openide.filesystems.FileChangeListener)}
+     * is definitely preferred variant.
+     * <p class="nonnormative">
+     * If you are running with the MasterFS module enabled, it guarantees
+     * that for files backed with real {@link File}, the system initializes
+     * itself to detect external changes on the whole subtree.
+     * This requires non-trivial amount of work and especially on slow
+     * disks (aka networks ones) may take a long time to add the listener
+     * and also refresh the system when {@link FileObject#refresh()}
+     * and especially {@link FileUtil#refreshAll()} is requested.
+     * </p>
+     *
+     * @param fcl the listener to register
+     * @since 7.28
+     */
+    public void addRecursiveListener(FileChangeListener fcl) {
+        if (!isFolder()) {
+            addFileChangeListener(fcl);
+            return;
+        }
+        try {
+            boolean allowsExternalChanges = getFileSystem() instanceof LocalFileSystem;
+            getFileSystem().addFileChangeListener(new RecursiveListener(this, fcl, allowsExternalChanges));
+        } catch (FileStateInvalidException ex) {
+            ExternalUtil.LOG.log(Level.FINE, "Cannot remove listener from " + this, ex);
+        }
+    }
+
+    /** Removes listener previously added by {@link #addRecursiveListener(org.openide.filesystems.FileChangeListener)}
+     *
+     * @param fcl the listener to remove
+     * @since 7.28
+     */
+    public void removeRecursiveListener(FileChangeListener fcl) {
+        if (!isFolder()) {
+            removeFileChangeListener(fcl);
+            return;
+        }
+        try {
+            getFileSystem().removeFileChangeListener(new RecursiveListener(this, fcl, false));
+        } catch (FileStateInvalidException ex) {
+            ExternalUtil.LOG.log(Level.FINE, "Cannot remove listener from " + this, ex);
+        }
+    }
+
     /** Fire data creation event.
     * @param en listeners that should receive the event
     * @param fe the event to fire in this object
@@ -692,7 +757,7 @@ public abstract class FileObject extends Object implements Serializable {
     * an "archive" feature it should archive only <code>.java</code> files.
     * @param b true if the file should be considered important
     * @deprecated No longer used. Instead use
-    * <a href="@PROJECTS/QUERIES@/org/netbeans/api/queries/SharabilityQuery.html"><code>SharabilityQuery</code></a>.
+    * <a href="@org-netbeans-modules-queries@/org/netbeans/api/queries/SharabilityQuery.html"><code>SharabilityQuery</code></a>.
     */
     @Deprecated
     public abstract void setImportant(boolean b);
@@ -961,7 +1026,7 @@ public abstract class FileObject extends Object implements Serializable {
 
     interface PriorityFileChangeListener extends FileChangeListener {}
 
-    private class ED extends FileSystem.EventDispatcher {
+    private static class ED extends FileSystem.EventDispatcher {
         private FCLSupport.Op op;
         private Enumeration<FileChangeListener> en;
         final private List<FileChangeListener> fsList;
@@ -982,11 +1047,8 @@ public abstract class FileObject extends Object implements Serializable {
             }
             ListenerList<FileChangeListener> fsll = (fs != null) ? fs.getFCLSupport().listeners : null;
             ListenerList<FileChangeListener> repll = (fs != null && fs.getRepository() != null) ? fs.getRepository().getFCLSupport().listeners : null;
-            fsList = (fsll != null) ? new ArrayList<FileChangeListener>(fsll.getAllListeners()) :
-                new ArrayList<FileChangeListener>();
-            repList = (repll != null) ? new ArrayList<FileChangeListener>(repll.getAllListeners()) :
-                new ArrayList<FileChangeListener>();
-            
+            fsList = ListenerList.allListeners(fsll);
+            repList = ListenerList.allListeners(repll);
         }
 
         public ED(Enumeration<FileChangeListener> en, FileEvent fe) {
@@ -996,7 +1058,7 @@ public abstract class FileObject extends Object implements Serializable {
         /** @param onlyPriority if true then invokes only priority listeners
          *  else all listeners are invoked.
          */
-        protected void dispatch(boolean onlyPriority) {
+        protected void dispatch(boolean onlyPriority, Collection<Runnable> postNotify) {
             if (this.op == null) {
                 this.op = fe.getFile().isFolder() ? FCLSupport.Op.FOLDER_CREATED : FCLSupport.Op.DATA_CREATED;
             }
@@ -1011,7 +1073,7 @@ public abstract class FileObject extends Object implements Serializable {
 
                     continue;
                 }
-                FCLSupport.dispatchEvent(fcl, fe, op);
+                FCLSupport.dispatchEvent(fcl, fe, op, postNotify);
             }
 
             if (onlyPriority) {
@@ -1049,14 +1111,14 @@ public abstract class FileObject extends Object implements Serializable {
                 }
                 if (fs != null && fsList != null) {
                     for (FileChangeListener fcl : fsList) {
-                        FCLSupport.dispatchEvent(fcl, fe, op);
+                        FCLSupport.dispatchEvent(fcl, fe, op, postNotify);
                     }  
                 }
 
 
                 if (rep != null && repList != null) {
                     for (FileChangeListener fcl : repList) {
-                        FCLSupport.dispatchEvent(fcl, fe, op);
+                        FCLSupport.dispatchEvent(fcl, fe, op, postNotify);
                     }                      
                 }
             }
