@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -67,6 +70,7 @@ import java.util.Set;
 import java.util.Stack;
 import java.util.StringTokenizer;
 import java.util.WeakHashMap;
+import java.util.concurrent.Callable;
 import java.util.jar.JarEntry;
 import java.util.jar.JarInputStream;
 import java.util.logging.Level;
@@ -79,6 +83,7 @@ import org.openide.filesystems.FileSystem.AtomicAction;
 import org.openide.util.Exceptions;
 import org.openide.util.NbBundle;
 import org.openide.util.Parameters;
+import org.openide.util.RequestProcessor;
 import org.openide.util.Utilities;
 import org.openide.util.WeakListeners;
 
@@ -87,6 +92,8 @@ import org.openide.util.WeakListeners;
  */
 public final class FileUtil extends Object {
 
+    private static final RequestProcessor REFRESH_RP = new RequestProcessor("FileUtil-Refresh-All");//NOI18N
+    private static RequestProcessor.Task refreshTask = null;
     /** Contains mapping of FileChangeListener to File. */
     private static final Map<FileChangeListener,Map<File,Holder>> holders = new WeakHashMap<FileChangeListener,Map<File,Holder>>();
 
@@ -112,6 +119,8 @@ public final class FileUtil extends Object {
         transientAttributes.add("SystemFileSystem.localizingBundle"); // NOI18N
         transientAttributes.add("SystemFileSystem.icon"); // NOI18N
         transientAttributes.add("SystemFileSystem.icon32"); // NOI18N
+        transientAttributes.add("displayName"); // NOI18N
+        transientAttributes.add("iconBase"); // NOI18N
         transientAttributes.add("position"); // NOI18N
     }
 
@@ -159,14 +168,38 @@ public final class FileUtil extends Object {
      * @since 7.7
      */
     public static void refreshAll() {
-        refreshFor(File.listRoots());
-        try {
-            getConfigRoot().getFileSystem().refresh(true);
-        } catch (FileStateInvalidException ex) {
-            Exceptions.printStackTrace(ex);
+        // run just one refreshTask in time and always wait for finish
+        final RequestProcessor.Task taskToWaitFor;  // prevent possible NPE with only refreshTask.waitFinished
+        synchronized (REFRESH_RP) {
+            if (refreshTask != null) {
+                refreshTask.cancel();
+            } else {
+                refreshTask = REFRESH_RP.create(new Runnable() {
+
+                    public void run() {
+                        LOG.fine("refreshAll - started");  //NOI18N
+                        refreshFor(File.listRoots());
+                        try {
+                            getConfigRoot().getFileSystem().refresh(true);
+                        } catch (FileStateInvalidException ex) {
+                            Exceptions.printStackTrace(ex);
+                        } finally {
+                            LOG.fine("refreshAll - finished");  //NOI18N
+                            synchronized (REFRESH_RP) {
+                                refreshTask = null;
+                            }
+                        }
+                    }
+                });
+            }
+            taskToWaitFor = refreshTask;
+            refreshTask.schedule(0);
+            LOG.fine("refreshAll - scheduled");  //NOI18N
         }
-    }         
-    
+        taskToWaitFor.waitFinished();
+        LOG.fine("refreshAll - finished");  //NOI18N
+    }
+
     /**
      * Registers <code>listener</code> so that it will receive
      * <code>FileEvent</code>s from <code>FileSystem</code>s providing instances
@@ -255,6 +288,10 @@ public final class FileUtil extends Object {
      * @since org.openide.filesystems 7.20
      */
     public static void removeFileChangeListener(FileChangeListener listener, File path) {
+        removeFileChangeListenerImpl(listener, path);
+    }
+
+    private static FileChangeListener removeFileChangeListenerImpl(FileChangeListener listener, File path) {
         assert path.equals(FileUtil.normalizeFile(path)) : "Need to normalize " + path + "!";  //NOI18N
         synchronized (holders) {
             Map<File, Holder> f2H = holders.get(listener);
@@ -265,9 +302,84 @@ public final class FileUtil extends Object {
                 throw new IllegalArgumentException(listener + " was not listening to " + path + "; only to " + f2H.keySet()); // NOI18N
             }
             // remove Holder instance from map and call run to unregister its current listener
-            f2H.remove(path).run();
+            Holder h = f2H.remove(path);
+            h.run();
+            return h.get();
         }
     }
+    /**
+     * Works like {@link #addRecursiveListener(org.openide.filesystems.FileChangeListener, java.io.File, java.util.concurrent.Callable)
+     * addRecursiveListener(listener, path, null)}.
+     *
+     * @param listener FileChangeListener to listen to changes in path
+     * @param path File path to listen to (even not existing)
+     *
+     * @since org.openide.filesystems 7.28
+     */
+    public static void addRecursiveListener(FileChangeListener listener, File path) {
+        addFileChangeListener(new DeepListener(listener, path, null), path);
+    }
+
+    /**
+     * Adds a listener to changes under given path. It permits you to listen to a file
+     * which does not yet exist, or continue listening to it after it is deleted and recreated, etc.
+     * <br/>
+     * When given path represents a file ({@code path.isDirectory() == false}), this
+     * code behaves exectly like {@link #addFileChangeListener(org.openide.filesystems.FileChangeListener, java.io.File)}.
+     * Usually the path shall represent a folder ({@code path.isDirectory() == true})
+     * <ul>
+     * <li>fileFolderCreated event is fired when the folder is created or a child folder created</li>
+     * <li>fileDataCreated event is fired when a child file is created</li>
+     * <li>fileDeleted event is fired when the folder is deleted or a child file/folder removed</li>
+     * <li>fileChanged event is fired when a child file is modified</li>
+     * <li>fileRenamed event is fired when the folder is renamed or a child file/folder is renamed</li>
+     * <li>fileAttributeChanged is fired when FileObject's attribute is changed</li>
+     *</ul>
+     * The above events are delivered for changes in all subdirectories (recursively).
+     * It is guaranteed that with each change at least one event is generated.
+     * For example adding a folder does not notify about content of the folder,
+     * hence one event is delivered.
+     *
+     * Can only add a given [listener, path] pair once. However a listener can
+     * listen to any number of paths. Note that listeners are always held weakly
+     * - if the listener is collected, it is quietly removed.
+     *
+     * <div class="nonnormative">
+     * As registering of the listener can take a long time, especially on deep
+     * hierarchies, it is possible provide a callback <code>stop</code>.
+     * This stop object is guaranteed to be called once per every folder on the
+     * default (when masterfs module is included) implemention. If the call
+     * to <code>stop.call()</code> returns true, then the registration of
+     * next recursive items is interrupted. The listener may or may not get
+     * some events from already registered folders.
+     * </div>
+     *
+     * @param listener FileChangeListener to listen to changes in path
+     * @param path File path to listen to (even not existing)
+     * @param stop an interface to interrupt the process of registering
+     *    the listener. If the <code>call</code> returns true, the process
+     *    of registering the listener is immediately interrupted
+     *
+     * @see FileObject#addRecursiveListener
+     * @since org.openide.filesystems 7.37
+     */
+    public static void addRecursiveListener(FileChangeListener listener, File path, Callable<Boolean> stop) {
+        addFileChangeListener(new DeepListener(listener, path, stop), path);
+    }
+
+    /**
+     * Removes a listener to changes under given path.
+     * @param listener FileChangeListener to be removed
+     * @param path File path in which listener was listening
+     * @throws IllegalArgumentException if listener was not listening to given path
+     *
+     * @see FileObject#removeRecursiveListener
+     * @since org.openide.filesystems 7.28
+     */
+    public static void removeRecursiveListener(FileChangeListener listener, File path) {
+        DeepListener dl = (DeepListener)removeFileChangeListenerImpl(new DeepListener(listener, path, null), path);
+        dl.run();
+    }
 
     /** Holds FileChangeListener and File pair and handle movement of auxiliary
      * FileChangeListener to the first existing upper folder and firing appropriate events.
@@ -316,6 +428,7 @@ public final class FileUtil extends Object {
         private void someChange() {
             FileChangeListener listener;
             boolean wasOnTarget;
+            FileObject currentNew;
             synchronized (this) {
                 if (current == null) {
                     return;
@@ -326,14 +439,15 @@ public final class FileUtil extends Object {
                 }
                 wasOnTarget = isOnTarget;
                 locateCurrent();
+                currentNew = current;
             }
             if (isOnTarget && !wasOnTarget) {
                 // fire events about itself creation (it is difference from FCL
                 // on FileOject - it cannot be fired because we attach FCL on already existing FileOject
-                if (current.isFolder()) {
-                    listener.fileFolderCreated(new FileEvent(current));
+                if (currentNew.isFolder()) {
+                    listener.fileFolderCreated(new FileEvent(currentNew));
                 } else {
-                    listener.fileDataCreated(new FileEvent(current));
+                    listener.fileDataCreated(new FileEvent(currentNew));
                 }
             }
         }
@@ -342,7 +456,9 @@ public final class FileUtil extends Object {
             if (fe.getSource() == current) {
                 if (isOnTarget) {
                     FileChangeListener listener = get();
-                    if (listener != null) {
+                    if (listener instanceof DeepListener) {
+                        ((DeepListener)listener).fileChanged(fe, true);
+                    } else if (listener != null) {
                         listener.fileChanged(fe);
                     }
                 } else {
@@ -355,7 +471,9 @@ public final class FileUtil extends Object {
             if (fe.getSource() == current) {
                 if (isOnTarget) {
                     FileChangeListener listener = get();
-                    if (listener != null) {
+                    if (listener instanceof DeepListener) {
+                        ((DeepListener)listener).fileDeleted(fe, true);
+                    } else if (listener != null) {
                         listener.fileDeleted(fe);
                     }
                 }
@@ -367,7 +485,9 @@ public final class FileUtil extends Object {
             if (fe.getSource() == current) {
                 if (isOnTarget) {
                     FileChangeListener listener = get();
-                    if (listener != null) {
+                    if (listener instanceof DeepListener) {
+                        ((DeepListener)listener).fileDataCreated(fe, true);
+                    } else if (listener != null) {
                         listener.fileDataCreated(fe);
                     }
                 } else {
@@ -380,7 +500,9 @@ public final class FileUtil extends Object {
             if (fe.getSource() == current) {
                 if (isOnTarget) {
                     FileChangeListener listener = get();
-                    if (listener != null) {
+                    if (listener instanceof DeepListener) {
+                        ((DeepListener)listener).fileFolderCreated(fe, true);
+                    } else if (listener != null) {
                         listener.fileFolderCreated(fe);
                     }
                 } else {
@@ -393,7 +515,9 @@ public final class FileUtil extends Object {
             if (fe.getSource() == current) {
                 if (isOnTarget) {
                     FileChangeListener listener = get();
-                    if (listener != null) {
+                    if (listener instanceof DeepListener) {
+                        ((DeepListener)listener).fileRenamed(fe, true);
+                    } else if (listener != null) {
                         listener.fileRenamed(fe);
                     }
                 }
@@ -828,19 +952,19 @@ public final class FileUtil extends Object {
         File retVal = (File) fo.getAttribute("java.io.File"); // NOI18N;        
 
         if (retVal == null) {
-            URL fileURL = null;
-            int[] types = new int[] { URLMapper.INTERNAL, URLMapper.EXTERNAL };
-
-            for (int i = 0; ((fileURL == null) || "file".equals(fileURL.getProtocol())) && (i < types.length); i++) { // NOI18N
-                fileURL = URLMapper.findURL(fo, types[i]);
+            URL fileURL = URLMapper.findURL(fo, URLMapper.INTERNAL);
+            if (fileURL == null || !"file".equals(fileURL.getProtocol())) {  //NOI18N
+                fileURL = URLMapper.findURL(fo, URLMapper.EXTERNAL);
             }
 
             if ((fileURL != null) && "file".equals(fileURL.getProtocol())) {
                 retVal = new File(URI.create(fileURL.toExternalForm()));
             }
+            if (retVal != null) {
+                retVal = normalizeFile(retVal);
+            }
         }
-
-        return (retVal != null) ? normalizeFile(retVal) : null;
+        return retVal;
     }
 
     /**
@@ -1141,7 +1265,7 @@ public final class FileUtil extends Object {
      *
     * @param folder parent folder
     * @param name preferred base name of file
-    * @param ext extension to use
+    * @param ext extension to use (or null)
     * @return a free file name <strong>(without the extension)</strong>
      */
     public static String findFreeFileName(FileObject folder, String name, String ext) {
@@ -1377,7 +1501,7 @@ public final class FileUtil extends Object {
     public static List<String> getMIMETypeExtensions(String mimeType) {
         Parameters.notEmpty("mimeType", mimeType);  //NOI18N
         HashMap<String, String> extensionToMime = new HashMap<String, String>();
-        for (FileObject mimeResolverFO : MIMEResolverImpl.getOrderedResolvers().values()) {
+        for (FileObject mimeResolverFO : MIMEResolverImpl.getOrderedResolvers()) {
             Map<String, Set<String>> mimeToExtensions = MIMEResolverImpl.getMIMEToExtensions(mimeResolverFO);
             for (Map.Entry<String, Set<String>> entry : mimeToExtensions.entrySet()) {
                 String mimeKey = entry.getKey();
@@ -1397,16 +1521,11 @@ public final class FileUtil extends Object {
     }
 
     /**
-     * Construct a stream handler that handles the <code>nbfs</code> URL protocol
-     * used for accessing file objects directly.
-     * This method is not intended for module use; only the core
-     * should need to call it.
-     * Modules probably need only use {@link URLMapper} to create and decode such
-     * URLs.
-     * @since 3.17
+     * @deprecated No longer used.
      */
+    @Deprecated
     public static URLStreamHandler nbfsURLStreamHandler() {
-        return FileURL.HANDLER;
+        return new FileURL.Handler();
     }
 
     /** Recursively checks whether the file is underneath the folder. It checks whether
@@ -1559,6 +1678,7 @@ public final class FileUtil extends Object {
         Parameters.notNull("file", file);  //NOI18N
         File retFile;
 
+        long now = System.currentTimeMillis();
         if ((Utilities.isWindows() || (Utilities.getOperatingSystem() == Utilities.OS_OS2))) {
             retFile = normalizeFileOnWindows(file);
         } else if (Utilities.isMac()) {
@@ -1566,8 +1686,12 @@ public final class FileUtil extends Object {
         } else {
             retFile = normalizeFileOnUnixAlike(file);
         }
-
-        return (file.getPath().equals(retFile.getPath())) ? file : retFile;
+        File ret = (file.getPath().equals(retFile.getPath())) ? file : retFile;
+        long took = System.currentTimeMillis() - now;
+        if (took > 500) {
+            LOG.log(Level.WARNING, "FileUtil.normalizeFile({0}) took {1} ms. Result is {2}", new Object[]{file, took, ret});
+        }
+        return ret;
     }
 
     private static File normalizeFileOnUnixAlike(File file) {
@@ -1588,7 +1712,12 @@ public final class FileUtil extends Object {
             // URI.normalize removes ../ and ./ sequences nicely.            
             File absoluteFile = new File(file.toURI().normalize());
             File canonicalFile = file.getCanonicalFile();
-            boolean isSymLink = !canonicalFile.getAbsolutePath().equalsIgnoreCase(absoluteFile.getAbsolutePath());
+            String absolutePath = absoluteFile.getAbsolutePath();
+            if (absolutePath.equals("/..")) { // NOI18N
+                // Special treatment.
+                absoluteFile = new File(absolutePath = "/"); // NOI18N
+            }
+            boolean isSymLink = !canonicalFile.getAbsolutePath().equalsIgnoreCase(absolutePath);
 
             if (isSymLink) {
                 retVal = normalizeSymLinkOnMac(absoluteFile);
@@ -1787,15 +1916,12 @@ public final class FileUtil extends Object {
         Parameters.notNull("fileObject", fo);  //NOI18N
 
         if (!fo.isValid()) {
-            return false;
+            return isArchiveFile(fo.getPath());
         }
         // XXX Special handling of virtual file objects: try to determine it using its name, but don't cache the
         // result; when the file is checked out the more correct method can be used
         if (fo.isVirtual()) {
-            String path = fo.getPath();
-            int index = path.lastIndexOf('.');
-
-            return (index != -1) && (index > path.lastIndexOf('/') + 1);
+            return isArchiveFile(fo.getPath());
         }
 
         if (fo.isFolder()) {
@@ -1833,9 +1959,7 @@ public final class FileUtil extends Object {
             }
 
             if (b == null) {
-                String path = fo.getPath();
-                int index = path.lastIndexOf('.');
-                b = ((index != -1) && (index > path.lastIndexOf('/') + 1)) ? Boolean.TRUE : Boolean.FALSE;
+                b = isArchiveFile(fo.getPath());
             }
 
             archiveFileCache.put(fo, b);
@@ -1864,12 +1988,12 @@ public final class FileUtil extends Object {
         FileObject fo = URLMapper.findFileObject(url);
 
         if ((fo != null) && !fo.isVirtual()) {
+            if (LOG.isLoggable(Level.FINEST)) {
+                LOG.log(Level.FINEST, "isArchiveFile_FILE_RESOLVED", fo); //NOI18N, used by FileUtilTest.testIsArchiveFileRace
+            }
             return isArchiveFile(fo);
         } else {
-            String urlPath = url.getPath();
-            int index = urlPath.lastIndexOf('.');
-
-            return (index != -1) && (index > urlPath.lastIndexOf('/') + 1);
+            return isArchiveFile(url.getPath());
         }
     }
 
@@ -1932,15 +2056,11 @@ public final class FileUtil extends Object {
      * @see <a href="http://www.netbeans.org/issues/show_bug.cgi?id=46459">Issue #46459</a>
      * @see <a href="http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4906607">JRE bug #4906607</a>
      * @since org.openide/1 4.42
+     * @deprecated Just use {@link JFileChooser#setCurrentDirectory}. JDK 6 does not have this bug.
      */
+    @Deprecated
     public static void preventFileChooserSymlinkTraversal(JFileChooser chooser, File currentDirectory) {
-        if (!(Utilities.isWindows() || (Utilities.getOperatingSystem() == Utilities.OS_OS2))
-                && System.getProperty("java.specification.version").startsWith("1.5")) { // NOI18N
-            chooser.setCurrentDirectory(wrapFileNoCanonicalize(currentDirectory));
-            chooser.setFileSystemView(new NonCanonicalizingFileSystemView());
-        } else {
-            chooser.setCurrentDirectory(currentDirectory);
-        }
+        chooser.setCurrentDirectory(currentDirectory);
     }
 
     /**
@@ -2220,5 +2340,15 @@ public final class FileUtil extends Object {
                 diskFileSystem = fs;
             }
         }
-    }    
+    }
+
+    /**
+     * Tests if a non existent path represents a file.
+     * @param path to be tested, separated by '/'.
+     * @return true if the file has '.' after last '/'.
+     */
+    private static boolean isArchiveFile (final String path) {
+        int index = path.lastIndexOf('.');  //NOI18N
+        return (index != -1) && (index > path.lastIndexOf('/') + 1);    //NOI18N
+    }
 }
