@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -58,6 +61,7 @@ import java.lang.ref.SoftReference;
 import java.lang.ref.WeakReference;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.Date;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -71,8 +75,8 @@ import java.util.jar.Manifest;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import java.util.zip.ZipException;
-import org.openide.util.Enumerations;
 import org.openide.util.Exceptions;
+import org.openide.util.NbBundle;
 import org.openide.util.RequestProcessor;
 import org.openide.util.Utilities;
 
@@ -89,7 +93,7 @@ public class JarFileSystem extends AbstractFileSystem {
     static final long serialVersionUID = -98124752801761145L;
 
     /** One request proccesor shared for all instances of JarFileSystem*/
-    private static RequestProcessor req = new RequestProcessor("JarFs - modification watcher"); // NOI18N
+    private static final RequestProcessor req = new RequestProcessor("JarFs - modification watcher", 1, false, false); // NOI18N
 
     /** Controlls the LocalFileSystem's automatic refresh.
     * If the refresh time interval is set from the System.property, than this value is used.
@@ -100,6 +104,11 @@ public class JarFileSystem extends AbstractFileSystem {
     /** maxsize for passing ByteArrayInputStream*/
     private static final long MEM_STREAM_SIZE = 100000;
 
+    /** Maximal time for which closing of jar is postponed. */
+    private static final int CLOSE_DELAY_MAX = 5000;
+    /** Mininal time for which closing of jar is postponed. */
+    private static final int CLOSE_DELAY_MIN = 300;
+
     /**
     * Opened zip file of this filesystem is stored here or null.
     */
@@ -138,6 +147,11 @@ public class JarFileSystem extends AbstractFileSystem {
     private transient FileObject foRoot;
     private transient FileChangeListener fcl;
 
+    /** Actual time for which closing of jar is postponed. */
+    private transient int closeDelay = CLOSE_DELAY_MIN;
+    /** Time of request for opening of jar. */
+    private transient long openRequestTime = 0;
+
     /**
     * Default constructor.
      * <p><strong>Most module code should never create an instance of this class directly.</strong>
@@ -162,6 +176,31 @@ public class JarFileSystem extends AbstractFileSystem {
         setCapability(cap);
     }
 
+    /** Creates new JAR for a given JAR file. This constructor
+     * behaves basically like:
+     * <pre>
+     * JarFileSystem fs = new JarFileSystem();
+     * fs.setJarFile(jar);
+     * </pre>
+     * but it is more effective in some situations. It does not open and
+     * read the content of the jar file immediately. Instead
+     * it waits until somebody asks for resources from inside the JAR.
+     *
+     * @param jar location of the JAR file
+     * @since 7.34
+     */
+    public JarFileSystem(File jar) throws IOException {
+        this();
+        try {
+            setJarFile(jar, true, false);
+        } catch (PropertyVetoException ex) {
+            // cannot happen, setSystemName can throw the exception only
+            // if the filesystem is already in Repository, which this one
+            // is not.
+            throw (IOException)new IOException().initCause(ex);
+        }
+    }
+
     /* Creates Reference. In FileSystem, which subclasses AbstractFileSystem, you can overload method
      * createReference(FileObject fo) to achieve another type of Reference (weak, strong etc.)
      * @param fo is FileObject. It`s reference yourequire to get.
@@ -224,7 +263,7 @@ public class JarFileSystem extends AbstractFileSystem {
     * @throws IOException if the file is not valid
     */
     public void setJarFile(final File aRoot) throws IOException, PropertyVetoException {
-        setJarFile(aRoot, true);
+        setJarFile(aRoot, true, true);
     }
     
     @SuppressWarnings("deprecation") // need to set it for compat
@@ -232,7 +271,7 @@ public class JarFileSystem extends AbstractFileSystem {
         setSystemName(s);
     }
 
-    private void setJarFile(final File aRoot, boolean refreshRoot)
+    private void setJarFile(final File aRoot, boolean refreshRoot, boolean openJar)
     throws IOException, PropertyVetoException {
         if (!aRoot.equals(FileUtil.normalizeFile(aRoot))) {
             throw new IllegalArgumentException(
@@ -249,19 +288,19 @@ public class JarFileSystem extends AbstractFileSystem {
         }
 
         if (aRoot == null) {
-            FSException.io("EXC_NotValidFile", aRoot); // NOI18N        
+            throw new FSException(NbBundle.getMessage(JarFileSystem.class, "EXC_NotValidFile", aRoot));
         }
 
         if (!aRoot.exists()) {
-            FSException.io("EXC_FileNotExists", aRoot.getAbsolutePath()); // NOI18N
+            throw new FSException(NbBundle.getMessage(JarFileSystem.class, "EXC_FileNotExists", aRoot.getAbsolutePath()));
         }
 
         if (!aRoot.canRead()) {
-            FSException.io("EXC_CanntRead", aRoot.getAbsolutePath()); // NOI18N
+            throw new FSException(NbBundle.getMessage(JarFileSystem.class, "EXC_CanntRead", aRoot.getAbsolutePath()));
         }
 
         if (!aRoot.isFile()) {
-            FSException.io("EXC_NotValidFile", aRoot.getAbsolutePath()); // NOI18N
+            throw new FSException(NbBundle.getMessage(JarFileSystem.class, "EXC_NotValidFile", aRoot.getAbsolutePath()));
         }
 
         String s;
@@ -270,18 +309,21 @@ public class JarFileSystem extends AbstractFileSystem {
 
         JarFile tempJar = null;
 
-        try {
-            tempJar = new JarFile(s);
-            LOGGER.log(Level.FINE, "opened: "+ System.currentTimeMillis()+ "   " + s);//NOI18N
-        } catch (ZipException e) {
-            FSException.io("EXC_NotValidJarFile2", e.getLocalizedMessage(), s); // NOI18N
+        if (openJar) {
+            try {
+                tempJar = new JarFile(s);
+                LOGGER.log(Level.FINE, "opened: "+ System.currentTimeMillis()+ "   " + s);//NOI18N
+            } catch (ZipException e) {
+                throw new FSException(NbBundle.getMessage(JarFileSystem.class, "EXC_NotValidJarFile2", e.getLocalizedMessage(), s));
+            }
         }
 
         synchronized (closeSync) {
             _setSystemName(s);
 
             closeCurrentRoot(false);
-            jar = tempJar;
+            setJar(tempJar);
+            openRequestTime = System.currentTimeMillis();
             root = new File(s);
 
             if (refreshRoot) {
@@ -318,7 +360,7 @@ public class JarFileSystem extends AbstractFileSystem {
 
                     if ((f != null) && !f.equals(aRoot)) {
                         try {
-                            setJarFile(f, false);
+                            setJarFile(f, false, true);
                         } catch (IOException iex) {
                             ExternalUtil.exception(iex);
                         } catch (PropertyVetoException pvex) {
@@ -359,7 +401,7 @@ public class JarFileSystem extends AbstractFileSystem {
     * @return user presentable name of the filesystem
     */
     public String getDisplayName() {
-        return (root != null) ? root.getAbsolutePath() : getString("JAR_UnknownJar");
+        return root != null ? root.getAbsolutePath() : NbBundle.getMessage(JarFileSystem.class, "JAR_UnknownJar");
     }
 
     /** This filesystem is read-only.
@@ -426,12 +468,18 @@ public class JarFileSystem extends AbstractFileSystem {
     //
     // Info
     //
-    protected java.util.Date lastModified(String name) {
-        try {
-            return new java.util.Date(getEntry(name).getTime());
-        } finally {
-            closeCurrentRoot(false);
+    protected Date lastModified(String name) {
+        long t;
+        if (name.length() == 0) {
+            t = getJarFile().lastModified();
+        } else {
+            try {
+                t = getEntry(name).getTime();
+            } finally {
+                closeCurrentRoot(false);
+            }
         }
+        return new Date(t);
     }
 
     protected boolean folder(String name) {
@@ -593,7 +641,7 @@ public class JarFileSystem extends AbstractFileSystem {
     }
 
     protected void lock(String name) throws IOException {
-        FSException.io("EXC_CannotLock", name, getDisplayName(), name); // NOI18N
+        throw new FSException(NbBundle.getMessage(JarFileSystem.class, "EXC_CannotLock_JAR", name, root));
     }
 
     protected void unlock(String name) {
@@ -603,6 +651,9 @@ public class JarFileSystem extends AbstractFileSystem {
     }
 
     protected Object readAttribute(String name, String attrName) {
+        if ("java.io.File".equals(attrName)) {
+            return null;
+        }
         Attributes attr1 = getManifest().getAttributes(name);
 
         try {
@@ -671,18 +722,14 @@ public class JarFileSystem extends AbstractFileSystem {
                 closeTask.cancel();
             }
 
-            JarFile j = jar;
+            // #167527 - calculate adaptive delay before closing jar
+            long now = System.currentTimeMillis();
+            long requestPeriod = now - openRequestTime;
+            openRequestTime = now;
+            // 150% of time from last open request, but between CLOSE_DELAY_MIN and CLOSE_DELAY_MAX
+            closeDelay = (int) Math.min(CLOSE_DELAY_MAX, Math.max(CLOSE_DELAY_MIN, (1.5 * requestPeriod)));
 
-            if (j != null) {
-                return j;
-            }
-
-            if ((jar == null) && (root != null)) {
-                jar = new JarFile(root);
-                LOGGER.log(Level.FINE, "opened: "+ System.currentTimeMillis()+ "   " + root.getAbsolutePath());//NOI18N
-            }
-
-            return jar;
+            return getJar(true);
         }
     }
 
@@ -698,7 +745,7 @@ public class JarFileSystem extends AbstractFileSystem {
             if (isRealClose) {
                 realClose().run();
             } else {
-                closeTask = req.post(realClose(), 300);
+                closeTask = req.post(realClose(), closeDelay);
             }
         }
     }
@@ -707,15 +754,16 @@ public class JarFileSystem extends AbstractFileSystem {
         return new Runnable() {
                 public void run() {
                     synchronized (closeSync) {
-                        if (jar != null) {
+                    final JarFile jarFile = getJar(false);
+                        if (jarFile != null) {
                             try {
-                                jar.close();
+                                jarFile.close();
                                 LOGGER.log(Level.FINE, "closed: "+ System.currentTimeMillis()+ "   " + root.getAbsolutePath());//NOI18N
                             } catch (Exception exc) {
                                 // ignore exception during closing, just log it
                                 ExternalUtil.exception(exc);
                             } finally {
-                                jar = null;
+                                setJar(null);
                                 closeTask = null;
                             }
                         }
@@ -811,7 +859,7 @@ public class JarFileSystem extends AbstractFileSystem {
                             uniqueEntries.add(entry);
                         } else {
                             if (!duplicateReported) {
-                                LOGGER.warning(getString("EXC_DuplicateEntries", getJarFile(), name));  //NOI18N
+                                LOGGER.warning("Duplicate entries in " + getJarFile() + ": " + name + "; please report to JAR creator.");
                                 // report just once
                                 duplicateReported = true;
                             }
@@ -878,7 +926,10 @@ public class JarFileSystem extends AbstractFileSystem {
             synchronized (closeSync) {
                 j = reOpenJarFile();
 
-                JarEntry je = j.getJarEntry(file);
+                JarEntry je = null;
+                if (j != null) {
+                    je = j.getJarEntry(file);
+                }
 
                 if (je != null) {
                     return je;
@@ -890,6 +941,33 @@ public class JarFileSystem extends AbstractFileSystem {
         return new JarEntry(file);
     }
 
+    /**
+     * @return the jar
+     */
+    private JarFile getJar(boolean create) {
+        assert Thread.holdsLock(closeSync);
+        if (jar == null && create) {
+            try {
+                if (root.canRead()) {
+                    jar = new JarFile(root);
+                    LOGGER.log(Level.FINE, "opened: {0} {1}", new Object[]{root.getAbsolutePath(), System.currentTimeMillis()}); //NOI18N
+                    return jar;
+                }
+            } catch (IOException ex) {
+                LOGGER.log(Level.INFO, ex.getMessage(), ex);
+            }
+            LOGGER.log(Level.WARNING, "cannot open {0}", root.getAbsolutePath()); // NOI18N
+        }
+        return jar;
+    }
+
+    /**
+     * @param jar the jar to set
+     */
+    private void setJar(JarFile jar) {
+        this.jar = jar;
+    }
+
     /** Use soft-references to not throw away the data that quickly.
      * JarFS if often queried for its FOs e.g. by java parser, which
      * leaves the references immediately.
