@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -49,6 +52,7 @@ import java.io.ObjectOutput;
 import java.io.OutputStream;
 import java.lang.ref.Reference;
 import java.lang.ref.WeakReference;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
@@ -60,6 +64,10 @@ import java.util.Map;
 import java.util.Properties;
 import java.util.Set;
 import java.util.WeakHashMap;
+import java.util.logging.Level;
+import java.util.logging.Logger;
+import org.openide.util.Exceptions;
+import org.openide.util.NbBundle;
 
 /** Implementation of the file object for multi file system.
 *
@@ -74,6 +82,7 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
 
     /** default path separator */
     private static final char PATH_SEP = '/';
+    private static final String WEIGHT_ATTRIBUTE = "weight"; // NOI18N
     private static final FileSystem.AtomicAction markAtomicAction = new FileSystem.AtomicAction() {
             public void run() {
             }
@@ -167,9 +176,11 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
 
         Set now = (delegates == null) ? Collections.EMPTY_SET : delegates;
         Set<FileObject> del = new HashSet<FileObject>(arr.length * 2);
+        Number maxWeight = 0;
         FileObject led = null;
 
         String name = getPath();
+        FileSystem writable = mfs.writableLayer(name);
 
         for (int i = 0; i < arr.length; i++) {
             if (arr[i] != null) {
@@ -183,8 +194,15 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
                         fo.addFileChangeListener(weakL);
                     }
 
-                    if ((led == null) && fo.isValid()) {
-                        led = fo;
+                    if (fo.isValid()) {
+                        Number weight = 
+                            fo.isRoot() &&
+                            !mfs.canHaveRootAttributeOnReadOnlyFS(WEIGHT_ATTRIBUTE)
+                            ? 0 : weightOf(fo, writable);
+                        if (led == null || weight.doubleValue() > maxWeight.doubleValue()) {
+                            led = fo;
+                            maxWeight = weight;
+                        }
                     }
                 }
             }
@@ -296,6 +314,10 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
     private FileObject findLeader(FileSystem[] fs, String path) {
         MultiFileSystem mfs = getMultiFileSystem();
 
+        Number maxWeight = 0;
+        FileObject _leader = null;
+        FileSystem writable = mfs.writableLayer(path);
+        
         for (FileSystem f : fs) {
             if (f == null) {
                 continue;
@@ -303,11 +325,36 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
             FileObject fo = mfs.findResourceOn(f, path);
 
             if (fo != null) {
-                return fo;
+                Number weight = weightOf(fo, writable);
+                if (_leader == null || weight.doubleValue() > maxWeight.doubleValue()) {
+                    _leader = fo;
+                    maxWeight = weight;
+                }
             }
         }
 
-        return null;
+        return _leader;
+    }
+
+    private static Number weightOf(FileObject f, FileSystem writable) {
+        try {
+            if (f.getFileSystem() == writable) {
+                return Double.MAX_VALUE;
+            }
+        } catch (FileStateInvalidException x) {/* ignore */}
+        Object weight = f.getAttribute(WEIGHT_ATTRIBUTE);
+        if (weight instanceof Number) {
+            return (Number) weight;
+        } else if (weight == null) {
+            return 0;
+        } else {
+            try {
+                Logger.getLogger(MultiFileObject.class.getName()).log(
+                        Level.WARNING, "File {0} in {1} has nonnumeric weight {2} of type {3}",
+                        new Object[] {f.getPath(), f.getFileSystem(), weight, weight.getClass().getName()});
+            } catch (FileStateInvalidException x) {/* ignore */}
+            return 0;
+        }
     }
 
     /** Getter for the right file system */
@@ -419,7 +466,8 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
 
                     // this name should be excluded from next rounds of cycle 1
                     if (local == null) {
-                        local = new Properties(exclude);
+                        local = new Properties();
+                        local.putAll(exclude);
                     }
 
                     local.setProperty(basename, basename);
@@ -661,11 +709,13 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
     */
     private MfLock testLock(FileLock l) throws java.io.IOException {
         if (lock == null) {
-            FSException.io("EXC_InvalidLock", l, getPath(), getMultiFileSystem().getDisplayName(), lock); // NOI18N
+            throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_InvalidLock",
+                    l, getPath(), getMultiFileSystem().getDisplayName(), lock));
         }
 
         if (lock.get() != l) {
-            FSException.io("EXC_InvalidLock", l, getPath(), getMultiFileSystem().getDisplayName(), lock.get()); // NOI18N                
+            throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_InvalidLock",
+                    l, getPath(), getMultiFileSystem().getDisplayName(), lock.get()));
         }
 
         return (MfLock) l;
@@ -722,6 +772,8 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
         return getAttribute(attrName, getPath());
     }
 
+    /** Special attributes which should not be checked for weight. See RemoveWritablesTest. */
+    private static final Set<String> SPECIAL_ATTR_NAMES = new HashSet<String>(Arrays.asList("removeWritables", WEIGHT_ATTRIBUTE, "java.io.File")); // NOI18N
     private final Object getAttribute(String attrName, String path) {
         // Look for attribute in any file system starting at the front.
         // Additionally, look for attribute in root folder, where
@@ -743,8 +795,7 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
                 if (localFo.isRoot() && (prefixattr != null)) {
                     try {
                         FileSystem foFs = localFo.getFileSystem();
-
-                        if (!(foFs instanceof XMLFileSystem)) {
+                        if (!foFs.isReadOnly() || getMultiFileSystem().canHaveRootAttributeOnReadOnlyFS(prefixattr)) {
                             localFo = foFs.getRoot();
                             oPerf = getAttribute(localFo, prefixattr, ""); // NOI18N
 
@@ -770,6 +821,10 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
 
         FileSystem[] systems = getMultiFileSystem().getDelegates();
 
+        Number maxWeight = 0;
+        Object attr = null;
+        FileSystem writable = getMultiFileSystem().writableLayer(path);
+
         //        boolean isLoaderAttr = /* DataObject.EA_ASSIGNED_LOADER */ "NetBeansAttrAssignedLoader".equals (attrName); // NOI18N                
         for (int i = 0; i < systems.length; i++) {
             if (systems[i] == null) {
@@ -789,26 +844,38 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
                 Object o = getAttribute(fo, attrName, fo.getPath()); // Performance tricks:                
 
                 if (o != null) {
-                    return devoidify(o);
+                    if (SPECIAL_ATTR_NAMES.contains(attrName)) {
+                        return devoidify(o);
+                    }
+                    Number weight = weightOf(fo, writable);
+                    if (attr == null || weight.doubleValue() > maxWeight.doubleValue()) {
+                        attr = o;
+                        maxWeight = weight;
+                    }
                 }
             }
 
-            // Don't check for root override on XMLFileSystem's; the override
-            // could only have been made on a writable filesystem to begin with.
-            // Could skip all RO FSs but then multi-user installs would not work
-            // quite right.
-            if ((prefixattr != null) && !(systems[i] instanceof XMLFileSystem)) {
-                fo = systems[i].getRoot();
+            if (prefixattr != null) {
+                if (
+                    !systems[i].isReadOnly() ||
+                    getMultiFileSystem().canHaveRootAttributeOnReadOnlyFS(prefixattr)
+                ) {
+                    fo = systems[i].getRoot();
 
-                Object o = getAttribute(fo, prefixattr, ""); // NOI18N
+                    Object o = getAttribute(fo, prefixattr, ""); // NOI18N
 
-                if (o != null) {
-                    return devoidify(o);
+                    if (o != null) {
+                        Number weight = weightOf(fo, writable);
+                        if (attr == null || weight.doubleValue() > maxWeight.doubleValue()) {
+                            attr = o;
+                            maxWeight = weight;
+                        }
+                    }
                 }
             }
         }
 
-        return null;
+        return devoidify(attr);
     }
 
     private static boolean sameFullName(FileObject f1, FileObject f2) {
@@ -975,6 +1042,7 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
     * @return the new folder
     * @exception IOException if the folder cannot be created (e.g. already exists)
     */
+    @Override
     public FileObject createFolder(String name) throws IOException {
         MultiFileObject fo;
 
@@ -985,21 +1053,23 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
                 MultiFileSystem fs = getMultiFileSystem();
 
                 if (fs.isReadOnly()) {
-                    FSException.io("EXC_FSisRO", fs.getDisplayName()); // NOI18N
+                    throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_FSisRO", fs.getDisplayName()));
                 }
 
                 if (isReadOnly()) {
-                    FSException.io("EXC_FisRO", name, fs.getDisplayName()); // NOI18N
+                    IOException ex = new IOException("Read only: " + leader + " delegates: " + delegates); // NOI18N
+                    Exceptions.attachLocalizedMessage(ex, NbBundle.getMessage(MultiFileObject.class, "EXC_FisRO", name, fs.getDisplayName()));
+                    throw ex;
                 }
 
                 String fullName = getPath() + PATH_SEP + name;
 
                 if (!isFolder()) {
-                    FSException.io("EXC_FoNotFolder", name, getPath(), fs.getDisplayName()); // NOI18N
+                    throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_FoNotFolder", name, getPath(), fs.getDisplayName()));
                 }
 
                 if (this.getFileObject(name) != null) {
-                    FSException.io("EXC_FolderAlreadyExist", name, fs.getDisplayName(), getPath()); // NOI18N
+                    throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_FolderAlreadyExist", name, fs.getDisplayName(), getPath()));
                 }
 
                 FileSystem simple = fs.createWritableOn(fullName);
@@ -1018,9 +1088,7 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
 
                 if (fo == null) {
                     // system error
-                    throw new FileStateInvalidException(
-                        FileSystem.getString("EXC_ApplicationCreateError", getPath(), name)
-                    );
+                    throw new FileStateInvalidException(NbBundle.getMessage(MultiFileObject.class, "EXC_ApplicationCreateError", getPath(), name));
                 }
 
                 FileObject[] chlds = fo.getChildren();
@@ -1060,21 +1128,21 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
                 MultiFileSystem fs = getMultiFileSystem();
 
                 if (fs.isReadOnly()) {
-                    FSException.io("EXC_FSisRO", fs.getDisplayName()); // NOI18N
+                    throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_FSisRO", fs.getDisplayName()));
                 }
 
                 if (isReadOnly()) {
-                    FSException.io("EXC_FisRO", name, fs.getDisplayName()); // NOI18N
+                    throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_FisRO", name, fs.getDisplayName()));
                 }
 
                 String n = "".equals(ext) ? name : (name + EXT_SEP + ext); // NOI18N
 
                 if (!isFolder()) {
-                    FSException.io("EXC_FoNotFolder", n, getPath(), fs.getDisplayName()); // NOI18N
+                    throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_FoNotFolder", n, getPath(), fs.getDisplayName()));
                 }
 
                 if (this.getFileObject(name, ext) != null) {
-                    FSException.io("EXC_DataAlreadyExist", n, fs.getDisplayName(), getPath()); // NOI18N
+                    throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_DataAlreadyExist", n, fs.getDisplayName(), getPath()));
                 }
 
                 String fullName = getPath() + PATH_SEP + n;
@@ -1095,9 +1163,7 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
 
                 if (fo == null) {
                     // system error
-                    throw new FileStateInvalidException(
-                        FileSystem.getString("EXC_ApplicationCreateError", getPath(), n)
-                    );
+                    throw new FileStateInvalidException(NbBundle.getMessage(MultiFileObject.class, "EXC_ApplicationCreateError", getPath(), n));
                 }
 
                 if (hasListeners()) {
@@ -1127,7 +1193,7 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
         MultiFileSystem fs = getMultiFileSystem();
 
         if (parent == null) {
-            FSException.io("EXC_CannotRenameRoot", fs.getDisplayName()); // NOI18N
+            throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_CannotRenameRoot", fs.getDisplayName()));
         }
 
         try {
@@ -1146,11 +1212,12 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
                 String oldFullName = getPath();
 
                 if (isReadOnly()) {
-                    FSException.io("EXC_CannotRename", getPath(), getMultiFileSystem().getDisplayName(), newFullName); // NOI18N
+                    throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_CannotRename",
+                            getPath(), getMultiFileSystem().getDisplayName(), newFullName));
                 }
 
                 if (getFileSystem().isReadOnly()) {
-                    FSException.io("EXC_FSisRO", getMultiFileSystem().getDisplayName()); // NOI18N
+                    throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_FSisRO", getMultiFileSystem().getDisplayName()));
                 }
 
                 String on = getName();
@@ -1232,8 +1299,7 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
     */
     void handleDelete(FileLock lock) throws IOException {
         if (parent == null) {
-            FSException.io("EXC_CannotDeleteRoot", getMultiFileSystem().getDisplayName() // NOI18N
-            );
+            throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_CannotDeleteRoot", getMultiFileSystem().getDisplayName()));
         }
 
         MultiFileSystem fs = getMultiFileSystem();
@@ -1302,8 +1368,7 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
             fs.beginAtomicAction();
 
             if (parent == null) {
-                FSException.io("EXC_CannotDeleteRoot", fs.getDisplayName() // NOI18N
-                );
+                throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_CannotDeleteRoot", fs.getDisplayName()));
             }
 
             MfLock lck = testLock(lock);
@@ -1312,7 +1377,7 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
             FileSystem simple = fs.createWritableOn(getPath());
 
             if (fs.isReadOnly()) {
-                FSException.io("EXC_FSisRO", fs.getDisplayName()); // NOI18N
+                throw new FSException(NbBundle.getMessage(MultiFileObject.class, "EXC_FSisRO", fs.getDisplayName()));
             }
 
             if ((l == null) && (leader.getFileSystem() != simple)) {
@@ -1444,8 +1509,13 @@ final class MultiFileObject extends AbstractFolder implements FileObject.Priorit
         ) {
             return;
         }
-
-        fileAttributeChanged0(new FileAttributeEvent(this, fe.getName(), fe.getOldValue(), fe.getNewValue()));
+        final FileAttributeEvent ev = new FileAttributeEvent(this, fe.getName(), fe.getOldValue(), fe.getNewValue());
+        try {
+            ev.inheritPostNotify(fe);
+            fileAttributeChanged0(ev);
+        } finally {
+            ev.setPostNotify(null);
+        }
     }
 
     /** Copies content of one folder into another.
