@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -48,6 +51,7 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.logging.Handler;
 import java.util.logging.Level;
 import java.util.logging.LogRecord;
@@ -55,7 +59,9 @@ import java.util.logging.Logger;
 import junit.framework.Test;
 import org.netbeans.junit.NbTestCase;
 import org.netbeans.junit.NbTestSuite;
+import org.netbeans.junit.RandomlyFails;
 import org.openide.filesystems.test.TestFileUtils;
+import org.openide.util.Exceptions;
 import org.openide.util.Lookup;
 import org.openide.util.Utilities;
 import org.openide.util.test.MockLookup;
@@ -187,7 +193,62 @@ public class FileUtilTest extends NbTestCase {
             assertEquals("FileUtil.getArchiveFile failed.", new URL(urls[i][1]), FileUtil.getArchiveFile(new URL(urls[i][0])));
         }
     }
-    
+
+    public void testIsArchiveFileRace() throws Exception {
+        final LocalFileSystem lfs = new LocalFileSystem();
+        clearWorkDir();
+        final File wd = getWorkDir();
+        lfs.setRootDirectory(wd);
+        MockLookup.setInstances(new URLMapper() {
+            String rootURL = lfs.getRoot().getURL().toString();
+            @Override
+            public FileObject[] getFileObjects(URL url) {
+                String u = url.toString();
+                FileObject f = null;
+                if (u.startsWith(rootURL)) {
+                    f = lfs.findResource(u.substring(rootURL.length()));
+                }
+                return f != null ? new FileObject[] {f} : null;
+            }
+            @Override
+            public URL getURL(FileObject fo, int type) {
+                return null;
+            }
+        });
+        URLMapper.reset();
+        final File testFile = new File (wd,"test.jar"); //NOI18N
+        FileUtil.createData(testFile);
+
+        final Logger log = Logger.getLogger(FileUtil.class.getName());
+        log.setLevel(Level.FINEST);
+        final Handler handler = new Handler() {
+            @Override
+            public void publish(LogRecord record) {
+                if ("isArchiveFile_FILE_RESOLVED".equals(record.getMessage())) {  //NOI18N
+                    try {
+                        final FileObject fo = (FileObject) record.getParameters()[0];
+                        fo.delete();
+                    } catch (IOException ex) {
+                        Exceptions.printStackTrace(ex);
+                    }
+                }
+            }
+            @Override
+            public void flush() {
+            }
+            @Override
+            public void close() throws SecurityException {
+            }
+        };
+        log.addHandler(handler);
+        try {
+            final boolean result = FileUtil.isArchiveFile(testFile.toURI().toURL());
+            assertTrue("The test.jar should be archive.",result);   //NOI18N
+        } finally {
+            log.removeHandler(handler);
+        }
+    }
+
     /** Tests normalizeFile() method. */
     public void testNormalizeFile() throws IOException {
         // pairs of path before and after normalization
@@ -390,4 +451,65 @@ public class FileUtilTest extends NbTestCase {
             // OK
         }
     }
+
+    /** Tests that refreshAll runs just once in time (see #170556). */
+    @RandomlyFails // NB-Core-Build #4062: FileUtil.refreshAll not called. expected:<2> but was:<1>
+    public void testRefreshConcurrency() throws Exception {
+        Logger logger = Logger.getLogger(FileUtil.class.getName());
+        logger.setLevel(Level.FINE);
+        final AtomicInteger concurrencyCounter = new AtomicInteger(0);
+        final AtomicInteger maxConcurrency = new AtomicInteger(0);
+        final AtomicInteger calledCounter = new AtomicInteger(0);
+        logger.addHandler(new Handler() {
+
+            private boolean concurrentStarted = false;
+
+            @Override
+            public void publish(LogRecord record) {
+                if (record.getMessage().equals("refreshAll - started")) {
+                    calledCounter.incrementAndGet();
+                    concurrencyCounter.incrementAndGet();
+                    if (!concurrentStarted) {
+                        concurrentStarted = true;
+                        new Thread("Concurrent refresh") {
+
+                            @Override
+                            public void run() {
+                                FileUtil.refreshAll();
+                            }
+                        }.start();
+                        synchronized (this) {
+                            try {
+                                wait(500);
+                            } catch (InterruptedException ex) {
+                                Exceptions.printStackTrace(ex);
+                            }
+                        }
+                    }
+                } else if (record.getMessage().equals("refreshAll - scheduled")) {
+                    if (concurrentStarted) {
+                        synchronized (this) {
+                            notifyAll();
+                        }
+                    }
+                } else if (record.getMessage().equals("refreshAll - finished")) {
+                    concurrencyCounter.decrementAndGet();
+                    if (concurrencyCounter.get() > maxConcurrency.get()) {
+                        maxConcurrency.set(concurrencyCounter.get());
+                    }
+                }
+            }
+
+            @Override
+            public void flush() {
+            }
+
+            @Override
+            public void close() throws SecurityException {
+            }
+        });
+        FileUtil.refreshAll();
+        assertEquals("FileUtil.refreshAll should not be called concurrently.", 0, maxConcurrency.get());
+        assertEquals("FileUtil.refreshAll not called.", 2, calledCounter.get());
+    }
 }
