@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -41,9 +44,12 @@
 
 package org.openide.filesystems;
 
+import java.io.ByteArrayInputStream;
 import java.io.FileNotFoundException;
+import java.io.FilterInputStream;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.InterruptedIOException;
 import java.io.OutputStream;
 import java.io.PrintStream;
 import java.lang.ref.WeakReference;
@@ -197,6 +203,189 @@ public class MIMESupportTest extends NbTestCase {
         }
     }
 
+    public void testRetryOnInterruptedIOException() throws Exception {
+        final byte[] arr = new byte[]{10, 11, 12, 13};
+        final FilterInputStream is = new FilterInputStream(new ByteArrayInputStream(arr)) {
+            boolean thrown = false;
+
+            @Override
+            public synchronized int read() throws IOException {
+                throwIfNotThrown();
+                return super.read();
+            }
+
+            @Override
+            public int read(byte[] b) throws IOException {
+                throwIfNotThrown();
+                return super.read(b);
+            }
+
+            @Override
+            public synchronized int read(byte[] b, int off, int len) throws IOException {
+                throwIfNotThrown();
+                return super.read(b, off, len);
+            }
+
+            private void throwIfNotThrown() throws InterruptedIOException {
+                if (!thrown) {
+                    thrown = true;
+                    throw new InterruptedIOException();
+                }
+            }
+        };
+
+        FileObject myFo = new FileObject() {
+            @Override
+            public String getName() {
+                return "fake";
+            }
+
+            @Override
+            public String getExt() {
+                return "ext";
+            }
+
+            @Override
+            public void rename(FileLock lock, String name, String ext) throws IOException {
+                throw new IOException();
+            }
+
+            @Override
+            public FileSystem getFileSystem() throws FileStateInvalidException {
+                throw new UnsupportedOperationException("Not supported yet.");
+            }
+
+            @Override
+            public FileObject getParent() {
+                throw new UnsupportedOperationException("Not supported yet.");
+            }
+
+            @Override
+            public boolean isFolder() {
+                return false;
+            }
+
+            @Override
+            public Date lastModified() {
+                return new Date(333);
+            }
+
+            @Override
+            public boolean isRoot() {
+                return false;
+            }
+
+            @Override
+            public boolean isData() {
+                return true;
+            }
+
+            @Override
+            public boolean isValid() {
+                return true;
+            }
+
+            @Override
+            public void delete(FileLock lock) throws IOException {
+                throw new IOException();
+            }
+
+            @Override
+            public Object getAttribute(String attrName) {
+                return null;
+            }
+
+            @Override
+            public void setAttribute(String attrName, Object value) throws IOException {
+                throw new IOException();
+            }
+
+            @Override
+            public Enumeration<String> getAttributes() {
+                return Enumerations.empty();
+            }
+
+            @Override
+            public void addFileChangeListener(FileChangeListener fcl) {
+            }
+
+            @Override
+            public void removeFileChangeListener(FileChangeListener fcl) {
+            }
+
+            @Override
+            public long getSize() {
+                return arr.length;
+            }
+
+            @Override
+            public InputStream getInputStream() throws FileNotFoundException {
+                return is;
+            }
+
+            @Override
+            public OutputStream getOutputStream(FileLock lock) throws IOException {
+                throw new IOException();
+            }
+
+            @Override
+            public FileLock lock() throws IOException {
+                throw new IOException();
+            }
+
+            @Override
+            public void setImportant(boolean b) {
+            }
+
+            @Override
+            public FileObject[] getChildren() {
+                return new FileObject[0];
+            }
+
+            @Override
+            public FileObject getFileObject(String name, String ext) {
+                return null;
+            }
+
+            @Override
+            public FileObject createFolder(String name) throws IOException {
+                throw new IOException();
+            }
+
+            @Override
+            public FileObject createData(String name, String ext) throws IOException {
+                throw new IOException();
+            }
+
+            @Override
+            public boolean isReadOnly() {
+                return true;
+            }
+        };
+
+        class MockMimeResolver extends MIMEResolver {
+            @Override
+            public String findMIMEType(FileObject fo) {
+                try {
+                    InputStream is = fo.getInputStream();
+                    if (is.read() == 10 && is.read() == 11 && is.read() == 12 && is.read() == 13) {
+                        is.close();
+                        return "hi/there";
+                    }
+                    is.close();
+                } catch (IOException ex) {
+                    throw new IllegalStateException(ex);
+                }
+                return null;
+            }
+        }
+
+        lookup.setLookups(new MockMimeResolver());
+
+        String type = FileUtil.getMIMEType(myFo);
+        assertEquals("The proper type is guessed", "hi/there", type);
+    }
+
     public void testWithinMimeTypes() throws Exception {
         MIMESupportTest.TestExtResolver testResolverA = new MIMESupportTest.TestExtResolver("a", "a/a");
         MIMESupportTest.TestExtResolver testResolverB = new MIMESupportTest.TestExtResolver("b", "b/b");
@@ -224,6 +413,34 @@ public class MIMESupportTest extends NbTestCase {
         assertEquals("Fallback MIME type for xml should not be cached.", "xml/xml", mimeType);
     }
 
+    public void testExtensionChangeNoticedInAtomicAction() throws Exception {
+        MIMESupportTest.TestExtResolver resHTML = new MIMESupportTest.TestExtResolver("html", "text/html");
+        assertTrue(Lookup.getDefault().lookupAll(MIMEResolver.class).isEmpty());
+
+        lookup.setLookups(resHTML);
+        assertEquals(1, Lookup.getDefault().lookupAll(MIMEResolver.class).size());
+
+        class R implements Runnable {
+            @Override
+            public void run() {
+                try {
+                    FileObject fo1 = FileUtil.createData(FileUtil.createMemoryFileSystem().getRoot(), "file.any");
+
+                    assertEquals("Unknown mime type", "content/unknown", fo1.getMIMEType());
+                    FileLock lock = fo1.lock();
+                    fo1.rename(lock, "file", "html");
+                    lock.releaseLock();
+
+                    assertEquals("Now the file is recognized", "text/html", fo1.getMIMEType());
+                } catch (IOException ex) {
+                    throw new IllegalStateException(ex);
+                }
+            }
+        }
+
+        FileUtil.runAtomicAction(new R());
+    }
+
     private static final class TestExtResolver extends MIMEResolver {
         private final String ext;
         private final String mime;
