@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -169,30 +172,48 @@ public class XMLDataObject extends MultiDataObject {
      *
      * @param fo the primary file object, never <code>null</code>
      * @param loader loader of this data object, never <code>null</code>
-     *
      */
     public XMLDataObject (FileObject fo, MultiFileLoader loader)
+    throws DataObjectExistsException {
+        this(fo, loader, true);
+    }
+
+    /**
+     * Constructs XMLDataObject without any registered cookies (for editor,
+     * open, etc.). Useful for subclasses.
+     *
+     * @param fo the primary file object, never <code>null</code>
+     * @param loader loader of this data object, never <code>null</code>
+     * @param registerEditor call with false to skip registrations of various
+     *   editor related cookies
+     * @since 7.10
+     */
+    protected XMLDataObject (FileObject fo, MultiFileLoader loader, boolean registerEditor)
     throws DataObjectExistsException {
         super (fo, loader);
         
-        fo.addFileChangeListener (FileUtil.weakFileChangeListener (getIP (), fo));
-
         status = STATUS_NOT;
 
+        if (registerEditor) {
+            registerEditor();
+        }
+    }
+
+    private void registerEditor() {
         // register provided cookies
         // EditorCookie must be for back compatability consulted with subclasses
         //
         // In new model subclasses should directly provide its CookieSet.Factory that
         // uses last prevails order instead of old CookieSet first prevails order.
         // It completely prevails over this factory :-)
-        
+
         CookieSet.Factory factory = new CookieSet.Factory() {
             public <T extends Node.Cookie> T createCookie(Class<T> klass) {
                 if (klass.isAssignableFrom(EditorCookie.class)
-                   || klass.isAssignableFrom(OpenCookie.class) 
-                   || klass.isAssignableFrom(CloseCookie.class) 
+                   || klass.isAssignableFrom(OpenCookie.class)
+                   || klass.isAssignableFrom(CloseCookie.class)
                    || klass.isAssignableFrom(PrintCookie.class) ) {
-                
+
                     if (editor == null) editor = createEditorCookie();  // the first pass
                     if (editor == null) return null;                    //??? gc unfriendly
 
@@ -202,15 +223,15 @@ public class XMLDataObject extends MultiDataObject {
                 }
             }
         };
-                
+
         CookieSet cookies = getCookieSet();
-        // EditorCookie.class must be synchronized with 
+        // EditorCookie.class must be synchronized with
         // XMLEditor.Env->findCloneableOpenSupport
         cookies.add(EditorCookie.class, factory);
         cookies.add(OpenCookie.class, factory);
         cookies.add(CloseCookie.class, factory);
         cookies.add(PrintCookie.class, factory);
-        
+
         // set info for this file
         //getIP ().resolveInfo ();        #16045
         cookies.assign( SaveAsCapable.class, new SaveAsCapable() {
@@ -248,11 +269,7 @@ public class XMLDataObject extends MultiDataObject {
     */
     @Override
     protected Node createNodeDelegate () {
-        XMLNode xn = new XMLNode (this);
-        // netbeans.core.nodes.description
-        xn.setShortDescription (NbBundle.getMessage ( 
-                                        XMLDataObject.class, "HINT_XMLDataObject")); // NOI18N
-        return xn;
+        return new XMLNode(this);
     }
 
 
@@ -326,8 +343,11 @@ public class XMLDataObject extends MultiDataObject {
         if (ERR.isLoggable(Level.FINE)) {
             ERR.fine("getCookie returns " + cake + " for " + this); // NOI18N
         }
-        
-        return cls.cast(cake);
+        if (cake instanceof Node.Cookie) {
+            assert cake == null || cls.isInstance(cake) : "Cannot return " + cake + " for " + cls + " from " + this;
+            return cls.cast(cake);
+        }
+        return null;
     }
 
     @Override
@@ -425,7 +445,7 @@ public class XMLDataObject extends MultiDataObject {
         }
         protected void notifyUnmodified () {
             super.notifyUnmodified ();
-            SaveCookie save = (SaveCookie) getDataObject ().getCookie (SaveCookie.class);
+            SaveCookie save = getDataObject().getCookie(SaveCookie.class);
             if (save != null) {
                 ((XMLDataObject) getDataObject ()).removeSaveCookie (save);
                 getDataObject ().setModified (false);
@@ -482,6 +502,12 @@ public class XMLDataObject extends MultiDataObject {
         firePropertyChange (PROP_DOCUMENT, null, null);
     }
 
+    @Override
+    void notifyFileChanged(FileEvent fe) {
+        super.notifyFileChanged(fe);
+        getIP().fileChanged(fe);
+    }
+
     /** 
      * @return one of STATUS_XXX constants representing PROP_DOCUMENT state. 
      */
@@ -588,12 +614,7 @@ public class XMLDataObject extends MultiDataObject {
     @Deprecated
     public static Document parse (URL url, ErrorHandler eh, boolean validate) throws IOException, SAXException {
         
-        DocumentBuilder builder = XMLDataObjectImpl.makeBuilder(validate);
-        builder.setErrorHandler(eh);
-        builder.setEntityResolver(getChainingEntityResolver());
-        
-        return builder.parse (new InputSource(url.toExternalForm()));
-
+        return XMLUtil.parse (new InputSource(url.toExternalForm()),validate, false, eh, getChainingEntityResolver());
     }
 
     /** Creates SAX parse that can be used to parse XML files.
@@ -884,7 +905,7 @@ public class XMLDataObject extends MultiDataObject {
     @Deprecated
     public static Info getRegisteredInfo(String publicId) {  //!!! to be replaced by lookup
         synchronized (infos) {
-            Info ret = (Info) infos.get(publicId);
+            Info ret = infos.get(publicId);
             return ret == null ? null : (Info)ret.clone ();
         }
     }
