@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -47,7 +50,6 @@ import org.netbeans.junit.RandomlyFails;
 import org.openide.ErrorManager;
 import org.openide.filesystems.FileObject;
 import org.openide.filesystems.FileUtil;
-import org.openide.filesystems.Repository;
 import org.openide.nodes.Node;
 import org.openide.util.RequestProcessor;
 
@@ -65,7 +67,7 @@ public class FolderChildrenGCRaceConditionTest extends LoggingTestCaseHid {
     	super.setUp();
         clearWorkDir();
 
-        FileObject[] arr = Repository.getDefault().getDefaultFileSystem().getRoot().getChildren();
+        FileObject[] arr = FileUtil.getConfigRoot().getChildren();
         for (int i = 0; i < arr.length; i++) {
             arr[i].delete();
         }
@@ -73,7 +75,7 @@ public class FolderChildrenGCRaceConditionTest extends LoggingTestCaseHid {
 
     @RandomlyFails // NB-Core-Build #1087
     public void testChildrenCanBeSetToNullIfGCKicksIn () throws Exception {
-        FileObject f = FileUtil.createData(Repository.getDefault().getDefaultFileSystem().getRoot(), "folder/node.txt");
+        FileObject f = FileUtil.createData(FileUtil.getConfigRoot(), "folder/node.txt");
         
         DataFolder df = DataFolder.findFolder(f.getParent());
         Node n = df.getNodeDelegate();
@@ -108,4 +110,4 @@ public class FolderChildrenGCRaceConditionTest extends LoggingTestCaseHid {
         assertEquals("Count is really one", 1, cnt);
     }
    
-}
\ No newline at end of file
+}
