@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -90,7 +93,7 @@ public abstract class DocumentLine extends Line {
     private transient DocumentListener docL;
 
     /** List of Line.Part which exist for this line*/
-    private List<Part> lineParts = new ArrayList<Part>(3);
+    private final List<Part> lineParts = Collections.synchronizedList(new ArrayList<Part>(3));
 
     /** Constructor.
     * @param obj context we belong to
@@ -309,7 +312,7 @@ public abstract class DocumentLine extends Line {
     private void readObject(ObjectInputStream ois) throws IOException, ClassNotFoundException {
         pos = (PositionRef) ois.readObject();
         setBreakpoint(ois.readBoolean());
-        lineParts = new ArrayList<Part>(3);
+        lineParts.clear();
     }
 
     /** Register line.
@@ -407,8 +410,10 @@ public abstract class DocumentLine extends Line {
         }
 
         // notify also all Line.Part attached to this Line
-        for (Part p : lineParts) {
-            p.attachDetachAnnotations(doc, closing);
+        synchronized (lineParts) {
+            for (Part p : lineParts) {
+                p.attachDetachAnnotations(doc, closing);
+            }
         }
     }
 
@@ -470,18 +475,20 @@ public abstract class DocumentLine extends Line {
     void notifyChange(DocumentEvent p0, DocumentLine.Set set, StyledDocument doc) {
         DocumentLine.Part part;
 
-        for (int i = 0; i < lineParts.size();) {
-            part = lineParts.get(i);
+        synchronized (lineParts) {
+            for (int i = 0; i < lineParts.size();) {
+                part = lineParts.get(i);
 
-            // notify Line.Part about the change
-            part.handleDocumentChange(p0);
+                // notify Line.Part about the change
+                part.handleDocumentChange(p0);
 
-            // if necessary move Line.Part to new Line
-            if (NbDocument.findLineNumber(doc, part.getOffset()) != part.getLine().getLineNumber()) {
-                DocumentLine line = (DocumentLine) set.getCurrent(NbDocument.findLineNumber(doc, part.getOffset()));
-                moveLinePart(part, line);
-            } else {
-                i++;
+                // if necessary move Line.Part to new Line
+                if (NbDocument.findLineNumber(doc, part.getOffset()) != part.getLine().getLineNumber()) {
+                    DocumentLine line = (DocumentLine) set.getCurrent(NbDocument.findLineNumber(doc, part.getOffset()));
+                    moveLinePart(part, line);
+                } else {
+                    i++;
+                }
             }
         }
 
@@ -537,8 +544,10 @@ public abstract class DocumentLine extends Line {
     void notifyMove() {
         updatePositionRef();
 
-        for (Part p : lineParts) {
-            p.firePropertyChange(Line.Part.PROP_LINE, null, null);
+        synchronized (lineParts) {
+            for (Part p : lineParts) {
+                p.firePropertyChange(Line.Part.PROP_LINE, null, null);
+            }
         }
     }
 
