@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -78,22 +81,30 @@ public final class NbDocument extends Object {
 
     /** Name of style attached to documents to mark a paragraph (line)
     * as a (debugger) breakpoint.
+    * @deprecated since 6.29. Use {@link Annotation} instead.
     */
+    @Deprecated
     public static final String BREAKPOINT_STYLE_NAME = "NbBreakpointStyle"; // NOI18N
 
     /** Name of style attached to documents to mark a paragraph (line)
     * as erroneous.
+    * @deprecated since 6.29. Use {@link Annotation} instead.
     */
+    @Deprecated
     public static final String ERROR_STYLE_NAME = "NbErrorStyle"; // NOI18N
 
     /** Name of style attached to documents to mark a paragraph (line)
     * as current (in a debugger).
+    * @deprecated since 6.29. Use {@link Annotation} instead.
     */
+    @Deprecated
     public static final String CURRENT_STYLE_NAME = "NbCurrentStyle"; // NOI18N
 
     /** Name of style attached to documents to unmark a paragraph (line)
     * as anything special.
+    * @deprecated since 6.29. Use {@link Annotation} instead.
     */
+    @Deprecated
     public static final String NORMAL_STYLE_NAME = "NbNormalStyle"; // NOI18N
     
     private NbDocument() {
