@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -47,7 +50,9 @@ import java.io.CharConversionException;
 import java.io.File;
 import java.io.IOException;
 import java.io.StringReader;
+import java.lang.ref.Reference;
 import java.lang.ref.WeakReference;
+import java.util.List;
 import javax.xml.XMLConstants;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.transform.stream.StreamSource;
@@ -67,6 +72,7 @@ import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
 import org.xml.sax.SAXParseException;
 import org.xml.sax.XMLReader;
+import org.xml.sax.helpers.DefaultHandler;
 
 public class XMLUtilTest extends NbTestCase {
     
@@ -190,25 +196,6 @@ public class XMLUtilTest extends NbTestCase {
             XMLUtil.validate(r, s);
             fail();
         } catch (SAXException x) {/*OK*/}
-        // #146081: xml:base attributes get inserted sometimes and can mess up validation.
-        xsd =
-                "<xsd:schema xmlns:xsd='http://www.w3.org/2001/XMLSchema' targetNamespace='some://where' xmlns='some://where' elementFormDefault='qualified'>\n" +
-                " <xsd:element name='root'>\n" +
-                "  <xsd:complexType>\n" +
-                "   <xsd:sequence>\n" +
-                "    <xsd:element name='hello' type='xsd:NMTOKEN'/>\n" +
-                "   </xsd:sequence>\n" +
-                "  </xsd:complexType>\n" +
-                " </xsd:element>\n" +
-                "</xsd:schema>\n";
-        s = f.newSchema(new StreamSource(new StringReader(xsd)));
-        File d = getWorkDir();
-        File main = new File(d, "main.xml");
-        File ent = new File(d, "ent.xml");
-        TestFileUtils.writeFile(main, "<!DOCTYPE root [<!ENTITY ent SYSTEM 'ent.xml'>]> <root xmlns='some://where'>&ent;</root>");
-        TestFileUtils.writeFile(ent, "<hello xmlns='some://where'>there</hello>");
-        r = XMLUtil.parse(new InputSource(main.toURI().toString()), false, true, null, null).getDocumentElement();
-        XMLUtil.validate(r, s);
     }
     
     public void testToAttributeValue() throws IOException {
@@ -460,4 +447,264 @@ public class XMLUtilTest extends NbTestCase {
         assertTrue("Expecting CDATASection node", child instanceof CDATASection);
         assertEquals("Wrong CDATASection content", cdataContent, ((CDATASection) child).getNodeValue());
     }
+
+    public void testEntityIncludes() throws Exception {
+        clearWorkDir();
+        // #146081: xml:base attributes get inserted sometimes and can mess up validation.
+        // #160806: problems writing: no system ID, xml:base inserted.
+        SchemaFactory f = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
+        String xsd =
+                "<xsd:schema xmlns:xsd='http://www.w3.org/2001/XMLSchema' targetNamespace='some://where' xmlns='some://where' elementFormDefault='qualified'>\n" +
+                " <xsd:element name='root'>\n" +
+                "  <xsd:complexType>\n" +
+                "   <xsd:sequence>\n" +
+                "    <xsd:element name='hello' type='xsd:NMTOKEN'/>\n" +
+                "   </xsd:sequence>\n" +
+                "  </xsd:complexType>\n" +
+                " </xsd:element>\n" +
+                "</xsd:schema>\n";
+        Schema s = f.newSchema(new StreamSource(new StringReader(xsd)));
+        File d = getWorkDir();
+        File main = new File(d, "main.xml");
+        File ent = new File(d, "ent.xml");
+        TestFileUtils.writeFile(main, "<!DOCTYPE root [<!ENTITY ent SYSTEM 'ent.xml'>]> <root xmlns='some://where'>&ent;</root>");
+        TestFileUtils.writeFile(ent, "<hello xmlns='some://where'>there</hello>");
+        Document doc = XMLUtil.parse(new InputSource(main.toURI().toString()), false, true, null, null);
+        XMLUtil.validate(doc.getDocumentElement(), s);
+        ByteArrayOutputStream baos = new ByteArrayOutputStream();
+        XMLUtil.write(doc, baos, "UTF-8");
+        String expanded =
+                "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
+                "<root xmlns=\"some://where\">\n" +
+                "    <hello>there</hello>\n" +
+                "</root>\n";
+        assertEquals(expanded, baos.toString("UTF-8"));
+        // XXX #160806 reported a problem with "xml:base" being consider a no-NS attr; not yet caught by test
+        // Try again with no xmlns specified in entity; should inherit from main.xml:
+        TestFileUtils.writeFile(ent, "<hello>there</hello>");
+        doc = XMLUtil.parse(new InputSource(main.toURI().toString()), false, true, null, null);
+        XMLUtil.validate(doc.getDocumentElement(), s);
+        baos = new ByteArrayOutputStream();
+        XMLUtil.write(doc, baos, "UTF-8");
+        assertEquals(expanded, baos.toString("UTF-8"));
+    }
+
+    public void testHandlerLeak() throws Exception {
+        ErrorHandler handler = new DefaultHandler();
+        EntityResolver resolver = new DefaultHandler();
+        Reference<?> handlerRef = new WeakReference<Object>(handler);
+        Reference<?> resolverRef = new WeakReference<Object>(resolver);
+        XMLUtil.parse(new InputSource(new StringReader("<hello/>")), false, false, handler, resolver);
+        handler = null;
+        resolver = null;
+        assertGC("can collect handler", handlerRef);
+        assertGC("can collect resolver", resolverRef);
+    }
+
+    public void testAppendChildElement() throws Exception {
+        Document doc = XMLUtil.parse(new InputSource(new StringReader("<root></root>")), false, true, null, null);
+        Element parent = doc.createElementNS("unittest", "parent");
+        Element newElement1 = doc.createElementNS("unittest", "new_element1");
+        Element newElement2 = doc.createElementNS("unittest", "new_element2");
+        Element newElement3 = doc.createElementNS("unittest", "new_element3");
+
+        // append to the default root node (no match in order)
+        XMLUtil.appendChildElement(parent, newElement1, new String[] { "new_element2", "new_element1" });
+        NodeList children = parent.getChildNodes();
+        assertEquals(1, children.getLength());
+
+        // append after the child we just appended
+        XMLUtil.appendChildElement(parent, newElement2, new String[] { "new_element2", "new_element1" });
+
+        children = parent.getChildNodes();
+        assertEquals(2, children.getLength());
+        Node firstChild = parent.getChildNodes().item(0);
+        Node secondChild = parent.getChildNodes().item(1);
+        assertEquals("new_element2", firstChild.getNodeName());
+        assertEquals("new_element1", secondChild.getNodeName());
+
+        // failures
+
+        try {
+            // new child is not in the order list
+            XMLUtil.appendChildElement(parent, newElement3, new String[] { "new_element2", "new_element1"});
+            fail("Expecting IAE");
+        } catch (IllegalArgumentException e) {
+            assertEquals("new child element 'new_element3' not specified in order [new_element2, new_element1]", e.getMessage());
+        }
+        try {
+            // existing child not in the order list
+            XMLUtil.appendChildElement(parent, newElement3, new String[] { "new_element3"});
+            fail("Expecting IAE");
+        } catch (IllegalArgumentException e) {
+            assertEquals("Existing child element 'new_element2' not specified in order [new_element3]", e.getMessage());
+        }
+    }
+
+    public void testFindSubElements() throws Exception {
+        Document doc = XMLUtil.parse(new InputSource(new StringReader("<root> <child1></child1><child2/><!-- comment --><child3/></root>")),
+                false, true, null, null);
+
+        Element parent = doc.getDocumentElement();
+        assertEquals(5, parent.getChildNodes().getLength());
+        List<Element> subElements = XMLUtil.findSubElements(parent);
+        assertEquals(3, subElements.size());
+        Element firstChild = subElements.get(0);
+        Element secondChild = subElements.get(1);
+        Element thirdChild = subElements.get(2);
+
+        assertEquals("child1", firstChild.getNodeName());
+        assertEquals("child2", secondChild.getNodeName());
+        assertEquals("child3", thirdChild.getNodeName());
+
+        Document failureDoc = XMLUtil.parse(new InputSource(new StringReader("<root>Non whitespace<!-- comment --></root>")),
+            false, true, null, null);
+        Element failedParent = failureDoc.getDocumentElement();
+        try {
+            XMLUtil.findSubElements(failedParent);
+            fail("expected IAE");
+        } catch (IllegalArgumentException e) { }
+    }
+
+    public void testFindElement() throws Exception {
+        String xmlDoc = "<root> " +
+                        " <h:table xmlns:h=\"http://www.w3.org/TR/html4/\">" +
+                        " </h:table>" +
+                        " <f:table xmlns:f=\"http://www.w3schools.com/furniture\">" +
+                        " </f:table>" +
+                        " <dup/><dup/>" +
+                        " <h:form xmlns:h=\"http://www.w3.org/TR/html4/\">" +
+                        " </h:form>" +
+                        "</root>";
+        
+        Document doc = XMLUtil.parse(new InputSource(new StringReader(xmlDoc)), false, true, null, null);
+        Element parent = doc.getDocumentElement();
+
+        Element doesNotExist1 = XMLUtil.findElement(parent, "doesNotExist", "h");
+        assertNull(doesNotExist1);
+
+        Element doesNotExist2 = XMLUtil.findElement(parent, "doesNotExist", null);
+        assertNull(doesNotExist2);
+
+        Element noTable2 = XMLUtil.findElement(parent, "table", "h");
+        assertNull(noTable2);
+
+        Element noTable3 = XMLUtil.findElement(parent, "table", "f");
+        assertNull(noTable3);
+
+        Element table1 = XMLUtil.findElement(parent, "table", "http://www.w3.org/TR/html4/");
+        assertNotNull(table1);
+
+        Element table2 = XMLUtil.findElement(parent, "table", "http://www.w3schools.com/furniture");
+        assertNotNull(table2);
+
+        Element form1 = XMLUtil.findElement(parent, "form", "http://www.w3.org/TR/html4/");
+        assertNotNull(form1);
+
+        Element form2 = XMLUtil.findElement(parent, "form", null);
+        assertNotNull(form2);
+
+        assertEquals(form1, form2);
+
+        try {
+            XMLUtil.findElement(parent, "dup", null);
+            fail("Expected IAE");
+        } catch (IllegalArgumentException e) { }
+
+        try {
+            XMLUtil.findElement(parent, "table", null);
+            fail("Expected IAE");
+        } catch (IllegalArgumentException e) { }
+    }
+
+    public void testFindText() throws Exception {
+        Document doc = XMLUtil.parse(new InputSource(new StringReader("<root>Text To Find<child></child></root>")),
+                false, true, null, null);
+
+        Element parent = doc.getDocumentElement();
+        String foundText = XMLUtil.findText(parent);
+
+        assertEquals("Text To Find", foundText);
+
+        String notFoundText = XMLUtil.findText(parent.getFirstChild());
+
+        assertNull(notFoundText);
+    }
+
+    public void testTranslateXML() throws Exception {
+        // don't add any whitespace to the first 3 nodes
+        String xmlDoc = "<root><table bgcolor='red'><tr>" +
+                        "   <td>Apples</td>" +
+                        "   <td>Bananas</td>" +
+                        "  </tr>" +
+                        " </table>" +
+                        " <!-- comment -->" +
+                        "<![CDATA[free form data]]>" +
+                        "</root>";
+
+        Document doc = XMLUtil.parse(new InputSource(new StringReader(xmlDoc)), false, true, null, null);
+        Element parent = doc.getDocumentElement();
+
+        Element translated = XMLUtil.translateXML((Element) parent.getFirstChild(), "http://www.w3.org/TR/html4/");
+
+        ByteArrayOutputStream baos = new ByteArrayOutputStream();
+        XMLUtil.write(doc, baos, "UTF-8");
+
+        Document destDoc = XMLUtil.parse(new InputSource(new StringReader("<root/>")), false, true, null, null);
+
+        destDoc.importNode(translated, true);
+        baos = new ByteArrayOutputStream();
+        XMLUtil.write(destDoc, baos, "UTF-8");
+
+        assertEquals("http://www.w3.org/TR/html4/", translated.getNamespaceURI());
+        assertEquals(1, translated.getAttributes().getLength());
+
+        assertEquals("http://www.w3.org/TR/html4/", translated.getFirstChild().getNamespaceURI());
+        assertEquals(0, translated.getFirstChild().getAttributes().getLength());
+    }
+
+    public void testCopyDocument() throws Exception {
+        String srcXml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
+                        "<root>" +
+                        "   <table bgcolor='red'>" +
+                        "    <tr>" +
+                        "           <td>Apples</td>" +
+                        "           <td>Bananas</td>" +
+                        "       </tr>" +
+                        "   </table>" +
+                        " <!-- comment -->" +
+                        "</root>";
+
+        String resultXml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
+                        "<root>\n" +
+                        "    <table xmlns=\"http://www.w3.org/TR/html4/\" bgcolor=\"red\">\n" +
+                        "        <tr>\n" +
+                        "            <td>Apples</td>\n" +
+                        "            <td>Bananas</td>\n" +
+                        "        </tr>\n" +
+                        "    </table>\n" +
+                        "    <!-- comment -->\n" +
+                        "</root>\n";
+
+        Document srcDoc = XMLUtil.parse(new InputSource(new StringReader(srcXml)), false, true, null, null);
+        Element srcRoot = srcDoc.getDocumentElement();
+
+        Document dstDoc = XMLUtil.parse(new InputSource(new StringReader("<root/>")), false, true, null, null);
+
+        Element dstRoot = dstDoc.getDocumentElement();
+        XMLUtil.copyDocument(srcRoot, dstRoot, "http://www.w3.org/TR/html4/");
+
+        ByteArrayOutputStream baos = new ByteArrayOutputStream();
+        XMLUtil.write(dstDoc, baos, "UTF-8");
+        assertEquals(resultXml, baos.toString());
+
+        assertEquals(1, dstDoc.getChildNodes().getLength());
+        Element root = dstDoc.getDocumentElement();
+        Element tableNode = (Element) root.getFirstChild().getNextSibling();
+
+        assertEquals("table", tableNode.getNodeName());
+        assertEquals("http://www.w3.org/TR/html4/", tableNode.getNamespaceURI());
+        assertEquals(1, tableNode.getAttributes().getLength());
+    }
+
 }
