@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -53,6 +56,7 @@ import java.lang.ref.WeakReference;
 import java.net.URL;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -86,7 +90,6 @@ final class DummyWindowManager extends WindowManager {
     private final Map<String,Workspace> workspaces;
     private transient Frame mw;
     private transient PropertyChangeSupport pcs;
-    private transient R r;
 
     public DummyWindowManager() {
         workspaces = new TreeMap<String,Workspace>();
@@ -108,21 +111,11 @@ final class DummyWindowManager extends WindowManager {
     }
 
     protected TopComponent.Registry componentRegistry() {
-        TopComponent.Registry reg = super.componentRegistry();
-
-        if (reg != null) {
-            return reg;
-        } else {
-            return registry();
-        }
+        return new R();
     }
 
-    synchronized R registry() {
-        if (r == null) {
-            r = new R();
-        }
-
-        return r;
+    private R registry() {
+        return (R) getRegistry();
     }
 
     protected WindowManager.Component createTopComponentManager(TopComponent c) {
@@ -635,7 +628,18 @@ final class DummyWindowManager extends WindowManager {
         private PropertyChangeSupport pcs;
 
         public R() {
-            opened = new HashSet<TopComponent>();
+            opened = new HashSet<TopComponent>() {
+                @Override
+                public Iterator<TopComponent> iterator() {
+                    HashSet<TopComponent> copy = new HashSet<TopComponent>();
+                    Iterator<TopComponent> it = super.iterator();
+                    while (it.hasNext()) {
+                        TopComponent topComponent = it.next();
+                        copy.add(topComponent);
+                    }
+                    return copy.iterator();
+                }
+            };
             nodes = new Node[0];
         }
 
@@ -676,7 +680,7 @@ final class DummyWindowManager extends WindowManager {
         }
 
         public synchronized Set<TopComponent> getOpened() {
-            return new HashSet<TopComponent>(opened);
+            return Collections.unmodifiableSet(opened);
         }
 
         synchronized void setActive(TopComponent tc) {
