@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -44,6 +47,8 @@ package org.netbeans.modules.options;
 
 import java.awt.Component;
 import java.awt.Dialog;
+import java.awt.Point;
+import java.awt.Rectangle;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.WindowEvent;
@@ -57,6 +62,8 @@ import javax.swing.JButton;
 import javax.swing.JLabel;
 import javax.swing.SwingUtilities;
 import org.netbeans.api.options.OptionsDisplayer;
+import org.netbeans.modules.options.classic.OptionsAction;
+import org.netbeans.modules.options.export.OptionsChooserPanel;
 import org.netbeans.spi.options.OptionsPanelController;
 
 import org.openide.DialogDescriptor;
@@ -69,13 +76,13 @@ import org.openide.filesystems.FileChangeListener;
 import org.openide.filesystems.FileEvent;
 import org.openide.filesystems.FileObject;
 import org.openide.filesystems.FileRenameEvent;
-import org.openide.filesystems.FileSystem;
-import org.openide.filesystems.Repository;
+import org.openide.filesystems.FileStateInvalidException;
+import org.openide.filesystems.FileUtil;
 import org.openide.util.Exceptions;
-import org.openide.util.Lookup;
 import org.openide.util.LookupEvent;
 import org.openide.util.LookupListener;
 import org.openide.util.NbBundle;
+import org.openide.util.NbPreferences;
 import org.openide.util.RequestProcessor;
 import org.openide.util.Utilities;
 import org.openide.util.actions.CallableSystemAction;
@@ -91,13 +98,23 @@ public class OptionsDisplayerImpl {
     private static Logger log = Logger.getLogger(OptionsDisplayerImpl.class.getName ());    
     private boolean modal;
     static final LookupListener lookupListener = new LookupListenerImpl();
+    /** OK button. */
+    private JButton bOK;
     /** Advanced Options button. */
     private JButton bClassic;
+    /** Export Options button */
+    private JButton btnExport;
+    /** Import Options button */
+    private JButton btnImport;
     
     public OptionsDisplayerImpl (boolean modal) {
         this.modal = modal;
-        // 91106 - listen to default FS changes to update Advanced Options button
-        Repository.getDefault().getDefaultFileSystem().addFileChangeListener(new AdvancedOptionsListener());
+        try {
+            // 91106 - listen to default FS changes to update Advanced Options, Export and Import buttons
+            FileUtil.getConfigRoot().getFileSystem().addFileChangeListener(new DefaultFSListener());
+        } catch (FileStateInvalidException ex) {
+            Exceptions.printStackTrace(ex);
+        }
     }
     
     public boolean isOpen() {
@@ -112,7 +129,7 @@ public class OptionsDisplayerImpl {
     static void selectCategory(String path) {
         DialogDescriptor descriptor = null;
         synchronized (lookupListener) {
-            descriptor = (DialogDescriptor) descriptorRef.get();
+            descriptor = descriptorRef.get();
         }
         if (descriptor != null) {
             OptionsPanel optionsPanel = (OptionsPanel) descriptor.getMessage();
@@ -134,25 +151,29 @@ public class OptionsDisplayerImpl {
         
         DialogDescriptor descriptor = null;
         synchronized(lookupListener) {
-            descriptor = (DialogDescriptor)descriptorRef.get ();
+            descriptor = descriptorRef.get ();
         }
 
         OptionsPanel optionsPanel = null;
         if (descriptor == null) {
             optionsPanel = categoryID == null ? new OptionsPanel () : new OptionsPanel(categoryID);            
-            JButton bOK = (JButton) loc(new JButton(), "CTL_OK");//NOI18N
+            bOK = (JButton) loc(new JButton(), "CTL_OK");//NOI18N
             bOK.getAccessibleContext().setAccessibleDescription(loc("ACS_OKButton"));//NOI18N
             bClassic = (JButton) loc(new JButton(), "CTL_Classic");//NOI18N
             bClassic.getAccessibleContext().setAccessibleDescription(loc("ACS_ClassicButton"));//NOI18N
-            setVisibleAdvancedOptionsButton();
+            btnExport = (JButton) loc(new JButton(), "CTL_Export");//NOI18N
+            btnExport.getAccessibleContext().setAccessibleDescription(loc("ACS_Export"));//NOI18N
+            btnImport = (JButton) loc(new JButton(), "CTL_Import");//NOI18N
+            btnImport.getAccessibleContext().setAccessibleDescription(loc("ACS_Import"));//NOI18N
+            updateButtons();
             boolean isMac = Utilities.isMac();
             Object[] options = new Object[2];            
             options[0] = isMac ? DialogDescriptor.CANCEL_OPTION : bOK;
             options[1] = isMac ? bOK : DialogDescriptor.CANCEL_OPTION;
             descriptor = new DialogDescriptor(optionsPanel,title,modal,options,DialogDescriptor.OK_OPTION,DialogDescriptor.DEFAULT_ALIGN, null, null, false);
-            descriptor.setAdditionalOptions(new Object[] {bClassic});
+            descriptor.setAdditionalOptions(new Object[] {bClassic, btnExport, btnImport});
             descriptor.setHelpCtx(optionsPanel.getHelpCtx());
-            OptionsPanelListener listener = new OptionsPanelListener(descriptor, optionsPanel, bOK, bClassic);
+            OptionsPanelListener listener = new OptionsPanelListener(descriptor, optionsPanel, bOK, bClassic, btnExport, btnImport);
             descriptor.setButtonListener(listener);
             optionsPanel.addPropertyChangeListener(listener);
             synchronized(lookupListener) {
@@ -170,15 +191,26 @@ public class OptionsDisplayerImpl {
         dialog = DialogDisplayer.getDefault ().createDialog (descriptor);
         optionsPanel.initCurrentCategory(categoryID, subpath);
         dialog.addWindowListener (new MyWindowListener (optionsPanel));
+        Point userLocation = getUserLocation();
+        if (userLocation != null) {
+            dialog.setLocation(userLocation);
+        }
         dialog.setVisible (true);     
     }
 
     /** Set visibility of Advanced Options button (AKA classic) according to
      * existence of advanced options. */
-    private void setVisibleAdvancedOptionsButton() {
+    private void updateButtons() {
         if(bClassic != null) {
             bClassic.setVisible(advancedOptionsNotEmpty());
         }
+        boolean optionsExportNotEmpty = optionsExportNotEmpty();
+        if (btnExport != null) {
+            btnExport.setVisible(optionsExportNotEmpty);
+        }
+        if (btnImport != null) {
+            btnImport.setVisible(optionsExportNotEmpty);
+        }
     }
 
     /** Returns true if some non hidden advanced options are registered
@@ -186,8 +218,7 @@ public class OptionsDisplayerImpl {
      * @return true if exists some advanced options, false otherwise
      */
     private boolean advancedOptionsNotEmpty() {
-        FileSystem defaultFS = Repository.getDefault().getDefaultFileSystem();
-        FileObject servicesFO = defaultFS.findResource("UI/Services");  //NOI18N
+        FileObject servicesFO = FileUtil.getConfigFile("UI/Services");  //NOI18N
         if(servicesFO != null) {
             FileObject[] advancedOptions = servicesFO.getChildren();
             for (FileObject advancedOption : advancedOptions) {
@@ -200,21 +231,59 @@ public class OptionsDisplayerImpl {
         return false;
     }
 
+    /** Returns true if some non hidden files are registered under OptionsExport
+     * folder.
+     * @return true if something is registered under OptionsExport, false otherwise
+     */
+    private boolean optionsExportNotEmpty() {
+        FileObject optionsExportFO = FileUtil.getConfigFile("OptionsExport");  //NOI18N
+        if(optionsExportFO != null) {
+            FileObject[] categories = optionsExportFO.getChildren();
+            for (FileObject category : categories) {
+                Object hiddenCategory = category.getAttribute("hidden");  //NOI18N
+                if (hiddenCategory != null && (Boolean)hiddenCategory) {
+                    // skip hidden category folder
+                    continue;
+                }
+                FileObject[] items = category.getChildren();
+                for (FileObject item : items) {
+                    Object hiddenItem = item.getAttribute("hidden");  //NOI18N
+                    if(hiddenItem == null || !(Boolean)hiddenItem) {
+                        return true;
+                    }
+                }
+            }
+        }
+        return false;
+    }
+
+    private Point getUserLocation() {
+        final Rectangle screenBounds = Utilities.getUsableScreenBounds();
+        int x = NbPreferences.forModule(OptionsDisplayerImpl.class).getInt("OptionsX", Integer.MAX_VALUE);//NOI18N
+        int y = NbPreferences.forModule(OptionsDisplayerImpl.class).getInt("OptionsY", Integer.MAX_VALUE);//NOI18N
+        if (x > screenBounds.getWidth() || y > screenBounds.getHeight()) {
+            return null;
+        } else {
+            return new Point(x, y);
+        }
+    }
+
     private static String loc (String key) {
         return NbBundle.getMessage (OptionsDisplayerImpl.class, key);
     }
     
     private static Component loc (Component c, String key) {
-        if (c instanceof AbstractButton)
+        if (c instanceof AbstractButton) {
             Mnemonics.setLocalizedText (
                 (AbstractButton) c, 
                 loc (key)
             );
-        else
+        } else {
             Mnemonics.setLocalizedText (
                 (JLabel) c, 
                 loc (key)
             );
+        }
         return c;
     }
     
@@ -224,18 +293,24 @@ public class OptionsDisplayerImpl {
         private OptionsPanel        optionsPanel;
         private JButton             bOK;
         private JButton             bClassic;
+        private JButton             btnExport;
+        private JButton             btnImport;
         
         
         OptionsPanelListener (
             DialogDescriptor    descriptor, 
             OptionsPanel        optionsPanel,
             JButton             bOK,
-            JButton             bClassic
+            JButton             bClassic,
+            JButton             btnExport,
+            JButton             btnImport
         ) {
             this.descriptor = descriptor;
             this.optionsPanel = optionsPanel;
             this.bOK = bOK;
             this.bClassic = bClassic;
+            this.btnExport = btnExport;
+            this.btnImport = btnImport;
         }
         
         public void propertyChange (PropertyChangeEvent ev) {
@@ -248,7 +323,9 @@ public class OptionsDisplayerImpl {
         
         @SuppressWarnings("unchecked")
         public void actionPerformed (ActionEvent e) {
-            if (!isOpen()) return; //WORKARROUND for some bug in NbPresenter
+            if (!isOpen()) { //WORKARROUND for some bug in NbPresenter
+                return;
+            }
                 // listener is called twice ...
             if (e.getSource () == bOK) {
                 log.fine("Options Dialog - Ok pressed."); //NOI18N
@@ -264,6 +341,7 @@ public class OptionsDisplayerImpl {
                 Dialog d = dialog;
                 dialog = null;
                 optionsPanel.cancel ();
+                bOK.setEnabled(true);
                 d.dispose ();                
             } else
             if (e.getSource () == bClassic) {
@@ -294,9 +372,7 @@ public class OptionsDisplayerImpl {
                     optionsPanel.cancel ();
                 }
                 try {
-                    ClassLoader cl = (ClassLoader) Lookup.getDefault ().lookup (ClassLoader.class);
-                    Class<CallableSystemAction> clz = (Class<CallableSystemAction>)cl.loadClass("org.netbeans.core.actions.OptionsAction");
-                    CallableSystemAction a = SystemAction.findObject(clz, true);
+                    CallableSystemAction a = SystemAction.get(OptionsAction.class);
                     a.putValue ("additionalActionName", loc ("CTL_Modern"));
                     a.putValue ("optionsDialogTitle", loc ("CTL_Classic_Title"));
                     a.putValue ("additionalActionListener", new OpenOptionsListener ()
@@ -306,13 +382,18 @@ public class OptionsDisplayerImpl {
                     Exceptions.printStackTrace(ex);
                 }
             } // classic
+            else if (e.getSource() == btnExport) {
+                OptionsChooserPanel.showExportDialog();
+            } else if (e.getSource() == btnImport) {
+                OptionsChooserPanel.showImportDialog();
+            }
         }
     }
     
     private class MyWindowListener implements WindowListener {        
         private OptionsPanel optionsPanel;
         private Dialog originalDialog;
-        
+
                 
         MyWindowListener (OptionsPanel optionsPanel) {
             this.optionsPanel = optionsPanel;
@@ -320,9 +401,12 @@ public class OptionsDisplayerImpl {
         }
         
         public void windowClosing (WindowEvent e) {
-            if (dialog == null) return;
+            if (dialog == null) {
+                return;
+            }
             log.fine("Options Dialog - windowClosing "); //NOI18N
             optionsPanel.cancel ();
+            bOK.setEnabled(true);
             if (this.originalDialog == dialog) {
                 dialog = null;            
             }
@@ -330,6 +414,9 @@ public class OptionsDisplayerImpl {
 
         public void windowClosed(WindowEvent e) {
             optionsPanel.storeUserSize();
+            // store location of dialog
+            NbPreferences.forModule(OptionsDisplayerImpl.class).putInt("OptionsX", originalDialog.getX());//NOI18N
+            NbPreferences.forModule(OptionsDisplayerImpl.class).putInt("OptionsY", originalDialog.getY());//NOI18N
             if (optionsPanel.needsReinit()) {
                 synchronized (lookupListener) {
                     descriptorRef = new WeakReference<DialogDescriptor>(null);
@@ -367,36 +454,41 @@ public class OptionsDisplayerImpl {
         public void resultChanged(LookupEvent ev) {
             synchronized (lookupListener) {
                 descriptorRef = new WeakReference<DialogDescriptor>(null);
+                // #156947 - close dialog when categories change
+                if (dialog != null) {
+                    dialog.setVisible(false);
+                    dialog = null;
+                }
             }
         }
         
     }
 
-    /** 91106 - used to listen to default FS changes to update Advanced Options button. */
-    private class AdvancedOptionsListener implements FileChangeListener {
+    /** 91106 - used to listen to default FS changes to update Advanced Options, Export and Import buttons. */
+    private class DefaultFSListener implements FileChangeListener {
 
         public void fileRenamed(FileRenameEvent fe) {
-            setVisibleAdvancedOptionsButton();
+            updateButtons();
         }
 
         public void fileChanged(FileEvent fe) {
-            setVisibleAdvancedOptionsButton();
+            updateButtons();
         }
 
         public void fileFolderCreated(FileEvent fe) {
-            setVisibleAdvancedOptionsButton();
+            updateButtons();
         }
 
         public void fileDataCreated(FileEvent fe) {
-            setVisibleAdvancedOptionsButton();
+            updateButtons();
         }
 
         public void fileDeleted(FileEvent fe) {
-            setVisibleAdvancedOptionsButton();
+            updateButtons();
         }
 
         public void fileAttributeChanged(FileAttributeEvent fe) {
-            setVisibleAdvancedOptionsButton();
+            updateButtons();
         }
     };
 }
