@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -42,31 +45,52 @@ import java.awt.Color;
 import java.awt.Component;
 import java.awt.event.KeyAdapter;
 import java.awt.event.KeyEvent;
+import java.beans.PropertyChangeEvent;
+import java.beans.PropertyChangeListener;
 import java.util.Arrays;
 import java.util.EventObject;
 import java.util.Set;
 import javax.swing.DefaultCellEditor;
+import javax.swing.FocusManager;
 import javax.swing.JComponent;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
 import javax.swing.JTable;
 import javax.swing.JTextField;
+import javax.swing.SwingUtilities;
 import javax.swing.border.EmptyBorder;
 import javax.swing.border.LineBorder;
+
 import org.netbeans.core.options.keymap.api.ShortcutAction;
 import org.openide.DialogDescriptor;
 import org.openide.DialogDisplayer;
 import org.openide.util.NbBundle;
 
+
 /**
  * Cell Editor for shortcuts column
  * @author Max Sauer
  */
 public class ButtonCellEditor extends DefaultCellEditor {
 
-    private Object action;
-    private KeymapViewModel model;
-    private String orig;
+    private Object              action;
+    private KeymapViewModel     model;
+    private String              orig;
+    private FocusManager        focusManager;
+
+    private PropertyChangeListener
+                                focusListener = new PropertyChangeListener () {
+        @Override
+        public void propertyChange (PropertyChangeEvent evt) {
+            Component c = focusManager.getFocusOwner ();
+            if (c != null &&
+                !SwingUtilities.isDescendingFrom (c, cell)
+            ) {
+                cancelCellEditing();
+                return;
+            }
+        }
+    };
 
     private KeyAdapter escapeAdapter = new KeyAdapter() {
 
@@ -82,7 +106,8 @@ public class ButtonCellEditor extends DefaultCellEditor {
 
     private static ShortcutCellPanel cell = new ShortcutCellPanel();
 
-    public ButtonCellEditor(KeymapViewModel model) {
+
+    public ButtonCellEditor (KeymapViewModel model) {
         super(new ShortcutTextField());
         this.model = model;
     }
@@ -140,6 +165,10 @@ public class ButtonCellEditor extends DefaultCellEditor {
         fireEditingStopped();
         setBorderEmpty();
         model.update();
+        if (focusManager != null) {
+            focusManager.removePropertyChangeListener (focusListener);
+            focusManager = null;
+        }
         return true;
     }
 
@@ -148,6 +177,10 @@ public class ButtonCellEditor extends DefaultCellEditor {
         cell.getTextField().setText(orig);
         fireEditingCanceled();
         setBorderEmpty();
+        if (focusManager != null) {
+            focusManager.removePropertyChangeListener (focusListener);
+            focusManager = null;
+        }
     }
 
     @Override
@@ -163,6 +196,8 @@ public class ButtonCellEditor extends DefaultCellEditor {
         if(!Arrays.asList(textField.getKeyListeners()).contains(escapeAdapter)) {
             textField.addKeyListener(escapeAdapter);
         }
+        focusManager = FocusManager.getCurrentManager ();
+        focusManager.addPropertyChangeListener (focusListener);
         return cell;
     }
 
@@ -203,5 +238,4 @@ public class ButtonCellEditor extends DefaultCellEditor {
     private void setBorderEmpty() {
         ((JComponent) getComponent()).setBorder(new EmptyBorder(0, 0, 0, 0));
     }
-
 }
